@@ -20,4 +20,15 @@
 
  */
 
+#ifndef _ADDPART_H
+#define _ADDPART_H
+/*@
+  @ requires \valid(disk);
+  @ requires valid_disk(disk);
+  @ requires valid_list_part(list_part);
+  @ requires \valid(current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @ requires separation: \separated(disk, list_part, current_cmd);
+  @*/
 list_part_t *add_partition_cli(disk_t *disk, list_part_t *list_part, char **current_cmd);
+#endif
