@@ -27,18 +27,30 @@
 #include "common.h"
 #include "autoset.h"
 
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_GPT)
 extern const arch_fnct_t arch_gpt;
+#endif
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_HUMAX)
 extern const arch_fnct_t arch_humax;
+#endif
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_MAC)
 extern const arch_fnct_t arch_mac;
+#endif
 
 void autoset_unit(disk_t *disk)
 {
   if(disk==NULL)
     return ;
   if(
-      disk->arch==&arch_mac || 
-      disk->arch==&arch_gpt || 
-      disk->arch==&arch_humax || 
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_GPT)
+      disk->arch==&arch_gpt ||
+#endif
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_HUMAX)
+      disk->arch==&arch_humax ||
+#endif
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_MAC)
+      disk->arch==&arch_mac ||
+#endif
       (disk->geom.heads_per_cylinder==1 && disk->geom.sectors_per_head==1))
     disk->unit=UNIT_SECTOR;
   else
