@@ -34,8 +34,31 @@
 #include "fnctdsk.h"
 #include "log.h"
 
-static void set_BeFS_info(const struct disk_super_block *beos_block, partition_t *partition);
-static int test_BeFS(disk_t *disk_car, const struct disk_super_block*beos_block, const partition_t *partition, const int dump_ind);
+static void set_BeFS_info(const struct disk_super_block *beos_block, partition_t *partition)
+{
+  partition->upart_type=UP_BEOS;
+  partition->blocksize= 1 << le32(beos_block->block_shift);
+  partition->info[0]='\0';
+  snprintf(partition->info, sizeof(partition->info), "BeFS blocksize=%u", partition->blocksize);
+  set_part_name(partition,beos_block->name,B_OS_NAME_LENGTH);
+}
+
+static int test_BeFS(const disk_t *disk_car, const struct disk_super_block*beos_block, const partition_t *partition, const int dump_ind)
+{
+  if(beos_block->magic1!=le32(SUPER_BLOCK_MAGIC1) &&
+      beos_block->magic2!=le32(SUPER_BLOCK_MAGIC2) &&
+      beos_block->magic3!=le32(SUPER_BLOCK_MAGIC3))
+    return 1;
+  if(partition==NULL)
+    return 0;
+  if(dump_ind!=0)
+  {
+    log_info("\nBeFS magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
+    dump_log(beos_block,DEFAULT_SECTOR_SIZE);
+  }
+  return 0;
+}
+
 
 int check_BeFS(disk_t *disk_car,partition_t *partition)
 {
@@ -56,37 +79,14 @@ int check_BeFS(disk_t *disk_car,partition_t *partition)
   return 0;
 }
 
-int recover_BeFS(disk_t *disk_car, const struct disk_super_block *beos_block, partition_t *partition, const int dump_ind)
+int recover_BeFS(const disk_t *disk_car, const struct disk_super_block *beos_block, partition_t *partition, const int dump_ind)
 {
   if(test_BeFS(disk_car,beos_block,partition,dump_ind)!=0)
     return 1;
   set_BeFS_info(beos_block, partition);
   partition->part_size=le64(beos_block->num_blocks) << le32(beos_block->block_shift);
   partition->part_type_i386=(unsigned char)P_BEOS;
+  partition->part_type_mac=PMAC_BEOS;
+  partition->part_type_gpt=GPT_ENT_TYPE_BEOS_BFS;
   return 0;
 }
-
-static int test_BeFS(disk_t *disk_car, const struct disk_super_block*beos_block, const partition_t *partition, const int dump_ind)
-{
-  if(beos_block->magic1!=le32(SUPER_BLOCK_MAGIC1) &&
-      beos_block->magic2!=le32(SUPER_BLOCK_MAGIC2) &&
-      beos_block->magic3!=le32(SUPER_BLOCK_MAGIC3))
-    return 1;
-  if(partition==NULL)
-    return 0;
-  if(dump_ind!=0)
-  {
-    log_info("\nBeFS magic value at %u/%u/%u\n", offset2cylinder(disk_car,partition->part_offset),offset2head(disk_car,partition->part_offset),offset2sector(disk_car,partition->part_offset));
-    dump_log(beos_block,DEFAULT_SECTOR_SIZE);
-  }
-  return 0;
-}
-
-static void set_BeFS_info(const struct disk_super_block *beos_block, partition_t *partition)
-{
-  partition->upart_type=UP_BEOS;
-  partition->blocksize= 1 << le32(beos_block->block_shift);
-  partition->info[0]='\0';
-  snprintf(partition->info, sizeof(partition->info), "BeFS blocksize=%u", partition->blocksize);
-  set_part_name(partition,beos_block->name,B_OS_NAME_LENGTH);
-}
