@@ -19,12 +19,24 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _CHGTYPE_H
+#define _CHGTYPE_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires \valid_read(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires \valid(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid(current_cmd);
+  @ requires \separated(disk_car, partition, current_cmd);
+  @ decreases 0;
+  @*/
 void change_part_type_cli(const disk_t *disk_car,partition_t *partition, char **current_cmd);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
