@@ -3,17 +3,17 @@
     File: common.h
 
     Copyright (C) 1998-2007 Christophe GRENIER <grenier@cgsecurity.org>
-  
+
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-  
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-  
+
     You should have received a copy of the GNU General Public License along
     with this program; if not, write the Free Software Foundation, Inc., 51
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
@@ -24,8 +24,13 @@
 #ifdef __cplusplus
 extern "C" {
 #endif
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_NCURSES
+#endif
+#if  defined(DISABLED_FOR_FRAMAC) && defined(HAVE_STRING_H)
+#include <string.h>
+#endif
 
-typedef struct efi_guid_s efi_guid_t;
 struct efi_guid_s
 {
   uint32_t time_low;
@@ -35,6 +40,7 @@ struct efi_guid_s
   uint8_t  clock_seq_low;
   uint8_t  node[6];
 } __attribute__ ((gcc_struct, __packed__));
+typedef struct efi_guid_s efi_guid_t;
 
 #define DEFAULT_SECTOR_SIZE     0x200u
 
@@ -105,6 +111,7 @@ struct efi_guid_s
 #define PMAC_FWDRIVER	5
 #define PMAC_SWAP	0x82
 #define PMAC_LINUX	0x83
+#define PMAC_BEOS	0xEB
 #define PMAC_HFS	0xAF
 #define PMAC_MAP	6
 #define PMAC_PATCHES	7
@@ -119,108 +126,115 @@ struct efi_guid_s
 #define PXBOX_FATX	1
 
 #define	GPT_ENT_TYPE_UNUSED		\
-	((const efi_guid_t){le32(0x00000000),le16(0x0000),le16(0x0000),0x00,0x00,{0x00,0x00,0x00,0x00,0x00,0x00}})
+	(const efi_guid_t){le32(0x00000000),le16(0x0000),le16(0x0000),0x00,0x00,{0x00,0x00,0x00,0x00,0x00,0x00}}
 #define	GPT_ENT_TYPE_EFI		\
-	((const efi_guid_t){le32(0xc12a7328),le16(0xf81f),le16(0x11d2),0xba,0x4b,{0x00,0xa0,0xc9,0x3e,0xc9,0x3b}})
+	(const efi_guid_t){le32(0xc12a7328),le16(0xf81f),le16(0x11d2),0xba,0x4b,{0x00,0xa0,0xc9,0x3e,0xc9,0x3b}}
 /* Extended Boot Partition */
 #define GPT_ENT_TYPE_EBP		\
-	((const efi_guid_t){le32(0xbc13c2ff),le16(0x59e6),le16(0x4262),0xa3,0x52,{0xb2,0x75,0xfd,0x6f,0x71,0x72}})
+	(const efi_guid_t){le32(0xbc13c2ff),le16(0x59e6),le16(0x4262),0xa3,0x52,{0xb2,0x75,0xfd,0x6f,0x71,0x72}}
 #define	GPT_ENT_TYPE_MBR		\
-	((const efi_guid_t){le32(0x024dee41),le16(0x33e7),le16(0x11d3),0x9d,0x69,{0x00,0x08,0xc7,0x81,0xf3,0x9f}})
+	(const efi_guid_t){le32(0x024dee41),le16(0x33e7),le16(0x11d3),0x9d,0x69,{0x00,0x08,0xc7,0x81,0xf3,0x9f}}
 #define	GPT_ENT_TYPE_FREEBSD		\
-	((const efi_guid_t){le32(0x516e7cb4),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	(const efi_guid_t){le32(0x516e7cb4),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}}
 #define	GPT_ENT_TYPE_FREEBSD_SWAP	\
-	((const efi_guid_t){le32(0x516e7cb5),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	(const efi_guid_t){le32(0x516e7cb5),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}}
 #define	GPT_ENT_TYPE_FREEBSD_UFS	\
-	((const efi_guid_t){le32(0x516e7cb6),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	(const efi_guid_t){le32(0x516e7cb6),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}}
 #define GPT_ENT_TYPE_FREEBSD_ZFS	\
-	((const efi_guid_t){le32(0x516e7cb),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	(const efi_guid_t){le32(0x516e7cb),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}}
 /*
  * The following is unused but documented here to avoid reuse.
  *
  * GPT_ENT_TYPE_FREEBSD_UFS2	\
- *	((const efi_guid_t){le32(0x516e7cb7),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+ *	(const efi_guid_t){le32(0x516e7cb7),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}}
  */
 
 #define	GPT_ENT_TYPE_FREEBSD_VINUM	\
-	((const efi_guid_t){le32(0x516e7cb8),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}})
+	(const efi_guid_t){le32(0x516e7cb8),le16(0x6ecf),le16(0x11d6),0x8f,0xf8,{0x00,0x02,0x2d,0x09,0x71,0x2b}}
 
 
-#define	GPT_ENT_TYPE_MS_RESERVED	\
-	((const efi_guid_t){le32(0xe3c9e316),le16(0x0b5c),le16(0x4db8),0x81,0x7d,{0xf9,0x2d,0xf0,0x02,0x15,0xae}})
 #define	GPT_ENT_TYPE_MS_BASIC_DATA	\
-	((const efi_guid_t){le32(0xebd0a0a2),le16(0xb9e5),le16(0x4433),0x87,0xc0,{0x68,0xb6,0xb7,0x26,0x99,0xc7}})
-#define	GPT_ENT_TYPE_MS_LDM_METADATA	\
-	((const efi_guid_t){le32(0x5808c8aa),le16(0x7e8f),le16(0x42e0),0x85,0xd2,{0xe1,0xe9,0x04,0x34,0xcf,0xb3}})
+	(const efi_guid_t){le32(0xebd0a0a2),le16(0xb9e5),le16(0x4433),0x87,0xc0,{0x68,0xb6,0xb7,0x26,0x99,0xc7}}
 #define	GPT_ENT_TYPE_MS_LDM_DATA	\
-	((const efi_guid_t){le32(0xaf9b60a0),le16(0x1431),le16(0x4f62),0xbc,0x68,{0x33,0x11,0x71,0x4a,0x69,0xad}})
+	(const efi_guid_t){le32(0xaf9b60a0),le16(0x1431),le16(0x4f62),0xbc,0x68,{0x33,0x11,0x71,0x4a,0x69,0xad}}
+#define	GPT_ENT_TYPE_MS_LDM_METADATA	\
+	(const efi_guid_t){le32(0x5808c8aa),le16(0x7e8f),le16(0x42e0),0x85,0xd2,{0xe1,0xe9,0x04,0x34,0xcf,0xb3}}
 #define	GPT_ENT_TYPE_MS_RECOVERY	\
-	((const efi_guid_t){le32(0xde94bba4),le16(0x06d1),le16(0x4d40),0xa1,0x6a,{0xbf,0xd5,0x01,0x79,0xd6,0xac}})
+	(const efi_guid_t){le32(0xde94bba4),le16(0x06d1),le16(0x4d40),0xa1,0x6a,{0xbf,0xd5,0x01,0x79,0xd6,0xac}}
+#define	GPT_ENT_TYPE_MS_RESERVED	\
+	(const efi_guid_t){le32(0xe3c9e316),le16(0x0b5c),le16(0x4db8),0x81,0x7d,{0xf9,0x2d,0xf0,0x02,0x15,0xae}}
+#define	GPT_ENT_TYPE_MS_SPACES	\
+	(const efi_guid_t){le32(0xe75caf8f),le16(0xf680),le16(0x4cee),0xaf,0xa3,{0xb0,0x01,0xe5,0x6e,0xfc,0x2d}}
 
-#define	GPT_ENT_TYPE_LINUX_RAID		\
-	((const efi_guid_t){le32(0xa19d880f),le16(0x05fc),le16(0x4d3b),0xa0,0x06,{0x74,0x3f,0x0f,0x84,0x91,0x1e}})
-#define	GPT_ENT_TYPE_LINUX_SWAP		\
-	((const efi_guid_t){le32(0x0657fd6d),le16(0xa4ab),le16(0x43c4),0x84,0xe5,{0x09,0x33,0xc8,0x4b,0x4f,0x4f}})
-#define	GPT_ENT_TYPE_LINUX_LVM		\
-	((const efi_guid_t){le32(0xe6d6d379),le16(0xf507),le16(0x44c2),0xa2,0x3c,{0x23,0x8f,0x2a,0x3d,0xf9,0x28}})
-#define GPT_ENT_TYPE_LINUX_RESERVED	\
-	((const efi_guid_t){le32(0x8da63339),le16(0x0007),le16(0x60c0),0xc4,0x36,{0x08,0x3a,0xc8,0x23,0x09,0x08}})
 #define GPT_ENT_TYPE_LINUX_DATA	\
-	((const efi_guid_t){le32(0x0fc63daf),le16(0x8483),le16(0x4772),0x8e,0x79,{0x3d,0x69,0xd8,0x47,0x7d,0xe4}})
+	(const efi_guid_t){le32(0x0fc63daf),le16(0x8483),le16(0x4772),0x8e,0x79,{0x3d,0x69,0xd8,0x47,0x7d,0xe4}}
 #define GPT_ENT_TYPE_LINUX_HOME \
-	((const efi_guid_t){le32(0x933ac7e1),le16(0x2eb4),le16(0x4f13),0xb8,0x44,{0x0e,0x14,0xe2,0xae,0xf9,0x15}})
+	(const efi_guid_t){le32(0x933ac7e1),le16(0x2eb4),le16(0x4f13),0xb8,0x44,{0x0e,0x14,0xe2,0xae,0xf9,0x15}}
+#define	GPT_ENT_TYPE_LINUX_LVM		\
+	(const efi_guid_t){le32(0xe6d6d379),le16(0xf507),le16(0x44c2),0xa2,0x3c,{0x23,0x8f,0x2a,0x3d,0xf9,0x28}}
+#define	GPT_ENT_TYPE_LINUX_RAID		\
+	(const efi_guid_t){le32(0xa19d880f),le16(0x05fc),le16(0x4d3b),0xa0,0x06,{0x74,0x3f,0x0f,0x84,0x91,0x1e}}
+#define GPT_ENT_TYPE_LINUX_RESERVED	\
+	(const efi_guid_t){le32(0x8da63339),le16(0x0007),le16(0x60c0),0xc4,0x36,{0x08,0x3a,0xc8,0x23,0x09,0x08}}
 #define GPT_ENT_TYPE_LINUX_SRV	\
-	((const efi_guid_t){le32(0x3b8f8425),le16(0x20e0),le16(0x4f3b),0x90,0x7f,{0x1a,0x25,0xa7,0x6f,0x98,0xe8}})
+	(const efi_guid_t){le32(0x3b8f8425),le16(0x20e0),le16(0x4f3b),0x90,0x7f,{0x1a,0x25,0xa7,0x6f,0x98,0xe8}}
+#define	GPT_ENT_TYPE_LINUX_SWAP		\
+	(const efi_guid_t){le32(0x0657fd6d),le16(0xa4ab),le16(0x43c4),0x84,0xe5,{0x09,0x33,0xc8,0x4b,0x4f,0x4f}}
 
 #define GPT_ENT_TYPE_HPUX_DATA	\
-	((const efi_guid_t){le32(0x75894c1e),le16(0x3aeb),le16(0x11d3),0xb7,0xc1,{0x7b,0x03,0xa0,0x00,0x00,0x00}})
+	(const efi_guid_t){le32(0x75894c1e),le16(0x3aeb),le16(0x11d3),0xb7,0xc1,{0x7b,0x03,0xa0,0x00,0x00,0x00}}
 #define GPT_ENT_TYPE_HPUX_SERVICE \
-	((const efi_guid_t){le32(0xe2a1e728),le16(0x32e3),le16(0x11d6),0xa6,0x82,{0x7b,0x03,0xa0,0x00,0x00,0x00}})
+	(const efi_guid_t){le32(0xe2a1e728),le16(0x32e3),le16(0x11d6),0xa6,0x82,{0x7b,0x03,0xa0,0x00,0x00,0x00}}
 
-#define GPT_ENT_TYPE_MAC_AFS		\
-	((const efi_guid_t){le32(0x7c3457ef),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
-#define GPT_ENT_TYPE_MAC_HFS		\
-	((const efi_guid_t){le32(0x48465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
-#define GPT_ENT_TYPE_MAC_UFS		\
-	((const efi_guid_t){le32(0x55465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
-#define GPT_ENT_TYPE_MAC_RAID	\
-	((const efi_guid_t){le32(0x52414944),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
-#define GPT_ENT_TYPE_MAC_RAID_OFFLINE 	\
-	((const efi_guid_t){le32(0x52414944),le16(0x5f4f),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+#define GPT_ENT_TYPE_APPLE_CORE_STORAGE \
+	(const efi_guid_t){le32(0x53746F72),le16(0x6167),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
+#define GPT_ENT_TYPE_MAC_APFS		\
+	(const efi_guid_t){le32(0x7c3457ef),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
 #define GPT_ENT_TYPE_MAC_BOOT	\
-	((const efi_guid_t){le32(0x426f6f74),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	(const efi_guid_t){le32(0x426f6f74),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
+#define GPT_ENT_TYPE_MAC_HFS		\
+	(const efi_guid_t){le32(0x48465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
 #define GPT_ENT_TYPE_MAC_LABEL 	\
-	((const efi_guid_t){le32(0x4c616265),le16(0x6c00),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	(const efi_guid_t){le32(0x4c616265),le16(0x6c00),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
+#define GPT_ENT_TYPE_MAC_RAID	\
+	(const efi_guid_t){le32(0x52414944),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
+#define GPT_ENT_TYPE_MAC_RAID_OFFLINE 	\
+	(const efi_guid_t){le32(0x52414944),le16(0x5f4f),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
 #define GPT_ENT_TYPE_MAC_TV_RECOVERY  	\
-	((const efi_guid_t){le32(0x5265636f),le16(0x7665),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}})
+	(const efi_guid_t){le32(0x5265636f),le16(0x7665),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
+#define GPT_ENT_TYPE_MAC_UFS		\
+	(const efi_guid_t){le32(0x55465300),le16(0x0000),le16(0x11aa),0xaa,0x11,{0x00,0x30,0x65,0x43,0xec,0xac}}
 
-#define GPT_ENT_TYPE_SOLARIS_BOOT  	\
-	((const efi_guid_t){le32(0x6a82cb45),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
-#define GPT_ENT_TYPE_SOLARIS_ROOT  	\
-	((const efi_guid_t){le32(0x6a85cf4d),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
-#define GPT_ENT_TYPE_SOLARIS_SWAP  	\
-	((const efi_guid_t){le32(0x6a87c46f),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
 #define GPT_ENT_TYPE_SOLARIS_BACKUP  	\
-	((const efi_guid_t){le32(0x6a8b642b),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a8b642b),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
+#define GPT_ENT_TYPE_SOLARIS_BOOT  	\
+	(const efi_guid_t){le32(0x6a82cb45),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
+#define GPT_ENT_TYPE_SOLARIS_ROOT  	\
+	(const efi_guid_t){le32(0x6a85cf4d),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
+#define GPT_ENT_TYPE_SOLARIS_SWAP  	\
+	(const efi_guid_t){le32(0x6a87c46f),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_USR  	\
-	((const efi_guid_t){le32(0x6a898cc3),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
-#define GPT_ENT_TYPE_MAC_ZFS		GPT_ENT_TYPE_SOLARIS_USR  
+	(const efi_guid_t){le32(0x6a898cc3),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
+#define GPT_ENT_TYPE_MAC_ZFS		GPT_ENT_TYPE_SOLARIS_USR
 #define GPT_ENT_TYPE_SOLARIS_VAR  	\
-	((const efi_guid_t){le32(0x6a8ef2e9),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a8ef2e9),le16(0x1dd2),le16(0x11b2),0x99,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_HOME  	\
-	((const efi_guid_t){le32(0x6a90ba39),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a90ba39),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_EFI_ALTSCTR \
-	((const efi_guid_t){le32(0x6a9283a5),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a9283a5),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_RESERVED1	\
-	((const efi_guid_t){le32(0x6a945a3b),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a945a3b),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_RESERVED2	\
-	((const efi_guid_t){le32(0x6a9630d1),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a9630d1),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_RESERVED3	\
-	((const efi_guid_t){le32(0x6a980767),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a980767),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_RESERVED4	\
-	((const efi_guid_t){le32(0x6a96237f),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a96237f),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
 #define GPT_ENT_TYPE_SOLARIS_RESERVED5	\
-	((const efi_guid_t){le32(0x6a8d2ac7),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}})
+	(const efi_guid_t){le32(0x6a8d2ac7),le16(0x1dd2),le16(0x11b2),0x96,0xa6,{0x08,0x00,0x20,0x73,0x66,0x31}}
+
+#define GPT_ENT_TYPE_BEOS_BFS	\
+	(const efi_guid_t){le32(0x42465331),le16(0x3ba3),le16(0x10f1),0x80,0x2a,{0x48,0x61,0x69,0x6b,0x75,0x21}}
 
 #define TESTDISK_O_RDONLY     00
 #define TESTDISK_O_RDWR	      02
@@ -231,6 +245,7 @@ struct efi_guid_s
 
 enum upart_type {
   UP_UNK=0,
+  UP_APFS,
   UP_BEOS,
   UP_BTRFS,
   UP_CRAMFS,
@@ -301,6 +316,10 @@ typedef enum errcode_type errcode_type_t;
 
 typedef struct param_disk_struct disk_t;
 typedef struct partition_struct partition_t;
+/*@
+    predicate valid_partition(partition_t *part) = (\valid_read(part));
+  @*/
+
 typedef struct CHS_struct CHS_t;
 typedef struct
 {
@@ -326,6 +345,16 @@ struct list_part_struct
   int to_be_removed;
 };
 
+/*@
+inductive valid_list_part{L} (list_part_t *list)
+{
+  case list_null{L}:
+    valid_list_part(\null);
+  case list_not_null{L}:
+    \forall list_part_t *list; \valid_read(list) ==> valid_list_part(list->next) ==> valid_list_part(list);
+}
+  @*/
+
 typedef struct list_disk_struct list_disk_t;
 struct list_disk_struct
 {
@@ -334,6 +363,18 @@ struct list_disk_struct
   list_disk_t *next;
 };
 
+/*@
+inductive ld_reachable{L} (list_disk_t* root, list_disk_t* node)
+{
+  case root_ld_reachable{L}:
+    \forall list_disk_t *root; ld_reachable(root,root);
+  case next_ld_reachable{L}:
+    \forall list_disk_t *root, *node; \valid(root) ==> ld_reachable(root->next, node) ==> ld_reachable(root,node);
+}
+*/
+
+/*@ predicate ld_finite{L}(list_disk_t* root) = ld_reachable(root,\null); */
+
 struct systypes {
   const unsigned int part_type;
   const char *name;
@@ -353,7 +394,7 @@ struct arch_fnct_struct
   int (*write_MBR_code)(disk_t *disk);
   void (*set_prev_status)(const disk_t *disk, partition_t *partition);
   void (*set_next_status)(const disk_t *disk, partition_t *partition);
-  int (*test_structure)(list_part_t *list_part);
+  int (*test_structure)(const list_part_t *list_part);
   unsigned int (*get_part_type)(const partition_t *partition);
   int (*set_part_type)(partition_t *partition, unsigned int part_type);
   void (*init_structure)(const disk_t *disk,list_part_t *list_part, const int verbose);
@@ -364,6 +405,13 @@ struct arch_fnct_struct
 
 typedef struct arch_fnct_struct arch_fnct_t;
 
+/*@
+    predicate valid_arch(arch_fnct_t *arch) = (
+      \valid_read(arch) &&
+      (arch->get_geometry_from_mbr==\null || \valid_function(arch->get_geometry_from_mbr))
+    );
+  @*/
+
 struct param_disk_struct
 {
   char description_txt[DISKDESCRIPTION_MAX];
@@ -399,6 +447,37 @@ struct param_disk_struct
   unsigned int sector_size;
 };
 
+/*@
+    predicate valid_disk(disk_t *disk) =
+      (\valid_read(disk) &&
+       \freeable(disk) &&
+       valid_read_string(disk->device) &&
+       \freeable(disk->device) &&
+       \valid_function(disk->clean) &&
+       \valid_function(disk->description) &&
+       (disk->model == \null || \freeable(disk->model)) &&
+       (disk->model == \null || valid_read_string(disk->model)) &&
+       (disk->serial_no == \null || \freeable(disk->serial_no)) &&
+       (disk->fw_rev == \null || \freeable(disk->fw_rev)) &&
+       (disk->data == \null || \freeable(disk->data)) &&
+       (disk->rbuffer == \null || (\freeable(disk->rbuffer) && disk->rbuffer_size > 0)) &&
+       (disk->wbuffer == \null || (\freeable(disk->wbuffer) && disk->wbuffer_size > 0)) &&
+       valid_arch(disk->arch) &&
+       disk->sector_size > 0
+      );
+*/
+
+/*@
+inductive valid_list_disk{L} (list_disk_t *list)
+{
+  case list_null{L}:
+    valid_list_disk(\null);
+  case list_not_null{L}:
+    \forall list_disk_t *list; \valid_read(list) && valid_list_disk(list) ==> valid_disk(list->disk) && valid_list_disk(list->next);
+}
+  @*/
+
+
 struct partition_struct
 {
   char          fsname[128];
@@ -438,14 +517,56 @@ struct my_data_struct
   uint64_t offset;
 };
 
+/*@
+  @ requires size > 0;
+  @ ensures \valid(((char *)\result)+(0..size-1));
+  @ ensures zero_initialization: \subset(((char *)\result)[0..size-1], {0});
+  @ assigns __fc_heap_status;
+  @*/
 void *MALLOC(size_t size);
-unsigned int up2power(const unsigned int number);
+
+/*@
+  @ requires \valid(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid_read(src + (0 .. max_size-1));
+  @ requires \separated(partition, src + (..));
+  @ terminates \true;
+  @ ensures valid_string((char *)&partition->fsname);
+  @*/
 void set_part_name(partition_t *partition, const char *src, const unsigned int max_size);
-void set_part_name_chomp(partition_t *partition, const unsigned char *src, const unsigned int max_size);
+
+/*@
+  @ requires \valid(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid_read(src + (0 .. max_size-1));
+  @ requires \separated(partition, src);
+  @ terminates \true;
+  @ ensures valid_string((char *)&partition->fsname);
+  @*/
+void set_part_name_chomp(partition_t *partition, const char *src, const unsigned int max_size);
+
+/*@
+  @ requires valid_read_string(str);
+  @ ensures \result == \null || valid_read_string(\result);
+  @*/
 char* strip_dup(char* str);
-int date_dos2unix(const unsigned short f_time,const unsigned short f_date);
+
+/*@
+  @ requires f_time <= 0xffffffff;
+  @ requires f_date <= 0xffffffff;
+  @ terminates \true;
+  @ assigns \nothing;
+  @*/
+time_t date_dos2unix(const unsigned short f_time,const unsigned short f_date);
+
 void set_secwest(void);
+
+/*@
+  @ terminates \true;
+  @ assigns \nothing;
+  @*/
 time_t td_ntfs2utc (int64_t ntfstime);
+
 #ifndef BSD_MAXPARTITIONS
 #define	BSD_MAXPARTITIONS	8
 #endif
@@ -453,47 +574,45 @@ time_t td_ntfs2utc (int64_t ntfstime);
 #define	OPENBSD_MAXPARTITIONS	16
 #endif
 
-#ifdef TESTDISK_LSB
-#define le16(x)  (x)             /* x as little endian */
-#define be16(x)  ((((x)&0xff00)>>8)                      | \
-    (((x)&0x00ff)<<8))
-#define le24(x)  (x)
-#define le32(x)  (x)
-#define be32(x)  ((((x)&0xff000000UL)>>24)                | \
-    (((x)&0x00ff0000UL)>>8)                  | \
-    (((x)&0x0000ff00UL)<<8)                  | \
-    (((x)&0x000000ffUL)<<24))
-#define le64(x)  (x)
-#define be64(x)  ((((x)&0xff00000000000000ULL)>>56)       | \
-    (((x)&0x00ff000000000000ULL)>>40)        | \
-    (((x)&0x0000ff0000000000ULL)>>24)        | \
-    (((x)&0x000000ff00000000ULL)>>8)         | \
-    (((x)&0x00000000ff000000ULL)<<8)         | \
-    (((x)&0x0000000000ff0000ULL)<<24)        | \
-    (((x)&0x000000000000ff00ULL)<<40)        | \
-    (((x)&0x00000000000000ffULL)<<56))
-#else /* bigendian */
-#define le16(x)  ((((x)&0xff00)>>8)                      | \
-    (((x)&0x00ff)<<8))
-#define be16(x)  (x)
-#define be24(x)  (x)
-#define le24(x) ((((x) & 0x000000ffUL) << 16) | \
+#define __swab16(x) (((((uint16_t)x)&(uint16_t)0xff00)>>8)                      | \
+    (((uint16_t)(x)&(uint16_t)0x00ff)<<8))
+
+#define __swab24(x) ((((x) & 0x000000ffUL) << 16) | \
     ((x) & 0x0000ff00UL)        | \
     (((x) & 0x00ff0000UL) >> 16))
-#define le32(x)  ((((x)&0xff000000UL)>>24)                | \
-    (((x)&0x00ff0000UL)>>8)                  | \
-    (((x)&0x0000ff00UL)<<8)                  | \
-    (((x)&0x000000ffUL)<<24))
+
+#define __swab32(x)  ((((uint32_t)(x)&(uint32_t)0xff000000UL)>>24)                | \
+    (((uint32_t)(x)&(uint32_t)0x00ff0000UL)>>8)                  | \
+    (((uint32_t)(x)&(uint32_t)0x0000ff00UL)<<8)                  | \
+    (((uint32_t)(x)&(uint32_t)0x000000ffUL)<<24))
+
+#define __swab64(x)  ((((uint64_t)(x)&(uint64_t)0xff00000000000000ULL)>>56)       | \
+    (((uint64_t)(x)&(uint64_t)0x00ff000000000000ULL)>>40)        | \
+    (((uint64_t)(x)&(uint64_t)0x0000ff0000000000ULL)>>24)        | \
+    (((uint64_t)(x)&(uint64_t)0x000000ff00000000ULL)>>8)         | \
+    (((uint64_t)(x)&(uint64_t)0x00000000ff000000ULL)<<8)         | \
+    (((uint64_t)(x)&(uint64_t)0x0000000000ff0000ULL)<<24)        | \
+    (((uint64_t)(x)&(uint64_t)0x000000000000ff00ULL)<<40)        | \
+    (((uint64_t)(x)&(uint64_t)0x00000000000000ffULL)<<56))
+
+#ifdef TESTDISK_LSB
+#define be16(x)  (__swab16(x))
+#define be24(x)  (__swab24(x))
+#define be32(x)  (__swab32(x))
+#define be64(x)  (__swab64(x))
+#define le16(x)  (x)             /* x as little endian */
+#define le24(x)  (x)
+#define le32(x)  (x)
+#define le64(x)  (x)
+#else /* bigendian */
+#define be16(x)  (x)
+#define be24(x)  (x)
 #define be32(x)  (x)
-#define le64(x)  ((((x)&0xff00000000000000ULL)>>56)       | \
-    (((x)&0x00ff000000000000ULL)>>40)        | \
-    (((x)&0x0000ff0000000000ULL)>>24)        | \
-    (((x)&0x000000ff00000000ULL)>>8)         | \
-    (((x)&0x00000000ff000000ULL)<<8)         | \
-    (((x)&0x0000000000ff0000ULL)<<24)        | \
-    (((x)&0x000000000000ff00ULL)<<40)        | \
-    (((x)&0x00000000000000ffULL)<<56))
 #define be64(x)  (x)
+#define le16(x)  (__swab16(x))
+#define le24(x)  (__swab24(x))
+#define le32(x)  (__swab32(x))
+#define le64(x)  (__swab64(x))
 #endif
 #ifndef HAVE_SNPRINTF
 int snprintf(char *str, size_t size, const char *format, ...);
@@ -508,7 +627,11 @@ int strncasecmp(const char * s1, const char * s2, size_t len);
 #ifndef HAVE_STRCASESTR
 char * strcasestr (const char *haystack, const char *needle);
 #endif
-#if ! defined(HAVE_LOCALTIME_R) && ! defined(__MINGW32__)
+#if ! defined(HAVE_LOCALTIME_R) && ! defined(__MINGW32__) && !defined(DISABLED_FOR_FRAMAC)
+/*@
+  @ requires valid_timer: \valid_read(timep);
+  @ requires \valid(result);
+  @*/
 struct tm *localtime_r(const time_t *timep, struct tm *result);
 #endif
 /*
@@ -529,8 +652,38 @@ struct tm *localtime_r(const time_t *timep, struct tm *result);
         (void) (&_x == &_y);            \
         _x > _y ? _x : _y; })
 
-int check_command(char **current_cmd, const char *cmd, size_t n);
+/*@
+  @ requires \valid(current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @ requires valid_read_string(cmd);
+  @ requires \separated(cmd+(..), current_cmd, *current_cmd);
+  @ requires strlen(cmd) == n;
+  @ assigns  *current_cmd;
+  @ ensures  valid_read_string(*current_cmd);
+  @ ensures  \result != 0 ==> *current_cmd == \old(*current_cmd);
+  @*/
+// ensures  \result == 0 ==> *current_cmd == \old(*current_cmd) + n;
+// assigns \result \from indirect:(*current_cmd)[0 .. n-1], indirect:cmd[0 ..n-1], indirect:n;
+int check_command(char **current_cmd, const char *cmd, const size_t n);
+
+/*@
+  @ requires \valid(current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @ requires \separated(current_cmd, *current_cmd);
+  @ terminates \true;
+  @ assigns  *current_cmd;
+  @ ensures  valid_read_string(*current_cmd);
+  @*/
 void skip_comma_in_command(char **current_cmd);
+
+/*@
+  @ requires \valid(current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @ requires \separated(current_cmd, *current_cmd);
+  @ terminates \true;
+  @ assigns  *current_cmd;
+  @ ensures  valid_read_string(*current_cmd);
+  @*/
 uint64_t get_int_from_command(char **current_cmd);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
