@@ -90,9 +90,16 @@ unsigned int get_crc32(const void*buf, const unsigned int len, const uint32_t se
   unsigned int i;
   register uint32_t crc32val;
   const unsigned char *s=(const unsigned char *)buf;
+  /*@ assert \valid_read(s + (0 .. len-1)); */
   crc32val = seed; 
+  /*@
+    @ loop invariant 0 <= i <= len;
+    @ loop assigns i, crc32val;
+    @ loop variant len - i;
+    @*/
   for (i = 0;  i < len;  i ++)
   {
+    /*@ assert i < len; */
     crc32val = crc32_tab[(crc32val ^ s[i]) & 0xff] ^ (crc32val >> 8);
   }
   return (unsigned int)crc32val;
