@@ -22,6 +22,10 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+
+#ifdef DISABLED_FOR_FRAMAC
+#undef HAVE_CHMOD
+#endif
  
 #include <stdio.h>
 #ifdef HAVE_STRING_H
@@ -53,8 +57,10 @@
 const char *monstr[] = { "Jan", "Feb", "Mar", "Apr", "May", "Jun",
 				"Jul", "Aug", "Sep", "Oct", "Nov", "Dec"};
 
-static char ftypelet (unsigned int bits);
-
+/*@
+  @ terminates \true;
+  @ assigns \result;
+  @*/
 static char ftypelet (unsigned int bits)
 {
 #ifdef LINUX_S_ISBLK
@@ -110,6 +116,7 @@ static char ftypelet (unsigned int bits)
 
 void mode_string (const unsigned int mode, char *str)
 {
+#ifndef DISABLED_FOR_FRAMAC
   str[0] = ftypelet(mode);
   str[1] = (mode & LINUX_S_IRUSR) ? 'r' : '-';
   str[2] = (mode & LINUX_S_IWUSR) ? 'w' : '-';
@@ -151,6 +158,7 @@ void mode_string (const unsigned int mode, char *str)
       str[9] = 't';
   }
 #endif
+#endif
 }
 
 int set_datestr(char *datestr, size_t n, const time_t timev)
@@ -164,7 +172,7 @@ int set_datestr(char *datestr, size_t n, const time_t timev)
     strncpy(datestr, "                 ", n);
     return 0;
   }
-#if defined(__MINGW32__)
+#if defined(__MINGW32__) || defined(DISABLED_FOR_FRAMAC)
   tm_p=localtime(&timev);
 #else
   tm_p=localtime_r(&timev, &tmp);
@@ -191,6 +199,7 @@ int dir_aff_log(const dir_data_t *dir_data, const file_info_t *dir_list)
   {
     log_info("Directory %s\n",dir_data->current_directory);
   }
+#ifndef DISABLED_FOR_FRAMAC
   td_list_for_each(file_walker, &dir_list->list)
   {
     const file_info_t *current_file=td_list_entry_const(file_walker, const file_info_t, list);
@@ -218,11 +227,13 @@ int dir_aff_log(const dir_data_t *dir_data, const file_info_t *dir_list)
     }
     log_info("%s\n", current_file->name);
   }
+#endif
   return test_date;
 }
 
 void log_list_file(const disk_t *disk, const partition_t *partition, const dir_data_t *dir_data, const file_info_t*list)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *tmp;
   log_partition(disk, partition);
   if(dir_data!=NULL)
@@ -246,6 +257,7 @@ void log_list_file(const disk_t *disk, const partition_t *partition, const dir_d
     log_info("%9llu", (long long unsigned int)current_file->st_size);
     log_info(" %s %s\n", datestr, current_file->name);
   }
+#endif
 }
 
 unsigned int delete_list_file(file_info_t *file_info)
@@ -253,6 +265,7 @@ unsigned int delete_list_file(file_info_t *file_info)
   unsigned int nbr=0;
   struct td_list_head *file_walker = NULL;
   struct td_list_head *file_walker_next = NULL;
+#ifndef DISABLED_FOR_FRAMAC
   td_list_for_each_safe(file_walker,file_walker_next, &file_info->list)
   {
     file_info_t *tmp;
@@ -262,9 +275,15 @@ unsigned int delete_list_file(file_info_t *file_info)
     free(tmp);
     nbr++;
   }
+#endif
   return nbr;
 }
 
+/*@
+  @ requires \valid_read(current_file);
+  @ requires \valid_read(inode_known + (0 .. dir_nbr-1));
+  @ assigns \nothing;
+  @*/
 static int is_inode_valid(const file_info_t *current_file, const unsigned int dir_nbr, const unsigned long int *inode_known)
 {
   const unsigned long int new_inode=current_file->st_ino;
@@ -273,12 +292,25 @@ static int is_inode_valid(const file_info_t *current_file, const unsigned int di
     return 0;
   if(strcmp(current_file->name, "..")==0)
     return 0;
+  /*@
+    @ loop assigns i;
+    @ loop variant dir_nbr - i;
+    @*/
   for(i=0; i<dir_nbr; i++)
     if(new_inode==inode_known[i]) /* Avoid loop */
       return 0;
   return 1;
 }
 
+/*@
+  @ requires \valid(disk);
+  @ requires valid_disk(disk);
+  @ requires \valid_read(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid(dir_data);
+  @ requires \separated(disk, partition, dir_data);
+  @ decreases 0;
+  @*/
 static int dir_whole_partition_log_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode)
 {
   struct td_list_head *file_walker = NULL;
@@ -321,6 +353,17 @@ int dir_whole_partition_log(disk_t *disk, const partition_t *partition, dir_data
   return dir_whole_partition_log_aux(disk, partition, dir_data, inode);
 }
 
+/*@
+  @ requires \valid(disk);
+  @ requires valid_disk(disk);
+  @ requires \valid_read(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid(dir_data);
+  @ requires \valid(copy_ok);
+  @ requires \valid(copy_bad);
+  @ requires \separated(disk, partition, dir_data, copy_ok, copy_bad);
+  @ decreases 0;
+  @*/
 static int dir_whole_partition_copy_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, unsigned int *copy_ok, unsigned int *copy_bad)
 {
   struct td_list_head *file_walker = NULL;
@@ -388,17 +431,21 @@ void dir_whole_partition_copy(disk_t *disk, const partition_t *partition, dir_da
 int filesort(const struct td_list_head *a, const struct td_list_head *b)
 {
   const file_info_t *file_a=td_list_entry_const(a, const file_info_t, list);
+  /*@ assert \valid_read(file_a); */
   const file_info_t *file_b=td_list_entry_const(b, const file_info_t, list);
+  /*@ assert \valid_read(file_b); */
   /* Directories must be listed before files */
   const int res=((file_b->st_mode&LINUX_S_IFDIR)-(file_a->st_mode&LINUX_S_IFDIR));
   if(res)
     return res;
+  /*@ assert valid_read_string(file_a->name); */
   /* . and .. must listed before the other directories */
   if((file_a->st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, ".")==0)
     return -1;
   if((file_a->st_mode&LINUX_S_IFDIR) && strcmp(file_a->name, "..")==0 &&
       strcmp(file_b->name, ".")!=0)
     return -1;
+  /*@ assert valid_read_string(file_b->name); */
   if((file_b->st_mode&LINUX_S_IFDIR) && strcmp(file_b->name, ".")==0)
     return 1;
   if((file_b->st_mode&LINUX_S_IFDIR) && strcmp(file_b->name, "..")==0 &&
@@ -446,12 +493,19 @@ static struct {
   { 0, 0 }
 };
 
+/*@
+  @ assigns \nothing;
+  @*/
 static mode_t mode_xlate(unsigned int lmode)
 {
+  unsigned int i;
   mode_t  mode = 0;
-  int     i;
-
-  for (i=0; mode_table[i].lmask; i++) {
+  /*@
+    @ loop unroll 20;
+    @ loop assigns i, mode;
+    @*/
+  for (i=0; mode_table[i].lmask; i++)
+  {
     if (lmode & mode_table[i].lmask)
       mode |= mode_table[i].mask;
   }
@@ -477,10 +531,20 @@ int set_mode(const char *pathname, unsigned int mode)
 #endif
 }
 
+/*@
+  @ requires valid_string(fn);
+  @*/
 static void strip_fn(char *fn)
 {
   unsigned int i;
+  /*@
+    @ loop assigns i;
+    @*/
   for(i=0;fn[i]!='\0';i++);
+  /*@
+    @ loop assigns i;
+    @ loop invariant i;
+    @*/
   while(i>0 && (fn[i-1]==' '||fn[i-1]=='.'))
     i--;
   if(i==0 && (fn[i]==' '||fn[i]=='.'))
@@ -548,8 +612,15 @@ static inline unsigned char convert_char_dos(unsigned char car)
 static unsigned int filename_convert(char *dst, const char*src, const unsigned int n)
 {
   unsigned int i;
+  /*@
+    @ loop assigns i, dst[0 .. i];
+    @ loop variant n - i;
+    @*/
   for(i=0;i<n && src[i]!='\0';i++)
     dst[i]=convert_char_dos(src[i]);
+  /*@
+    @ loop variant i;
+    @*/
   while(i>0 && (dst[i-1]==' '||dst[i-1]=='.'))
     i--;
   if(i==0 && (dst[i]==' '||dst[i]=='.'))
@@ -638,9 +709,18 @@ static unsigned int filename_convert(char *dst, const char*src, const unsigned i
   return j;
 }
 #else
+/*@
+  @ requires \valid(dst + (0 .. n));
+  @ requires \valid_read(src + (0 .. n-1));
+  @ requires \separated(dst + (..), src + (..));
+  @*/
 static unsigned int filename_convert(char *dst, const char*src, const unsigned int n)
 {
   unsigned int i;
+  /*@
+    @ loop assigns i, dst[0 .. i];
+    @ loop invariant n - i;
+    @*/
   for(i=0;i<n && src[i]!='\0';i++)
     dst[i]=src[i];
   dst[i]='\0';
@@ -662,6 +742,9 @@ char *gen_local_filename(const char *filename)
 
 char *mkdir_local(const char *localroot, const char *pathname)
 {
+#ifdef DISABLED_FOR_FRAMAC
+  return NULL;
+#else
   const int l1=(localroot==NULL?0:strlen(localroot));
   const int l2=strlen(pathname);
   char *localdir=(char *)MALLOC(l1+l2+1);
@@ -725,6 +808,7 @@ char *mkdir_local(const char *localroot, const char *pathname)
 #warning "You need a mkdir function!"
 #endif
   return localdir;
+#endif
 }
 
 void mkdir_local_for_file(const char *filename)
@@ -743,6 +827,9 @@ void mkdir_local_for_file(const char *filename)
 
 FILE *fopen_local(char **localfilename, const char *localroot, const char *filename)
 {
+#ifdef DISABLED_FOR_FRAMAC
+  return NULL;
+#else
   const int l1=strlen(localroot);
   const int l2=strlen(filename);
   const char *src;
@@ -807,4 +894,5 @@ FILE *fopen_local(char **localfilename, const char *localroot, const char *filen
     return f_out;
   filename_convert(dst_org, src_org, l2);
   return fopen(*localfilename,"wb");
+#endif
 }
