@@ -22,6 +22,10 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_NCURSES
+#endif
  
 #ifdef HAVE_NCURSES
 #include <stdio.h>
@@ -45,8 +49,9 @@
 #include "askloc.h"
 #include "setdate.h"
 
-static int copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir, unsigned int *copy_ok, unsigned int *copy_bad);
-static int copy_selection(file_info_t*dir_list, WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, unsigned int *copy_ok, unsigned int *copy_bad);
+typedef enum { CD_FINISHED = 0, CD_STOPPED = 1, CD_NOSPACE = 2 } copy_dir_t;
+static copy_dir_t copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir, unsigned int *copy_ok, unsigned int *copy_bad);
+static copy_dir_t copy_selection(file_info_t*dir_list, WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, unsigned int *copy_ok, unsigned int *copy_bad);
 
 #define INTER_DIR (LINES-25+15)
 #define MAX_DIR_NBR 256
@@ -75,7 +80,7 @@ static int copy_progress(WINDOW *window, const unsigned int copy_ok, const unsig
   return check_enter_key_or_s(window);
 }
 
-static void copy_done(WINDOW *window, const unsigned int copy_ok, const unsigned int copy_bad, const unsigned int copy_stopped)
+static void copy_done(WINDOW *window, const unsigned int copy_ok, const unsigned int copy_bad, const copy_dir_t copy_stopped)
 {
   wmove(window,5,0);
   wclrtoeol(window);
@@ -86,8 +91,10 @@ static void copy_done(WINDOW *window, const unsigned int copy_ok, const unsigned
     else
       wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(2));
   }
-  if(copy_stopped)
+  if(copy_stopped == CD_STOPPED)
     wprintw(window,"Copy stopped! %u ok, %u failed", copy_ok, copy_bad);
+  else if(copy_stopped == CD_NOSPACE)
+    wprintw(window,"Copy stopped! %u ok, %u failed - Not enough space", copy_ok, copy_bad);
   else
     wprintw(window,"Copy done! %u ok, %u failed", copy_ok, copy_bad);
   if(has_colors())
@@ -202,7 +209,7 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	waddstr(window,", ");
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-	waddstr(window,"h");
+	waddstr(window,"'h'");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	if((dir_data->param&FLAG_LIST_DELETED)==0)
@@ -215,7 +222,7 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	waddstr(window,", ");
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-	waddstr(window,"h");
+	waddstr(window,"'h'");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	if((dir_data->param&FLAG_LIST_ADS)==0)
@@ -226,7 +233,7 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
       wmove(window,LINES-2,4);
       if(has_colors())
 	wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-      waddstr(window,"q");
+      waddstr(window,"'q'");
       if(has_colors())
 	wbkgdset(window,' ' | COLOR_PAIR(0));
       waddstr(window," to quit");
@@ -235,13 +242,13 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	waddstr(window,", ");
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-	waddstr(window,":");
+	waddstr(window,"':'");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	waddstr(window," to select the current file, ");
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-	waddstr(window,"a");
+	waddstr(window,"'a'");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	if((status&FILE_STATUS_MARKED)==FILE_STATUS_MARKED)
@@ -250,13 +257,13 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	  waddstr(window," to deselect all files");
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-	mvwaddstr(window,LINES-1,4,"C");
+	mvwaddstr(window,LINES-1,4,"'C'");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	waddstr(window," to copy the selected files, ");
 	if(has_colors())
 	  wbkgdset(window,' ' | A_BOLD | COLOR_PAIR(0));
-	waddstr(window,"c");
+	waddstr(window,"'c'");
 	if(has_colors())
 	  wbkgdset(window,' ' | COLOR_PAIR(0));
 	waddstr(window," to copy the current file");
@@ -407,21 +414,30 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 		  strcat(dir_data->current_directory,tmp->name);
 		if(dir_data->local_dir==NULL || ask_destination>0)
 		{
-		  char *local_dir=dir_data->local_dir;
+		  char dst_directory[4096];
+		  dst_directory[0]='\0';
+		  if(dir_data->local_dir!=NULL)
+		  {
+		    strncpy(dst_directory, dir_data->local_dir, sizeof(dst_directory)-1);
+		    dst_directory[sizeof(dst_directory)-1]='\0';
+		  }
 		  if(LINUX_S_ISDIR(tmp->st_mode)!=0)
-		    dir_data->local_dir=ask_location("Please select a destination where %s and any files below will be copied.",
-			dir_data->current_directory, local_dir);
+		    ask_location(dst_directory, sizeof(dst_directory), "Please select a destination where %s and any files below will be copied.",
+			dir_data->current_directory);
 		  else
-		    dir_data->local_dir=ask_location("Please select a destination where %s will be copied.",
-			dir_data->current_directory, local_dir);
-		  free(local_dir);
+		    ask_location(dst_directory, sizeof(dst_directory), "Please select a destination where %s will be copied.",
+			dir_data->current_directory);
+		  free(dir_data->local_dir);
+		  dir_data->local_dir=NULL;
+		  if(dst_directory[0]!='\0')
+		    dir_data->local_dir=strdup(dst_directory);
 		  ask_destination=0;
 		}
 		if(dir_data->local_dir!=NULL)
 		{
 		  unsigned int copy_bad=0;
 		  unsigned int copy_ok=0;
-		  unsigned int copy_stopped=0;
+		  copy_dir_t copy_stopped=CD_FINISHED;
 		  aff_copy(window);
 		  wmove(window,3,0);
 		  aff_part(window, AFF_PART_ORDER|AFF_PART_STATUS, disk, partition);
@@ -437,12 +453,18 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 		  }
 		  else if(LINUX_S_ISREG(tmp->st_mode)!=0)
 		  {
+		    copy_file_t res;
 		    copy_progress(window, copy_ok, copy_bad);
-		    if(dir_data->copy_file(disk, partition, dir_data, tmp)==0)
+		    res=dir_data->copy_file(disk, partition, dir_data, tmp);
+		    if(res==CP_NOSPACE)
+		      copy_stopped=CD_NOSPACE;
+		    if(res==CP_OK)
 		      copy_ok++;
 		    else
 		      copy_bad++;
 		  }
+		  wmove(window,22,0);
+		  wclrtoeol(window);
 		  copy_done(window, copy_ok, copy_bad, copy_stopped);
 		}
 		dir_data->current_directory[current_directory_namelength]='\0';
@@ -454,16 +476,25 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 	    {
 	      if(dir_data->local_dir==NULL || ask_destination>0)
 	      {
-		char *local_dir=dir_data->local_dir;
-		dir_data->local_dir=ask_location("Please select a destination where the marked files will be copied.", NULL, local_dir);
-		free(local_dir);
+		char dst_directory[4096];
+		dst_directory[0]='\0';
+		if(dir_data->local_dir!=NULL)
+		{
+		  strncpy(dst_directory, dir_data->local_dir, sizeof(dst_directory)-1);
+		  dst_directory[sizeof(dst_directory)-1]='\0';
+		}
+		ask_location(dst_directory, sizeof(dst_directory), "Please select a destination where the marked files will be copied.", NULL);
+		free(dir_data->local_dir);
+		dir_data->local_dir=NULL;
+		if(dst_directory[0]!='\0')
+		  dir_data->local_dir=strdup(dst_directory);
 		ask_destination=0;
 	      }
 	      if(dir_data->local_dir!=NULL)
 	      {
 		unsigned int copy_bad=0;
 		unsigned int copy_ok=0;
-		unsigned int copy_stopped;
+		copy_dir_t copy_stopped;
 		aff_copy(window);
 		wmove(window,3,0);
 		aff_part(window, AFF_PART_ORDER|AFF_PART_STATUS, disk, partition);
@@ -474,6 +505,8 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
 		waddstr(window,"  Stop  ");
 		wattroff(window, A_REVERSE);
 		copy_stopped=copy_selection(dir_list, window, disk, partition, dir_data, &copy_ok, &copy_bad);
+		wmove(window,22,0);
+		wclrtoeol(window);
 		copy_done(window, copy_ok, copy_bad, copy_stopped);
 	      }
 	    }
@@ -576,6 +609,24 @@ static long int dir_aff_ncurses(disk_t *disk, const partition_t *partition, dir_
   return -1;
 }
 
+static int is_inode_valid(const unsigned long int new_inode, const unsigned long int inode_known[MAX_DIR_NBR], const unsigned int dir_nbr)
+{
+  unsigned int i;
+  if(new_inode<2)
+    return 0;
+  for(i=0; i<dir_nbr; i++)
+    if(new_inode == inode_known[i]) /* Avoid loop */
+      return 0;
+  return 1;
+}
+
+static int can_copy_dir(const file_info_t *current_file, const unsigned long int inode_known[MAX_DIR_NBR], const unsigned int dir_nbr)
+{
+  if(strcmp(current_file->name,"..")==0 || strcmp(current_file->name,".")==0)
+    return 0;
+  return is_inode_valid(current_file->st_ino, inode_known, dir_nbr);
+}
+
 static int dir_partition_aux(disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const unsigned long int inode, const unsigned int depth, char**current_cmd)
 {
   static unsigned long int inode_known[MAX_DIR_NBR];
@@ -609,17 +660,9 @@ static int dir_partition_aux(disk_t *disk, const partition_t *partition, dir_dat
       delete_list_file(&dir_list);
       return new_inode;
     }
-    if(new_inode>=2)
+    if(is_inode_valid(new_inode, &inode_known[0], depth) > 0)
     {
-      unsigned int new_inode_ok=1;
-      unsigned int i;
-      for(i=0;i<=depth && new_inode_ok!=0;i++)
-	if((unsigned)new_inode==inode_known[i]) /* Avoid loop */
-	  new_inode_ok=0;
-      if(new_inode_ok>0)
-      {
-	dir_partition_aux(disk, partition, dir_data, (unsigned long int)new_inode, depth+1, current_cmd);
-      }
+      dir_partition_aux(disk, partition, dir_data, (unsigned long int)new_inode, depth+1, current_cmd);
     }
     /* restore current_directory name */
     dir_data->current_directory[current_directory_namelength]='\0';
@@ -634,14 +677,7 @@ int dir_partition_aff(disk_t *disk, const partition_t *partition, dir_data_t *di
   return dir_partition_aux(disk, partition, dir_data, inode, 0, current_cmd);
 }
 
-/*
-Returns
--2: no file copied
--1: failed to copy some files
-0: all files has been copied
-*/
-#define MAX_DIR_NBR 256
-static int copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir, unsigned int *copy_ok, unsigned int *copy_bad)
+static copy_dir_t copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, const file_info_t *dir, unsigned int *copy_ok, unsigned int *copy_bad)
 {
   static unsigned int dir_nbr=0;
   static unsigned long int inode_known[MAX_DIR_NBR];
@@ -651,7 +687,7 @@ static int copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition,
   struct td_list_head *file_walker = NULL;
   TD_INIT_LIST_HEAD(&dir_list.list);
   if(dir_data->get_dir==NULL || dir_data->copy_file==NULL)
-    return 0;
+    return CD_FINISHED;
   inode_known[dir_nbr++]=dir->st_ino;
   dir_name=mkdir_local(dir_data->local_dir, dir_data->current_directory);
   dir_data->get_dir(disk, partition, dir_data, (const unsigned long int)dir->st_ino, &dir_list);
@@ -662,43 +698,40 @@ static int copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition,
     dir_data->current_directory[current_directory_namelength]='\0';
     if(current_directory_namelength+1+strlen(current_file->name)<sizeof(dir_data->current_directory)-1)
     {
-      int copy_stopped=0;
+      copy_dir_t copy_stopped=CD_FINISHED;
       if(strcmp(dir_data->current_directory,"/"))
 	strcat(dir_data->current_directory,"/");
       strcat(dir_data->current_directory,current_file->name);
       if(LINUX_S_ISDIR(current_file->st_mode)!=0)
       {
-	const unsigned long int new_inode=current_file->st_ino;
-	unsigned int new_inode_ok=1;
-	unsigned int i;
-	if(new_inode<2)
-	  new_inode_ok=0;
-	if(strcmp(current_file->name,"..")==0 || strcmp(current_file->name,".")==0)
-	  new_inode_ok=0;
-	for(i=0;i<dir_nbr && new_inode_ok!=0;i++)
-	  if(new_inode==inode_known[i]) /* Avoid loop */
-	    new_inode_ok=0;
-	if(new_inode_ok>0)
+	if(can_copy_dir(current_file, &inode_known[0], dir_nbr) > 0)
 	{
 	  copy_stopped=copy_dir(window, disk, partition, dir_data, current_file, copy_ok, copy_bad);
 	}
       }
       else if(LINUX_S_ISREG(current_file->st_mode)!=0)
       {
-	copy_stopped=copy_progress(window, *copy_ok, *copy_bad);
-	if(dir_data->copy_file(disk, partition, dir_data, current_file)==0)
-	  (*copy_ok)++;
+	if(copy_progress(window, *copy_ok, *copy_bad))
+	  copy_stopped=CD_STOPPED;
 	else
-	  (*copy_bad)++;
+	{
+	  const copy_file_t res=dir_data->copy_file(disk, partition, dir_data, current_file);
+	  if(res==CP_NOSPACE)
+	    copy_stopped=CD_NOSPACE;
+	  if(res==CP_OK)
+	    (*copy_ok)++;
+	  else
+	    (*copy_bad)++;
+	}
       }
-      if(copy_stopped)
+      if(copy_stopped != CD_FINISHED)
       {
 	dir_data->current_directory[current_directory_namelength]='\0';
 	delete_list_file(&dir_list);
 	set_date(dir_name, dir->td_atime, dir->td_mtime);
 	free(dir_name);
 	dir_nbr--;
-	return 1;
+	return copy_stopped;
       }
     }
   }
@@ -707,15 +740,16 @@ static int copy_dir(WINDOW *window, disk_t *disk, const partition_t *partition,
   set_date(dir_name, dir->td_atime, dir->td_mtime);
   free(dir_name);
   dir_nbr--;
-  return 0;
+  return CD_FINISHED;
 }
 
-static int copy_selection(file_info_t*dir_list, WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, unsigned int *copy_ok, unsigned int *copy_bad)
+static copy_dir_t copy_selection(file_info_t*dir_list, WINDOW *window, disk_t *disk, const partition_t *partition, dir_data_t *dir_data, unsigned int *copy_ok, unsigned int *copy_bad)
 {
   const unsigned int current_directory_namelength=strlen(dir_data->current_directory);
   struct td_list_head *tmpw=NULL;
   td_list_for_each(tmpw, &dir_list->list)
   {
+    copy_dir_t copy_stopped=CD_FINISHED;
     file_info_t *tmp=td_list_entry(tmpw, file_info_t, list);
     if((tmp->status&FILE_STATUS_MARKED)!=0 &&
 	current_directory_namelength + 1 + strlen(tmp->name) <
@@ -728,23 +762,30 @@ static int copy_selection(file_info_t*dir_list, WINDOW *window, disk_t *disk, co
 	strcat(dir_data->current_directory,tmp->name);
       if(LINUX_S_ISDIR(tmp->st_mode)!=0)
       {
-	if(copy_dir(window, disk, partition, dir_data, tmp, copy_ok, copy_bad))
+	copy_stopped=copy_dir(window, disk, partition, dir_data, tmp, copy_ok, copy_bad);
+	if(copy_stopped!=CD_FINISHED)
 	{
 	  dir_data->current_directory[current_directory_namelength]='\0';
-	  return 1;
+	  return copy_stopped;
 	}
       }
       else if(LINUX_S_ISREG(tmp->st_mode)!=0)
       {
+	copy_file_t res;
 	copy_progress(window, *copy_ok, *copy_bad);
-	if(dir_data->copy_file(disk, partition, dir_data, tmp) == 0)
+	res=dir_data->copy_file(disk, partition, dir_data, tmp);
+	if(res==CP_NOSPACE)
+	  copy_stopped=CD_NOSPACE;
+	if(res == CP_OK)
 	  (*copy_ok)++;
 	else
 	  (*copy_bad)++;
       }
     }
     dir_data->current_directory[current_directory_namelength]='\0';
+    if(copy_stopped!=CD_FINISHED)
+      return copy_stopped;
   }
-  return 0;
+  return CD_FINISHED;
 }
 #endif
