@@ -50,14 +50,14 @@
 #include "log.h"
 #include "log_part.h"
 
-static dir_partition_t dir_partition_init(disk_t *disk, const partition_t *partition, const int verbose, dir_data_t *dir_data)
+static dir_partition_t dir_partition_init(disk_t *disk, const partition_t *partition, const int verbose, const int expert, dir_data_t *dir_data)
 {
   dir_partition_t res=DIR_PART_ENOIMP;
   if(is_part_fat(partition))
     res=dir_partition_fat_init(disk, partition, dir_data, verbose);
   else if(is_part_ntfs(partition))
   {
-    res=dir_partition_ntfs_init(disk, partition, dir_data, verbose);
+    res=dir_partition_ntfs_init(disk, partition, dir_data, verbose, expert);
     if(res!=DIR_PART_OK)
       res=dir_partition_exfat_init(disk, partition, dir_data, verbose);
   }
@@ -84,7 +84,7 @@ static dir_partition_t dir_partition_init(disk_t *disk, const partition_t *parti
     case UP_RFS3:
       return dir_partition_reiser_init(disk, partition, dir_data, verbose);
     case UP_NTFS:
-      return dir_partition_ntfs_init(disk, partition, dir_data, verbose);
+      return dir_partition_ntfs_init(disk, partition, dir_data, verbose, expert);
     case UP_EXFAT:
       return dir_partition_exfat_init(disk, partition, dir_data, verbose);
     default:
@@ -92,7 +92,7 @@ static dir_partition_t dir_partition_init(disk_t *disk, const partition_t *parti
   }
 }
 
-dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const int verbose, char **current_cmd)
+dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const int verbose, const int expert, char **current_cmd)
 {
   dir_data_t dir_data;
 #ifdef HAVE_NCURSES
@@ -101,7 +101,7 @@ dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const
   dir_partition_t res;
   fflush(stderr);
   dir_data.local_dir=NULL;
-  res=dir_partition_init(disk, partition, verbose, &dir_data);
+  res=dir_partition_init(disk, partition, verbose, expert, &dir_data);
 #ifdef HAVE_NCURSES
   window=newwin(LINES, COLS, 0, 0);	/* full screen */
   dir_data.display=window;
@@ -122,7 +122,7 @@ dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const
       log_partition(disk,partition);
       screen_buffer_add("Support for this filesystem hasn't been implemented.\n");
       screen_buffer_to_log();
-      if(*current_cmd==NULL)
+      if(current_cmd==NULL || *current_cmd==NULL)
       {
 #ifdef HAVE_NCURSES
 	screen_buffer_display(window,"",NULL);
@@ -137,9 +137,9 @@ dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const
       aff_part(window,AFF_PART_ORDER|AFF_PART_STATUS,disk,partition);
 #endif
       log_partition(disk,partition);
-      screen_buffer_add("Support for this filesystem hasn't been enable during compilation.\n");
+      screen_buffer_add("Support for this filesystem wasn't enabled during compilation.\n");
       screen_buffer_to_log();
-      if(*current_cmd==NULL)
+      if(current_cmd==NULL || *current_cmd==NULL)
       {
 #ifdef HAVE_NCURSES
 	screen_buffer_display(window,"",NULL);
@@ -156,7 +156,7 @@ dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const
       log_partition(disk,partition);
       screen_buffer_add("Can't open filesystem. Filesystem seems damaged.\n");
       screen_buffer_to_log();
-      if(*current_cmd==NULL)
+      if(current_cmd==NULL || *current_cmd==NULL)
       {
 #ifdef HAVE_NCURSES
 	screen_buffer_display(window,"",NULL);
@@ -166,26 +166,29 @@ dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const
     case DIR_PART_OK:
       {
 	int recursive=0;
-	if(*current_cmd!=NULL)
+	int copy_files=0;
+	if(current_cmd!=NULL && *current_cmd!=NULL)
 	{
 	  int do_continue;
 	  do
 	  {
 	    do_continue=0;
-	    while(*current_cmd[0]==',')
-	      (*current_cmd)++;
-	    if(strncmp(*current_cmd,"recursive",9)==0)
+	    skip_comma_in_command(current_cmd);
+	    if(check_command(current_cmd,"recursive",9)==0)
 	    {
-	      (*current_cmd)+=9;
 	      recursive=1;
 	      do_continue=1;
 	    }
-	    else if(strncmp(*current_cmd,"fullpathname",12)==0)
+	    else if(check_command(current_cmd,"fullpathname",12)==0)
 	    {
-	      (*current_cmd)+=12;
 	      dir_data.param|=FLAG_LIST_PATHNAME;
 	      do_continue=1;
 	    }
+	    else if(check_command(current_cmd, "filecopy", 8)==0)
+	    {
+	      copy_files=1;
+	      do_continue=1;
+	    }
 	  } while(do_continue==1);
 	}
 	if(recursive>0)
@@ -195,17 +198,22 @@ dir_partition_t dir_partition(disk_t *disk, const partition_t *partition, const
 #ifdef HAVE_NCURSES
 	  dir_partition_aff(disk, partition, &dir_data, dir_data.current_inode, current_cmd);
 #else
+	  if(dir_data.verbose>0)
 	  {
-	    file_info_t dir_list = {
-	      .list = TD_LIST_HEAD_INIT(dir_list.list),
-	      .name = NULL
-	    };
+	    log_info("\ndir_partition inode=%lu\n", dir_data.current_inode);
+	    log_partition(disk, partition);
+	  }
+	  {
+	    file_info_t dir_list;
+	    TD_INIT_LIST_HEAD(&dir_list.list);
 	    dir_data.get_dir(disk, partition, &dir_data, dir_data.current_inode, &dir_list);
 	    dir_aff_log(&dir_data, &dir_list);
 	    delete_list_file(&dir_list);
 	  }
 #endif
 	}
+	if(copy_files>0)
+	  dir_whole_partition_copy(disk,partition,&dir_data,dir_data.current_inode);
 	dir_data.close(&dir_data);
       }
       break;
