@@ -19,6 +19,17 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _EXT2_INC_H
+#define _EXT2_INC_H
+
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_LIBEXT2FS
+#endif
+
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 #if defined(HAVE_LIBEXT2FS)
 struct ext2_dir_struct {
 	file_info_t *dir_list;
@@ -27,3 +38,8 @@ struct ext2_dir_struct {
 	dir_data_t *dir_data;
 };
 #endif
+
+#ifdef __cplusplus
+} /* closing brace for extern "C" */
+#endif
+#endif
