@@ -19,12 +19,24 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _EXT2_SB_H
+#define _EXT2_SB_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-int interface_superblock(disk_t *disk_car,list_part_t *list_part,char**current_cmd);
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires valid_list_part(list_part);
+  @ requires \valid(current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @ requires separation: \separated(disk_car, list_part, current_cmd);
+  @ ensures  valid_read_string(*current_cmd);
+  @*/
+int interface_superblock(disk_t *disk_car, const list_part_t *list_part, char**current_cmd);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
