@@ -55,19 +55,50 @@ struct fat_dir_struct
   struct fat_boot_sector*boot_sector;
 };
 
-
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires \valid_read(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid_read(dir_data);
+  @ requires \valid_read(fat_header);
+  @ requires \valid(dir_list);
+  @ requires \separated(disk_car, partition, dir_data, fat_header, dir_list);
+  @ decreases 0;
+  @*/
 static int fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const dir_data_t *dir_data, const struct fat_boot_sector*fat_header, file_info_t *dir_list);
-static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster, file_info_t *dir_list);
-static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len);
-static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file);
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires \valid_read(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid_read(dir_data);
+  @ requires \valid(file);
+  @ requires \separated(disk_car, partition, dir_data, file);
+  @*/
+static copy_file_t fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file);
+
+/*@
+  @ requires \valid(dir_data);
+  @*/
 static void dir_partition_fat_close(dir_data_t *dir_data);
 
+/*@
+  @ requires len > 0;
+  @ requires \valid_read(src + (0 .. 2*len-1));
+  @ requires \valid((char *)dst + (0 .. 2*len-1));
+  @*/
 static inline void fat16_towchar(wchar_t *dst, const uint8_t *src, size_t len)
 {
-	while (len--) {
-		*dst++ = src[0] | (src[1] << 8);
-		src += 2;
-	}
+  /*@
+    @ loop assigns len, *dst, dst, src;
+    @ loop variant len;
+    @*/
+  while (len--) {
+    *dst++ = src[0] | (src[1] << 8);
+    src += 2;
+  }
 }
 
 int dir_fat_aux(const unsigned char*buffer, const unsigned int size, const unsigned int param, file_info_t *dir_list)
@@ -82,12 +113,13 @@ int dir_fat_aux(const unsigned char*buffer, const unsigned int size, const unsig
   if(wctomb(NULL, 0) < 0)
     utf8=0;
 #endif
+#ifndef DISABLED_FOR_FRAMAC
 GetNew:
   status=0;
   long_slots = 0;
   unicode[0]=0;
   if (de->attr == ATTR_EXT &&
-      de->name[0] == (int8_t) DELETED_FLAG &&
+      de->name[0] == DELETED_FLAG &&
       (param & FLAG_LIST_DELETED)==FLAG_LIST_DELETED)
   {
     unsigned int i;
@@ -107,7 +139,7 @@ ParseLongDeleted:
       if((const void*)de>=(const void*)(buffer+size))
 	return 0;
       ds = (const struct msdos_dir_slot *) de;
-      if(de->name[0] != (int8_t) DELETED_FLAG)
+      if(de->name[0] != DELETED_FLAG)
 	goto GetNew;
       if (ds->attr !=  ATTR_EXT)
 	goto ParseLongDeletedNext;
@@ -235,7 +267,7 @@ RecEnd:
 	unicode[j++]=de->ext[i];
     }
     unicode[j]=0;
-    if(((int8_t) unicode[0] == (int8_t) DELETED_FLAG) &&
+    if((unicode[0] == DELETED_FLAG) &&
       ((param & FLAG_LIST_DELETED)==FLAG_LIST_DELETED) &&
       inode!=0 && de->name[1]!='\0' &&
       de->name[2]!='\0' && de->name[3]!='\0' &&
@@ -256,7 +288,7 @@ RecEnd:
   {
     if(unicode[0]==0)
       return 0;
-    if((int8_t) unicode[0] != (int8_t) DELETED_FLAG)
+    if(unicode[0] != DELETED_FLAG)
     {
       unsigned int i,o;
       file_info_t *new_file=(file_info_t *)MALLOC(sizeof(*new_file));
@@ -272,16 +304,16 @@ RecEnd:
 #endif
 	  if(sizec <= 0)
 	  {
-	    new_file->name[o++]=(char) unicode[i];
+	    new_file->name[o++]=unicode[i];
 	    utf8=0;
 	  }
 	  else
 	    o += sizec;
 	}
 	else
-	  new_file->name[o++]=(char) unicode[i];
+	  new_file->name[o++]=unicode[i];
       }
-      new_file->name[o]=0;
+      new_file->name[o]='\0';
       new_file->st_ino=inode;
       new_file->st_mode = MSDOS_MKMODE(de->attr,(LINUX_S_IRWXUGO & ~(LINUX_S_IWGRP|LINUX_S_IWOTH)));
       new_file->st_uid=0;
@@ -298,11 +330,16 @@ RecEnd:
   if((const void *)de<(const void *)(buffer+size-1) &&
       de->name[0] != (int8_t) 0)
     goto GetNew;
+#endif
   return 0;
 }
 
 typedef enum {FAT_FOLLOW_CLUSTER, FAT_NEXT_FREE_CLUSTER, FAT_NEXT_CLUSTER} fat_method_t;
 
+/*@
+  @ terminates \true;
+  @ assigns \nothing;
+  @*/
 static int is_EOC(const unsigned int cluster, const upart_type_t upart_type)
 {
   if(upart_type==UP_FAT12)
@@ -314,6 +351,17 @@ static int is_EOC(const unsigned int cluster, const upart_type_t upart_type)
 }
 
 #define NBR_ENTRIES_MAX 65536
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires \valid_read(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid_read(dir_data);
+  @ requires \valid(dir_list);
+  @ requires \separated(disk_car, partition, dir_data, dir_list);
+  @ decreases 0;
+  @*/
 static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const unsigned long int first_cluster, file_info_t *dir_list)
 {
   const struct fat_dir_struct *ls=(const struct fat_dir_struct*)dir_data->private_dir_data;
@@ -321,12 +369,16 @@ static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *d
   unsigned int cluster=first_cluster;
   if(fat_header->sectors_per_cluster<1)
   {
+#ifndef DISABLED_FOR_FRAMAC
     log_error("FAT: Can't list files, bad cluster size.\n");
+#endif
     return -1;
   }
   if(fat_sector_size(fat_header)==0)
   {
+#ifndef DISABLED_FOR_FRAMAC
     log_error("FAT: Can't list files, bad sector size.\n");
+#endif
     return -1;
   }
   if(cluster==0)
@@ -335,14 +387,18 @@ static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *d
       return fat1x_rootdir(disk_car, partition, dir_data, fat_header, dir_list);
     if(le32(fat_header->root_cluster)<2)
     {
+#ifndef DISABLED_FOR_FRAMAC
       log_error("FAT32: Can't list files, bad root cluster.\n");
+#endif
       return -1;
     }
     cluster=le32(fat_header->root_cluster);
   }
   if(get_next_cluster(disk_car, partition, partition->upart_type, le16(fat_header->reserved), cluster)==0)
   {
+#ifndef DISABLED_FOR_FRAMAC
     log_warning("FAT: Directory entry is marked as free.\n");
+#endif
   }
   {
     const unsigned int cluster_size=fat_header->sectors_per_cluster * fat_sector_size(fat_header);
@@ -365,11 +421,15 @@ static int fat_dir(disk_t *disk_car, const partition_t *partition, dir_data_t *d
       const uint64_t start=partition->part_offset+(uint64_t)(start_data+(cluster-2)*fat_header->sectors_per_cluster)*fat_sector_size(fat_header);
 //      if(dir_data->verbose>0)
       {
+#ifndef DISABLED_FOR_FRAMAC
         log_info("FAT: cluster=%u(0x%x), pos=%lu\n",cluster,cluster,(long unsigned)(start/fat_sector_size(fat_header)));
+#endif
       }
       if((unsigned)disk_car->pread(disk_car, buffer_dir + (uint64_t)cluster_size * nbr_cluster, cluster_size, start) != cluster_size)
       {
+#ifndef DISABLED_FOR_FRAMAC
 	log_error("FAT: Can't read directory cluster.\n");
+#endif
 	stop=1;
       }
       if(stop==0 && nbr_cluster==0 &&
@@ -423,10 +483,12 @@ static int fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const d
   const unsigned int root_size=(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size*disk_car->sector_size;
   if(root_size==0)
     return -1;
+#ifndef DISABLED_FOR_FRAMAC
   if(dir_data->verbose>1)
   {
     log_trace("fat1x_rootdir root_size=%u sectors\n",root_size/disk_car->sector_size);
   }
+#endif
   {
     int res;
     uint64_t start;
@@ -435,7 +497,9 @@ static int fat1x_rootdir(disk_t *disk_car, const partition_t *partition, const d
     start=partition->part_offset+(uint64_t)((le16(fat_header->reserved)+fat_header->fats*le16(fat_header->fat_length))*disk_car->sector_size);
     if((unsigned)disk_car->pread(disk_car, buffer_dir, root_size, start) != root_size)
     {
+#ifndef DISABLED_FOR_FRAMAC
       log_error("FAT 1x: Can't read root directory.\n");
+#endif
       /* Don't return yet, it may have been a partial read */
     }
     res=dir_fat_aux(buffer_dir, root_size, dir_data->param, dir_list);
@@ -452,7 +516,9 @@ dir_partition_t dir_partition_fat_init(disk_t *disk_car, const partition_t *part
   buffer=(unsigned char*)MALLOC(0x200);
   if(disk_car->pread(disk_car, buffer, 0x200, partition->part_offset) != 0x200)
   {
+#ifndef DISABLED_FOR_FRAMAC
     log_error("Can't read FAT boot sector.\n");
+#endif
     free(buffer);
     return DIR_PART_EIO;
   }
@@ -483,7 +549,17 @@ static void dir_partition_fat_close(dir_data_t *dir_data)
   free(ls);
 }
 
-static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file)
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires \valid_read(partition);
+  @ requires valid_partition(partition);
+  @ requires \valid(dir_data);
+  @ requires \valid_read(file);
+  @ requires \separated(disk_car, partition, dir_data, file);
+  @ decreases 0;
+  @*/
+static copy_file_t fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *dir_data, const file_info_t *file)
 {
   char *new_file;	
   FILE *f_out;
@@ -500,10 +576,12 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
   f_out=fopen_local(&new_file, dir_data->local_dir, dir_data->current_directory);
   if(!f_out)
   {
+#ifndef DISABLED_FOR_FRAMAC
     log_critical("Can't create file %s: \n",new_file);
+#endif
     free(new_file);
     free(buffer_file);
-    return -1;
+    return CP_CREATE_FAILED;
   }
   cluster = file->st_ino;
   fat_length=le16(fat_header->fat_length)>0?le16(fat_header->fat_length):le32(fat_header->fat32_length);
@@ -511,11 +589,16 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
   start_fat1=le16(fat_header->reserved);
   start_data=start_fat1+fat_header->fats*fat_length+(get_dir_entries(fat_header)*32+disk_car->sector_size-1)/disk_car->sector_size;
   no_of_cluster=(part_size-start_data)/sectors_per_cluster;
+#ifndef DISABLED_FOR_FRAMAC
   log_trace("fat_copy dst=%s first_cluster=%u (%llu) size=%lu\n", new_file,
       cluster,
       (long long unsigned)start_data+(cluster-2)*sectors_per_cluster,
       (long unsigned)file_size);
+#endif
 
+  /*@
+    @ loop variant file_size;
+    @*/
   while(cluster>=2 && cluster<=no_of_cluster+2 && file_size>0)
   {
     const uint64_t start=partition->part_offset+(uint64_t)(start_data+(cluster-2)*sectors_per_cluster)*fat_sector_size(fat_header);
@@ -524,16 +607,20 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
       toread = file_size;
     if((unsigned)disk_car->pread(disk_car, buffer_file, toread, start) != toread)
     {
+#ifndef DISABLED_FOR_FRAMAC
       log_error("fat_copy: Can't read cluster %u.\n", cluster);
+#endif
     }
     if(fwrite(buffer_file, 1, toread, f_out) != toread)
     {
+#ifndef DISABLED_FOR_FRAMAC
       log_error("fat_copy: failed to write data %s\n", strerror(errno));
+#endif
       fclose(f_out);
       set_date(new_file, file->td_atime, file->td_mtime);
       free(new_file);
       free(buffer_file);
-      return -1;
+      return CP_NOSPACE;
     }
     file_size -= toread;
     if(file_size>0)
@@ -561,5 +648,5 @@ static int fat_copy(disk_t *disk_car, const partition_t *partition, dir_data_t *
   set_date(new_file, file->td_atime, file->td_mtime);
   free(new_file);
   free(buffer_file);
-  return 0;
+  return CP_OK;
 }
