@@ -3,23 +3,24 @@
     File: file_3dm.c
 
     Copyright (C) 2013 Christophe GRENIER <grenier@cgsecurity.org>
-  
+
     This software is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
     the Free Software Foundation; either version 2 of the License, or
     (at your option) any later version.
-  
+
     This program is distributed in the hope that it will be useful,
     but WITHOUT ANY WARRANTY; without even the implied warranty of
     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
     GNU General Public License for more details.
-  
+
     You should have received a copy of the GNU General Public License along
     with this program; if not, write the Free Software Foundation, Inc., 51
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_3dm)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_3dm(file_stat_t *file_stat);
 
 const file_hint_t file_hint_3dm= {
@@ -41,9 +43,20 @@ const file_hint_t file_hint_3dm= {
   .register_header_check=&register_header_check_3dm
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_3dm, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ terminates \true;
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_3dm(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   unsigned int i;
+  /*@
+    @ loop assigns i;
+    @ loop variant buffer_size - i;
+    @*/
   for(i=24;
       i<buffer_size && buffer[i]==' ';
       i++);
@@ -58,3 +71,4 @@ static void register_header_check_3dm(file_stat_t *file_stat)
 {
   register_header_check(0, "3D Geometry File Format ", 24, &header_check_3dm, file_stat);
 }
+#endif
