@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_abr)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -32,7 +33,9 @@
 #include "filegen.h"
 #include "common.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_abr(file_stat_t *file_stat);
+
 struct abr_header
 {
   char magic[4];
@@ -49,13 +52,28 @@ const file_hint_t file_hint_abr= {
   .register_header_check=&register_header_check_abr
 };
 
+/*@
+  @ requires file_recovery->data_check==&data_check_abr;
+  @ requires valid_data_check_param(buffer, buffer_size, file_recovery);
+  @ ensures  valid_data_check_result(\result, file_recovery);
+  @ assigns  file_recovery->calculated_file_size;
+  @*/
 static data_check_t data_check_abr(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
+  /*@ assert file_recovery->calculated_file_size <= PHOTOREC_MAX_FILE_SIZE; */
+  /*@ assert file_recovery->file_size <= PHOTOREC_MAX_FILE_SIZE; */
+  /*@
+    @ loop invariant buffer_size <= 2 * PHOTOREC_MAX_BLOCKSIZE;
+    @ loop assigns file_recovery->calculated_file_size;
+    @ loop variant file_recovery->file_size + buffer_size/2 - (file_recovery->calculated_file_size + 12);
+    @*/
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + 12 < file_recovery->file_size + buffer_size/2)
   {
-    const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
+    const unsigned int i=file_recovery->calculated_file_size + buffer_size/2 - file_recovery->file_size;
+    /*@ assert 0 <= i < buffer_size - 12 ; */
     const struct abr_header *hdr=(const struct abr_header*)&buffer[i];
+    /*@ assert \valid_read(hdr); */
     if(memcmp(hdr->magic, "8BIM", 4)!=0)
       return DC_STOP;
     file_recovery->calculated_file_size+=(uint64_t)12 + be32(hdr->size);
@@ -63,11 +81,22 @@ static data_check_t data_check_abr(const unsigned char *buffer, const unsigned i
   return DC_CONTINUE;
 }
 
+/*@
+  @ requires buffer_size >= 4 + sizeof(struct abr_header) ;
+  @ requires separation: \separated(&file_hint_abr, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns *file_recovery_new;
+  @*/
 static int header_check_abr(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct abr_header *hdr=(const struct abr_header*)&buffer[4];
   uint64_t i=4;
   assert(buffer_size >= 12);
+  /*@
+    @ loop assigns i;
+    @ loop variant 512 - 12 - i;
+    @*/
   while(i < buffer_size - 12 && i < 512 - 12)
   {
     const struct abr_header *h=(const struct abr_header*)&buffer[i];
@@ -94,3 +123,4 @@ static void register_header_check_abr(file_stat_t *file_stat)
   };
   register_header_check(2, abr_header, sizeof(abr_header), &header_check_abr, file_stat);
 }
+#endif
