@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_aif)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,6 +32,7 @@
 #include "filegen.h"
 #include "common.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_aif(file_stat_t *file_stat);
 
 const file_hint_t file_hint_aif= {
@@ -49,6 +51,13 @@ struct aif_header
   char formType[4];
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ requires buffer_size >= 16;
+  @ requires separation: \separated(&file_hint_aif, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_aif(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct aif_header *hdr=(const struct aif_header *)buffer;
@@ -79,3 +88,4 @@ static void register_header_check_aif(file_stat_t *file_stat)
 {
   register_header_check(0, "FORM", 4, &header_check_aif, file_stat);
 }
+#endif
