@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_als)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_als(file_stat_t *file_stat);
 
 const file_hint_t file_hint_als= {
@@ -41,6 +43,14 @@ const file_hint_t file_hint_als= {
   .register_header_check=&register_header_check_als
 };
 
+/*@
+  @ requires file_recovery->file_check == &file_check_als;
+  @ requires \separated(file_recovery, file_recovery->handle, file_recovery->extension, &errno, &Frama_C_entropy_source);
+  @ requires valid_file_check_param(file_recovery);
+  @ ensures  valid_file_check_result(file_recovery);
+  @ assigns *file_recovery->handle, errno, file_recovery->file_size;
+  @ assigns Frama_C_entropy_source;
+  @*/
 static void file_check_als(file_recovery_t *file_recovery)
 {
   static const unsigned char als_footer[0x16]= {
@@ -51,6 +61,13 @@ static void file_check_als(file_recovery_t *file_recovery)
   file_search_footer(file_recovery, als_footer, sizeof(als_footer), 7);
 }
 
+/*@
+  @ requires buffer_size >= 11+13;
+  @ requires separation: \separated(&file_hint_als, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_als(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   static const unsigned char als_header2[13]= {
@@ -76,3 +93,4 @@ static void register_header_check_als(file_stat_t *file_stat)
   };
   register_header_check(0, als_header,sizeof(als_header), &header_check_als, file_stat);
 }
+#endif
