@@ -19,6 +19,7 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_apple)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -29,6 +30,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_apple(file_stat_t *file_stat);
 
 const file_hint_t file_hint_apple= {
@@ -40,6 +42,12 @@ const file_hint_t file_hint_apple= {
   .register_header_check=&register_header_check_apple
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_apple, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_apple(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -54,3 +62,4 @@ static void register_header_check_apple(file_stat_t *file_stat)
   };
   register_header_check(0, apple_header,sizeof(apple_header), &header_check_apple, file_stat);
 }
+#endif
