@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_chm)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_chm(file_stat_t *file_stat);
 
 const file_hint_t file_hint_chm= {
@@ -41,6 +43,13 @@ const file_hint_t file_hint_chm= {
   .register_header_check=&register_header_check_chm
 };
 
+/*@
+  @ requires buffer_size >= 108;
+  @ requires separation: \separated(&file_hint_chm, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_chm(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const uint64_t size=(buffer[104]<<0)+(buffer[105]<<8)+(buffer[106]<<16)+((uint64_t)buffer[107]<<24);
@@ -60,3 +69,4 @@ static void register_header_check_chm(file_stat_t *file_stat)
     0x60, 0x00, 0x00, 0x00, 0x01, 0x00, 0x00, 0x00};
   register_header_check(0, chm_header,sizeof(chm_header), &header_check_chm, file_stat);
 }
+#endif
