@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_cm)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,8 +31,8 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_cm(file_stat_t *file_stat);
-static int header_check_cm(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_cm= {
   .extension="comicdoc",
@@ -42,6 +43,12 @@ const file_hint_t file_hint_cm= {
   .register_header_check=&register_header_check_cm
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_cm, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_cm(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -54,3 +61,4 @@ static void register_header_check_cm(file_stat_t *file_stat)
   static const unsigned char cm_header[8]	= { 'f','L','m','C','0','0','0','1'};
   register_header_check(0x0, cm_header,sizeof(cm_header), &header_check_cm, file_stat);
 }
+#endif
