@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_dbn)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,6 +32,7 @@
 #include "filegen.h"
 #include "memmem.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_dbn(file_stat_t *file_stat);
 
 const file_hint_t file_hint_dbn= {
@@ -42,6 +44,13 @@ const file_hint_t file_hint_dbn= {
   .register_header_check=&register_header_check_dbn
 };
 
+/*@
+  @ requires buffer_size >= 512;
+  @ requires separation: \separated(&file_hint_dbn, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_dbn(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   if(td_memmem(buffer, 512, "[HEADER]", 8)==NULL)
@@ -59,3 +68,4 @@ static void register_header_check_dbn(file_stat_t *file_stat)
   };
   register_header_check(0, dbn_header, sizeof(dbn_header), &header_check_dbn, file_stat);
 }
+#endif
