@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_dump)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -32,6 +33,7 @@
 #include "filegen.h"
 #define TS_TAPE         1       /* dump tape header */
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_dump(file_stat_t *file_stat);
 
 const file_hint_t file_hint_dump= {
@@ -123,6 +125,13 @@ struct	dump_struct
 #define TS_CLRI 	6	/* map of inodes deleted since last dump */
 #define TS_END  	5	/* end of volume marker */
 
+/*@
+  @ requires buffer_size >= sizeof(struct dump_struct);
+  @ requires separation: \separated(&file_hint_dump, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_dump(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct dump_struct *dump=(const struct dump_struct*)buffer;
@@ -143,5 +152,8 @@ static void register_header_check_dump(file_stat_t *file_stat)
   static const unsigned char dump_header_le_old_fs[4]  = { 0x6b, 0xea, 0x00, 0x00};
   static const unsigned char dump_header_le_new_fs[4]  = { 0x6c, 0xea, 0x00, 0x00};
   register_header_check(0x18, dump_header_le_old_fs,sizeof(dump_header_le_old_fs), &header_check_dump, file_stat);
+#ifndef DISABLED_FOR_FRAMAC
   register_header_check(0x18, dump_header_le_new_fs,sizeof(dump_header_le_new_fs), &header_check_dump, file_stat);
+#endif
 }
+#endif
