@@ -19,6 +19,7 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_fcp)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -29,6 +30,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_fcp(file_stat_t *file_stat);
 
 const file_hint_t file_hint_fcp= {
@@ -46,6 +48,12 @@ const file_hint_t file_hint_fcp= {
    Mac Creator code: KeyG
 */
 
+/*@
+  @ requires separation: \separated(&file_hint_fcp, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_fcp(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -58,3 +66,4 @@ static void register_header_check_fcp(file_stat_t *file_stat)
   static const unsigned char fcp_header[5]= { 0xA2, 'K','e','y','G'};
   register_header_check(0, fcp_header,sizeof(fcp_header), &header_check_fcp, file_stat);
 }
+#endif
