@@ -22,6 +22,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_fh10)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -32,6 +33,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_fh10(file_stat_t *file_stat);
 
 const file_hint_t file_hint_fh10= {
@@ -43,6 +45,12 @@ const file_hint_t file_hint_fh10= {
   .register_header_check=&register_header_check_fh10
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_fh10, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_fh10(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -64,3 +72,4 @@ static void register_header_check_fh10(file_stat_t *file_stat)
   };
   register_header_check(0, fh10_header,sizeof(fh10_header), &header_check_fh10, file_stat);
 }
+#endif
