@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_gm6)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_gm6(file_stat_t *file_stat);
 
 const file_hint_t file_hint_gm6= {
@@ -42,6 +44,12 @@ const file_hint_t file_hint_gm6= {
 };
 
 //Version 8.1 file (.gm81)
+/*@
+  @ requires separation: \separated(&file_hint_gm6, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_gm81(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -50,6 +58,12 @@ static int header_check_gm81(const unsigned char *buffer, const unsigned int buf
 }
 
 //Version 7.0-8.0 file (.gmk)
+/*@
+  @ requires separation: \separated(&file_hint_gm6, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_gmk(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -58,6 +72,12 @@ static int header_check_gmk(const unsigned char *buffer, const unsigned int buff
 }
 
 //Version 6.0-6.1 file (.gm6)
+/*@
+  @ requires separation: \separated(&file_hint_gm6, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_gm6(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -66,6 +86,12 @@ static int header_check_gm6(const unsigned char *buffer, const unsigned int buff
 }
 
 //Version 4.3-5.3A file (.gmd)
+/*@
+  @ requires separation: \separated(&file_hint_gm6, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_gmd(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -140,3 +166,4 @@ static void register_header_check_gm6(file_stat_t *file_stat)
   register_header_check(0, gm50_header, sizeof(gm50_header), &header_check_gmd, file_stat);
   register_header_check(0, gm43_header, sizeof(gm43_header), &header_check_gmd, file_stat);
 }
+#endif
