@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_gp5)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_gp5(file_stat_t *file_stat);
 
 const file_hint_t file_hint_gp5= {
@@ -41,6 +43,13 @@ const file_hint_t file_hint_gp5= {
   .register_header_check=&register_header_check_gp5
 };
 
+/*@
+  @ requires file_recovery->file_check == &file_check_gp5;
+  @ requires valid_file_check_param(file_recovery);
+  @ ensures  valid_file_check_result(file_recovery);
+  @ assigns *file_recovery->handle, errno, file_recovery->file_size;
+  @ assigns Frama_C_entropy_source;
+  @*/
 static void file_check_gp5(file_recovery_t *file_recovery)
 {
   const unsigned char gp5_footer[13]= {
@@ -50,6 +59,13 @@ static void file_check_gp5(file_recovery_t *file_recovery)
   file_search_footer(file_recovery, gp5_footer, sizeof(gp5_footer), 0);
 }
 
+/*@
+  @ requires buffer_size >= 23;
+  @ requires separation: \separated(&file_hint_gp5, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_gp5(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   if(buffer[22]!='.')
@@ -77,3 +93,4 @@ static void register_header_check_gp5(file_stat_t *file_stat)
   };
   register_header_check(0, gp5_header, sizeof(gp5_header), &header_check_gp5, file_stat);
 }
+#endif
