@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_gsm)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,8 +32,8 @@
 #include "filegen.h"
 #include "common.h"
 #include "log.h"
-#include "memmem.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_gsm(file_stat_t *file_stat);
 
 const file_hint_t file_hint_gsm= {
@@ -50,12 +51,26 @@ struct block_header
   unsigned char payload[32];
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ requires file_recovery->data_check==&data_check_gsm;
+  @ requires valid_data_check_param(buffer, buffer_size, file_recovery);
+  @ terminates \true;
+  @ ensures  valid_data_check_result(\result, file_recovery);
+  @ assigns file_recovery->calculated_file_size;
+  @*/
 static data_check_t data_check_gsm(const unsigned char *buffer, const unsigned int buffer_size, file_recovery_t *file_recovery)
 {
+  /*@ assert file_recovery->calculated_file_size <= PHOTOREC_MAX_FILE_SIZE; */
+  /*@ assert file_recovery->file_size <= PHOTOREC_MAX_FILE_SIZE; */
+  /*@
+    @ loop assigns file_recovery->calculated_file_size;
+    @ loop variant file_recovery->file_size + buffer_size/2 - (file_recovery->calculated_file_size + sizeof(struct block_header));
+    @*/
   while(file_recovery->calculated_file_size + buffer_size/2  >= file_recovery->file_size &&
       file_recovery->calculated_file_size + sizeof(struct block_header) < file_recovery->file_size + buffer_size/2)
   {
-    const unsigned int i=file_recovery->calculated_file_size - file_recovery->file_size + buffer_size/2;
+    const unsigned int i=file_recovery->calculated_file_size + buffer_size/2 - file_recovery->file_size;
+    /*@ assert 0 <= i < buffer_size - sizeof(struct block_header); */
     const struct block_header *hdr=(const struct block_header *)&buffer[i];
     if(hdr->marker < 0xd0 || hdr->marker > 0xdf)
       return DC_STOP;
@@ -64,22 +79,40 @@ static data_check_t data_check_gsm(const unsigned char *buffer, const unsigned i
   return DC_CONTINUE;
 }
 
+/*@
+  @ requires separation: \separated(&file_hint_gsm, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ terminates \true;
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @*/
 static int header_check_gsm(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  const struct block_header *hdr;
   unsigned int i=0;
-  for(i=0, hdr=(const struct block_header *)buffer;
-      i * sizeof(struct block_header) < file_recovery_new->blocksize;
-      i++, hdr++)
+  /*@ assert file_recovery_new->blocksize <= buffer_size; */
+  /*@
+    @ loop assigns i;
+    @ loop variant file_recovery_new->blocksize - (i+1) * sizeof(struct block_header);
+    @*/
+  for(i=0;
+      (i+1) * sizeof(struct block_header) <= file_recovery_new->blocksize;
+      i++)
   {
+    /*@ assert (i+1) * sizeof(struct block_header) <= file_recovery_new->blocksize; */
+    /*@ assert (i+1) * sizeof(struct block_header) <= buffer_size; */
+    /*@ assert \valid_read(buffer + (0 .. buffer_size-1)); */
+    /*@ assert \valid_read(buffer + (0 .. (i+1) * sizeof(struct block_header)-1)); */
+    const struct block_header *hdr=(const struct block_header *)&buffer[i*sizeof(struct block_header)];
+    /*@ assert \valid_read(hdr); */
     if(hdr->marker < 0xd0 || hdr->marker > 0xdf)
       return 0;
   }
   if(i<3)
     return 0;
-  if(file_recovery->file_stat!=NULL
-      &&  file_recovery->file_stat->file_hint==&file_hint_gsm)
+  if(file_recovery->file_stat!=NULL &&
+      file_recovery->file_check!=NULL &&
+      file_recovery->file_stat->file_hint==&file_hint_gsm)
   {
+    /*@ assert \valid_function(file_recovery->file_check); */
     header_ignored(file_recovery_new);
     return 0;
   }
@@ -112,6 +145,7 @@ static void register_header_check_gsm(file_stat_t *file_stat)
   static const unsigned char gsm_header16[1]={ 0xdf };
 
   register_header_check(0, gsm_header1, sizeof(gsm_header1), &header_check_gsm, file_stat);
+#ifndef DISABLED_FOR_FRAMAC
   register_header_check(0, gsm_header2, sizeof(gsm_header2), &header_check_gsm, file_stat);
   register_header_check(0, gsm_header3, sizeof(gsm_header3), &header_check_gsm, file_stat);
   register_header_check(0, gsm_header4, sizeof(gsm_header4), &header_check_gsm, file_stat);
@@ -127,4 +161,6 @@ static void register_header_check_gsm(file_stat_t *file_stat)
   register_header_check(0, gsm_header14, sizeof(gsm_header14), &header_check_gsm, file_stat);
   register_header_check(0, gsm_header15, sizeof(gsm_header15), &header_check_gsm, file_stat);
   register_header_check(0, gsm_header16, sizeof(gsm_header16), &header_check_gsm, file_stat);
+#endif
 }
+#endif
