@@ -19,13 +19,19 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _FILE_GZ_H
+#define _FILE_GZ_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ terminates \true;
+  @ assigns \result;
+  @*/
 const char*td_zlib_version(void);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
-
+#endif
