@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_hdf)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -37,6 +38,7 @@
 #include "log.h"
 #endif
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_hdf(file_stat_t *file_stat);
 
 const file_hint_t file_hint_hdf= {
@@ -62,57 +64,109 @@ struct dd_struct
   uint32_t	length;
 } __attribute__ ((gcc_struct, __packed__));
 
-static void file_check_hdf(file_recovery_t *file_recovery)
+/*@
+  @ requires \valid(handle);
+  @ requires \valid(dd_buf + (0 .. 65536 * sizeof(struct dd_struct)-1));
+  @ requires \separated(handle, dd_buf + (..), &errno, &Frama_C_entropy_source);
+  @ assigns *(dd_buf + (0 .. 65536 * sizeof(struct dd_struct)-1));
+  @ assigns Frama_C_entropy_source;
+  @ assigns *handle, errno;
+  @*/
+static uint64_t file_check_hdf_aux(FILE *handle, char *dd_buf)
 {
   uint64_t file_size=0;
   uint64_t offset_old;
   uint64_t offset=4;
-  struct dd_struct *dd=(struct dd_struct *)MALLOC(sizeof(struct dd_struct)*65536);
+  const struct dd_struct *dd=(const struct dd_struct *)dd_buf;
+  /*@
+    @ loop assigns file_size, offset_old, offset;
+    @ loop assigns *(dd_buf + (0 .. 65536 * sizeof(struct dd_struct)-1));
+    @ loop assigns Frama_C_entropy_source;
+    @ loop assigns *handle, errno;
+    @*/
   do
   {
-    struct ddh_struct ddh;
-    const struct dd_struct *p;
+    char ddh_buf[sizeof(struct ddh_struct)];
+    const struct ddh_struct *ddh=(const struct ddh_struct *)&ddh_buf;
     unsigned int i;
     unsigned int size;
-    if(my_fseek(file_recovery->handle, offset, SEEK_SET) < 0 ||
-	fread(&ddh, sizeof(ddh), 1, file_recovery->handle) !=1 ||
-	be16(ddh.size)==0 ||
-	fread(dd, sizeof(struct dd_struct)*be16(ddh.size), 1, file_recovery->handle) !=1)
+    if(my_fseek(handle, offset, SEEK_SET) < 0 ||
+	fread(&ddh_buf, sizeof(ddh_buf), 1, handle) !=1)
     {
-      free(dd);
-      file_recovery->file_size=0;
-      return ;
+      return 0;
     }
-    size=be16(ddh.size);
+#ifdef __FRAMAC__
+    Frama_C_make_unknown(&ddh_buf, sizeof(ddh_buf));
+#endif
+    size=be16(ddh->size);
+    /*@ assert 0 <= size < 65536; */
+    if(size==0 ||
+	fread(dd_buf, sizeof(struct dd_struct)*size, 1, handle) !=1)
+    {
+      return 0;
+    }
+#ifdef __FRAMAC__
+    Frama_C_make_unknown(dd_buf, sizeof(struct dd_struct)*size);
+#endif
     if(file_size < offset + sizeof(struct dd_struct) * size)
       file_size = offset + sizeof(struct dd_struct) * size;
 #ifdef DEBUG_HDF
-    log_info("size=%u next=%lu\n", size, be32(ddh.next));
+    log_info("size=%u next=%lu\n", size, be32(ddh->next));
 #endif
-    for(i=0, p=dd; i < size; i++,p++)
+    /*@
+      @ loop invariant 0 <= i <= size;
+      @ loop assigns i, file_size;
+      @ loop variant size - i;
+      @*/
+    for(i=0; i < size; i++)
     {
+      const struct dd_struct *p=&dd[i];
+      const unsigned int p_offset=be32(p->offset);
+      const unsigned int p_length=be32(p->length);
 #ifdef DEBUG_HDF
       log_info("tag=0x%04x, ref=%u, offset=%lu, length=%lu\n",
-	  be16(p->tag), be16(p->ref), be32(p->offset), be32(p->length));
+	  be16(p->tag), be16(p->ref), p_offset, p_length);
 #endif
-      if((unsigned)be32(p->offset)!=(unsigned)(-1) &&
-	file_size < (uint64_t)be32(p->offset) + (uint64_t)be32(p->length))
-	file_size = (uint64_t)be32(p->offset) + (uint64_t)be32(p->length);
+      if(p_offset!=0xffffffff &&
+	file_size < (uint64_t)p_offset + (uint64_t)p_length)
+	file_size = (uint64_t)p_offset + (uint64_t)p_length;
     }
     offset_old=offset;
-    offset=be32(ddh.next);
+    offset=be32(ddh->next);
   } while(offset > offset_old);
-  free(dd);
   file_size++;
+  return file_size;
+}
+
+/*@
+  @ requires \separated(file_recovery, file_recovery->handle, &errno, &Frama_C_entropy_source, &__fc_heap_status);
+  @ requires valid_file_check_param(file_recovery);
+  @ ensures  valid_file_check_result(file_recovery);
+  @*/
+static void file_check_hdf(file_recovery_t *file_recovery)
+{
+  uint64_t file_size;
+  char *dd;
+  dd=(char *)MALLOC(sizeof(struct dd_struct)*65536);
+  file_size = file_check_hdf_aux(file_recovery->handle, dd);
+  free(dd);
 #ifdef DEBUG_HDF
   log_info("file_size %llu\n", (long long unsigned)file_size);
 #endif
-  if(file_recovery->file_size < file_size)
+  if(file_recovery->file_size < file_size || file_size==0)
     file_recovery->file_size=0;
   else
     file_recovery->file_size = file_size;
 }
 
+/*@
+  @ requires buffer_size >= sizeof(struct ddh_struct);
+  @ requires separation: \separated(&file_hint_hdf, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ terminates \true;
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_hdf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct ddh_struct *ddh=(const struct ddh_struct *)&buffer[4];
@@ -129,3 +183,4 @@ static void register_header_check_hdf(file_stat_t *file_stat)
   static const unsigned char hdf_header[4]=  { 0x0e, 0x03, 0x13, 0x01};
   register_header_check(0, hdf_header, sizeof(hdf_header), &header_check_hdf, file_stat);
 }
+#endif
