@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_it)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_it(file_stat_t *file_stat);
 
 const file_hint_t file_hint_it= {
@@ -41,7 +43,7 @@ const file_hint_t file_hint_it= {
   .register_header_check=&register_header_check_it
 };
 
-/* http://schismtracker.org/wiki/ITTECH.TXT */
+/* https://github.com/schismtracker/schismtracker/wiki/ITTECH.TXT */
 struct impulse_header
 {
   uint32_t magic;
@@ -68,6 +70,13 @@ struct impulse_header
   char     Chnl_Vol[64];
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ requires buffer_size >= sizeof(struct impulse_header);
+  @ requires separation: \separated(&file_hint_it, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_it(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct impulse_header *header=(const struct impulse_header *)buffer;
@@ -82,3 +91,4 @@ static void register_header_check_it(file_stat_t *file_stat)
 {
   register_header_check(0, "IMPM", 4, &header_check_it, file_stat);
 }
+#endif
