@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_itunes)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_itunes(file_stat_t *file_stat);
 
 const file_hint_t file_hint_itunes= {
@@ -41,12 +43,18 @@ const file_hint_t file_hint_itunes= {
   .register_header_check=&register_header_check_itunes
 };
 
-
+/*@
+  @ requires buffer_size >= 12;
+  @ requires separation: \separated(&file_hint_itunes, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_itunes(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const uint64_t size= (uint64_t)buffer[8] +
     (((uint64_t)buffer[9])<<8) + (((uint64_t)buffer[10])<<16) + (((uint64_t)buffer[11])<<24);
-  if(size < 12)
+  if(size < 0x68)
     return 0;
   /* mhbd */
   reset_file_recovery(file_recovery_new);
@@ -63,3 +71,4 @@ static void register_header_check_itunes(file_stat_t *file_stat)
   static const unsigned char itunes_header[8]= {'m', 'h', 'b', 'd', 0x68, 0x00, 0x00, 0x00};
   register_header_check(0, itunes_header,sizeof(itunes_header), &header_check_itunes, file_stat);
 }
+#endif
