@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_ldf)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,9 +31,8 @@
 #include "types.h"
 #include "filegen.h"
 
-
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_ldf(file_stat_t *file_stat);
-static int header_check_ldf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_ldf= {
   .extension="ldf",
@@ -43,13 +43,13 @@ const file_hint_t file_hint_ldf= {
   .register_header_check=&register_header_check_ldf
 };
 
-static const unsigned char ldf_header[4]= { 0x01, 0x0f, 0x00, 0x00 };
-
-static void register_header_check_ldf(file_stat_t *file_stat)
-{
-  register_header_check(0, ldf_header,sizeof(ldf_header), &header_check_ldf, file_stat);
-}
-
+/*@
+  @ requires buffer_size >= 0x1c;
+  @ requires separation: \separated(&file_hint_ldf, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_ldf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   if(buffer[0x00]==0x01 && buffer[0x01]==0x0f && buffer[0x02]==0x00 && buffer[0x03]==0x00 &&
@@ -65,3 +65,10 @@ static int header_check_ldf(const unsigned char *buffer, const unsigned int buff
   }
   return 0;
 }
+
+static void register_header_check_ldf(file_stat_t *file_stat)
+{
+  static const unsigned char ldf_header[4]= { 0x01, 0x0f, 0x00, 0x00 };
+  register_header_check(0, ldf_header,sizeof(ldf_header), &header_check_ldf, file_stat);
+}
+#endif
