@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_lso)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_lso(file_stat_t *file_stat);
 
 const file_hint_t file_hint_lso= {
@@ -41,12 +43,25 @@ const file_hint_t file_hint_lso= {
   .register_header_check=&register_header_check_lso
 };
 
+/*@
+  @ requires file_recovery->file_check == &file_check_lso;
+  @ requires valid_file_check_param(file_recovery);
+  @ ensures  valid_file_check_result(file_recovery);
+  @ assigns *file_recovery->handle, errno, file_recovery->file_size;
+  @ assigns Frama_C_entropy_source;
+  @*/
 static void file_check_lso(file_recovery_t *file_recovery)
 {
   const unsigned char lso_footer[6]= {0xFF, 0xFF, 0xFF, 0x7F, 0x7F, 0x7F};
   file_search_footer(file_recovery, lso_footer, sizeof(lso_footer), 0x46);
 }
 
+/*@
+  @ requires separation: \separated(&file_hint_lso, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_lso(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -63,3 +78,4 @@ static void register_header_check_lso(file_stat_t *file_stat)
   };
   register_header_check(0, lso_header, sizeof(lso_header), &header_check_lso, file_stat);
 }
+#endif
