@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_luks)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,8 +32,9 @@
 #include "types.h"
 #include "filegen.h"
 #include "common.h"
-#include "luks.h"
+#include "luks_struct.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_luks(file_stat_t *file_stat);
 
 const file_hint_t file_hint_luks= {
@@ -44,14 +46,18 @@ const file_hint_t file_hint_luks= {
   .register_header_check=&register_header_check_luks
 };
 
-static const unsigned char luks_header[6]=  {
-  'L' , 'U' , 'K' , 'S' , 0xba, 0xbe
-};
-
+/*@
+  @ requires buffer_size >= sizeof(struct luks_phdr);
+  @ requires separation: \separated(&file_hint_luks, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_luks(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct luks_phdr *hdr=(const struct luks_phdr *)buffer;
-  if(be16(hdr->version)<1 || be16(hdr->version)>2)
+  const unsigned int version=be16(hdr->version);
+  if(version<1 || version>2)
     return 0;
   if(!isalpha(hdr->cipherName[0]))
     return 0;
@@ -63,5 +69,9 @@ static int header_check_luks(const unsigned char *buffer, const unsigned int buf
 
 static void register_header_check_luks(file_stat_t *file_stat)
 {
+  static const unsigned char luks_header[6]=  {
+    'L' , 'U' , 'K' , 'S' , 0xba, 0xbe
+  };
   register_header_check(0, luks_header, sizeof(luks_header), &header_check_luks, file_stat);
 }
+#endif
