@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_mus)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,34 +31,49 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_mus(file_stat_t *file_stat);
 
-const file_hint_t file_hint_mus= {
-  .extension="mus",
-  .description="Finale Music Score",
-  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .recover=1,
-  .enable_by_default=1,
-  .register_header_check=&register_header_check_mus
+const file_hint_t file_hint_mus = {
+  .extension = "mus",
+  .description = "Finale Music Score",
+  .max_filesize = PHOTOREC_MAX_FILE_SIZE,
+  .recover = 1,
+  .enable_by_default = 1,
+  .register_header_check = &register_header_check_mus
 };
 
+/*@
+  @ requires file_recovery->file_check == &file_check_mus;
+  @ requires valid_file_check_param(file_recovery);
+  @ ensures  valid_file_check_result(file_recovery);
+  @ assigns *file_recovery->handle, errno, file_recovery->file_size;
+  @ assigns Frama_C_entropy_source;
+  @*/
 static void file_check_mus(file_recovery_t *file_recovery)
 {
-  const unsigned char mus_footer[5]= {'-','^','e','n','d'};
+  const unsigned char mus_footer[5] = { '-', '^', 'e', 'n', 'd' };
   file_search_footer(file_recovery, mus_footer, sizeof(mus_footer), 0);
 }
 
+/*@
+  @ requires separation: \separated(&file_hint_mus, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_mus(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
-  file_recovery_new->min_filesize=18;
-  file_recovery_new->file_check=&file_check_mus;
-  file_recovery_new->extension=file_hint_mus.extension;
+  file_recovery_new->min_filesize = 18;
+  file_recovery_new->file_check = &file_check_mus;
+  file_recovery_new->extension = file_hint_mus.extension;
   return 1;
 }
 
 static void register_header_check_mus(file_stat_t *file_stat)
 {
-  static const unsigned char mus_header[18]  = { 'E','N','I','G','M','A',' ','B','I','N','A','R','Y',' ','F','I','L','E' };
-  register_header_check(0, mus_header,sizeof(mus_header), &header_check_mus, file_stat);
+  static const unsigned char mus_header[18] = { 'E', 'N', 'I', 'G', 'M', 'A', ' ', 'B', 'I', 'N', 'A', 'R', 'Y', ' ', 'F', 'I', 'L', 'E' };
+  register_header_check(0, mus_header, sizeof(mus_header), &header_check_mus, file_stat);
 }
+#endif
