@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_njx)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,10 +31,8 @@
 #include "types.h"
 #include "filegen.h"
 
-
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_njx(file_stat_t *file_stat);
-static int header_check_njx(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
-static void file_check_njx(file_recovery_t *file_recovery);
 
 const file_hint_t file_hint_njx= {
   .extension="njx",
@@ -44,13 +43,26 @@ const file_hint_t file_hint_njx= {
   .register_header_check=&register_header_check_njx
 };
 
-static const unsigned char njx_header[4]= {0x04, 'N', 'j', 0x0f};
-
-static void register_header_check_njx(file_stat_t *file_stat)
+/*@
+  @ requires file_recovery->file_check == &file_check_njx;
+  @ requires valid_file_check_param(file_recovery);
+  @ ensures  valid_file_check_result(file_recovery);
+  @ assigns *file_recovery->handle, errno, file_recovery->file_size;
+  @ assigns Frama_C_entropy_source;
+  @*/
+static void file_check_njx(file_recovery_t *file_recovery)
 {
-  register_header_check(0, njx_header,sizeof(njx_header), &header_check_njx, file_stat);
+  const unsigned char njx_footer[4]= {'N', 'J', '*', 0x04};
+  file_search_footer(file_recovery, njx_footer, sizeof(njx_footer), 0);
 }
 
+/*@
+  @ requires buffer_size >= 12;
+  @ requires separation: \separated(&file_hint_njx, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_njx(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   if(buffer[0]==0x04 && buffer[1]=='N' && buffer[2]=='j' && buffer[3]==0x0f &&
@@ -64,8 +76,9 @@ static int header_check_njx(const unsigned char *buffer, const unsigned int buff
   return 0;
 }
 
-static void file_check_njx(file_recovery_t *file_recovery)
+static void register_header_check_njx(file_stat_t *file_stat)
 {
-  const unsigned char njx_footer[4]= {'N', 'J', '*', 0x04};
-  file_search_footer(file_recovery, njx_footer, sizeof(njx_footer), 0);
+  static const unsigned char njx_header[4]= {0x04, 'N', 'j', 0x0f};
+  register_header_check(0, njx_header,sizeof(njx_header), &header_check_njx, file_stat);
 }
+#endif
