@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_paf)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_paf(file_stat_t *file_stat);
 
 const file_hint_t file_hint_paf= {
@@ -44,6 +46,12 @@ const file_hint_t file_hint_paf= {
 /* Personal Ancestral File Family Database is handled by
  * The Church of Jesus Christ of Latter-day Saints	*/
 
+/*@
+  @ requires separation: \separated(&file_hint_paf, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_paf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -56,3 +64,4 @@ static void register_header_check_paf(file_stat_t *file_stat)
   static const unsigned char paf_header[11]= {'5', '0', '0', '\0', '5', '0', '0', '\0', 'P','A','F'};
   register_header_check(0, paf_header,sizeof(paf_header), &header_check_paf, file_stat);
 }
+#endif
