@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_pfx)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_pfx(file_stat_t *file_stat);
 
 const file_hint_t file_hint_pfx= {
@@ -63,6 +65,13 @@ const file_hint_t file_hint_pfx= {
  *    Here, contentType=data
  */
 
+/*@
+  @ requires buffer_size >= 9;
+  @ requires separation: \separated(&file_hint_pfx, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_pfx(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   if(buffer[0]==0x30 && buffer[1]==0x82 &&
@@ -90,3 +99,4 @@ static void register_header_check_pfx(file_stat_t *file_stat)
   };
   register_header_check(11, pfx_header,sizeof(pfx_header), &header_check_pfx, file_stat);
 }
+#endif
