@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_plist)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,9 +31,12 @@
 #include "types.h"
 #include "filegen.h"
 
+#if !defined(SINGLE_FORMAT)
 extern const file_hint_t file_hint_qbb;
 extern const file_hint_t file_hint_sqlite;
+#endif
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_plist(file_stat_t *file_stat);
 
 const file_hint_t file_hint_plist= {
@@ -44,8 +48,14 @@ const file_hint_t file_hint_plist= {
   .register_header_check=&register_header_check_plist
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_plist, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @*/
 static int header_check_plist(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
+#if !defined(SINGLE_FORMAT)
   if(file_recovery->file_stat!=NULL)
   {
     if( file_recovery->file_stat->file_hint==&file_hint_qbb ||
@@ -55,6 +65,7 @@ static int header_check_plist(const unsigned char *buffer, const unsigned int bu
 	return 0;
     }
   }
+#endif
   reset_file_recovery(file_recovery_new);
   file_recovery_new->extension=file_hint_plist.extension;
   return 1;
@@ -64,3 +75,4 @@ static void register_header_check_plist(file_stat_t *file_stat)
 {
   register_header_check(0, "bplist00",  8,  &header_check_plist, file_stat);
 }
+#endif
