@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_pzh)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_pzh(file_stat_t *file_stat);
 
 /* Presto http://www.soft.es/ */
@@ -48,6 +50,11 @@ static const unsigned char pzh_header[10]=  {
   0x05, 0x03
 };
 
+/*@
+  @ requires file_recovery->file_rename==&file_rename_pzh;
+  @ requires valid_file_rename_param(file_recovery);
+  @ ensures  valid_file_rename_result(file_recovery);
+  @*/
 static void file_rename_pzh(file_recovery_t *file_recovery)
 {
   unsigned char buffer[512];
@@ -66,6 +73,12 @@ static void file_rename_pzh(file_recovery_t *file_recovery)
     file_rename(file_recovery, buffer, buffer_size, 0, "pzh", 0);
 }
 
+/*@
+  @ requires separation: \separated(&file_hint_pzh, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_pzh(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -79,3 +92,4 @@ static void register_header_check_pzh(file_stat_t *file_stat)
 {
   register_header_check(0x9c4, pzh_header, sizeof(pzh_header), &header_check_pzh, file_stat);
 }
+#endif
