@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_reg)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,6 +32,7 @@
 #include "common.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_reg(file_stat_t *file_stat);
 
 const file_hint_t file_hint_reg= {
@@ -69,6 +71,13 @@ struct rgdb_block
   uint32_t chksum;
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ requires buffer_size > sizeof(struct creg_file_header);
+  @ requires separation: \separated(&file_hint_reg, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_reg_9x(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct creg_file_header*header=(const struct creg_file_header*)buffer;
@@ -78,11 +87,11 @@ static int header_check_reg_9x(const unsigned char *buffer, const unsigned int b
     const struct rgdb_block*block=(const struct rgdb_block*)(buffer+le32(header->rgdb_offset));
     if(memcmp(block,"RGDB",4)!=0)
       return 0;
-    reset_file_recovery(file_recovery_new);
-    file_recovery_new->min_filesize=0x1000;
-    file_recovery_new->extension=file_hint_reg.extension;
-    return 1;
   }
+  reset_file_recovery(file_recovery_new);
+  file_recovery_new->min_filesize=0x1000;
+  file_recovery_new->extension=file_hint_reg.extension;
+  return 1;
 }
 
 struct regf_file_header
@@ -90,7 +99,7 @@ struct regf_file_header
   uint32_t signature;
   uint32_t primary_sequence_number;
   uint32_t secondary_sequence_number;
-  uint64_t modification_time;
+  int64_t  modification_time;
   uint32_t major_version;
   uint32_t minor_version;
   uint32_t file_type;
@@ -103,6 +112,13 @@ struct regf_file_header
   uint32_t xor_checksum;
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ requires buffer_size > sizeof(struct regf_file_header);
+  @ requires separation: \separated(&file_hint_reg, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_reg_nt(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct regf_file_header *header=(const struct regf_file_header*)buffer;
@@ -125,3 +141,4 @@ static void register_header_check_reg(file_stat_t *file_stat)
   register_header_check(0, reg_header_nt,sizeof(reg_header_nt), &header_check_reg_nt, file_stat);
   register_header_check(0, reg_header_9x,sizeof(reg_header_9x), &header_check_reg_9x, file_stat);
 }
+#endif
