@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_sib)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,6 +31,7 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_sib(file_stat_t *file_stat);
 
 const file_hint_t file_hint_sib= {
@@ -41,6 +43,12 @@ const file_hint_t file_hint_sib= {
   .register_header_check=&register_header_check_sib
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_sib, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_sib(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -53,3 +61,4 @@ static void register_header_check_sib(file_stat_t *file_stat)
   static const unsigned char sib_header[9]= {0x0F, 'S', 'I', 'B', 'E', 'L', 'I', 'U', 'S'};
   register_header_check(0, sib_header,sizeof(sib_header), &header_check_sib, file_stat);
 }
+#endif
