@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_sit)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,8 +32,8 @@
 #include "filegen.h"
 
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_sit(file_stat_t *file_stat);
-static int header_check_sit(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new);
 
 const file_hint_t file_hint_sit= {
   .extension="sit",
@@ -43,7 +44,12 @@ const file_hint_t file_hint_sit= {
   .register_header_check=&register_header_check_sit
 };
 
-
+/*@
+  @ requires separation: \separated(&file_hint_sit, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_sit(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
@@ -56,3 +62,4 @@ static void register_header_check_sit(file_stat_t *file_stat)
   static const unsigned char sit_header[14]  = { '7','1','0','0',' ','3','.','3','D',' ','7','1','0','0'};
   register_header_check(0, sit_header,sizeof(sit_header), &header_check_sit, file_stat);
 }
+#endif
