@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_ttf)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -31,6 +32,7 @@
 #include "common.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_ttf(file_stat_t *file_stat);
 
 const file_hint_t file_hint_ttf= {
@@ -43,7 +45,7 @@ const file_hint_t file_hint_ttf= {
 };
 
 /*
- * http://www.microsoft.com/typography/otspec/otff.htm
+ * https://docs.microsoft.com/en-us/typography/opentype/spec/otff
  */
 
 struct ttf_offset_table
@@ -63,41 +65,74 @@ struct ttf_table_directory
   uint32_t 	length; 	/* Length of this table. */
 };
 
+/*@
+  @ terminates \true;
+  @ assigns \nothing;
+  @*/
 static unsigned int td_ilog2(unsigned int v)
 {
   unsigned int l = 0;
+  /*@
+    @ loop assigns v,l;
+    @ loop unroll 16;
+    @ loop variant v;
+    @*/
   while(v >>= 1)
     l++;
   return l;
 }
 
+/*@
+  @ requires buffer_size >= sizeof(struct ttf_offset_table);
+  @ requires separation: \separated(&file_hint_ttf, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ terminates \true;
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_ttf(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   const struct ttf_offset_table *ttf=(const struct ttf_offset_table *)buffer;
-  unsigned int numTables;
-  numTables=be16(ttf->numTables);
+  const unsigned int numTables=be16(ttf->numTables);
+  const unsigned int entrySelector=be16(ttf->entrySelector);
+  const unsigned int searchRange=be16(ttf->searchRange);
+  const unsigned int rangeShift=be16(ttf->rangeShift);
+  if(numTables == 0)
+    return 0;
   /* searchRange 	(Maximum power of 2 <= numTables) x 16.
    * entrySelector 	Log2(maximum power of 2 <= numTables).
    * rangeShift 	NumTables x 16-searchRange.
    * */
-  if(td_ilog2(numTables) != (uint16_t)be16(ttf->entrySelector))
+  if(td_ilog2(numTables) != entrySelector)
     return 0;
-  if((16<<be16(ttf->entrySelector)) != be16(ttf->searchRange))
+  if((16<<entrySelector) != searchRange)
     return 0;
-  if(numTables * 16 != (unsigned int)be16(ttf->rangeShift)+be16(ttf->searchRange))
+  if(numTables * 16 != rangeShift+searchRange)
     return 0;
   reset_file_recovery(file_recovery_new);
   file_recovery_new->extension=file_hint_ttf.extension;
   if(sizeof(struct ttf_offset_table) + numTables * sizeof(struct ttf_table_directory)
       <= buffer_size)
   {
-    const struct ttf_table_directory*ttf_dir=(const struct ttf_table_directory*)(ttf+1);
+    /*@ assert sizeof(struct ttf_offset_table) + numTables * sizeof(struct ttf_table_directory) <= buffer_size; */
+    /*@ assert numTables * sizeof(struct ttf_table_directory) <= buffer_size - sizeof(struct ttf_offset_table); */
+    /*@ assert numTables <= (buffer_size - sizeof(struct ttf_offset_table)) / sizeof(struct ttf_table_directory); */
+    /*@ assert \valid_read(buffer + (0 .. buffer_size - 1)); */
+    /*@ assert \valid_read(buffer + (0 .. sizeof(struct ttf_offset_table) + numTables * sizeof(struct ttf_table_directory) - 1)); */
     uint64_t max_offset=0;
     unsigned int i;
-    for(i=0; i<numTables; i++,ttf_dir++)
+    const struct ttf_table_directory*ttf_dir=(const struct ttf_table_directory*)&buffer[sizeof(struct ttf_offset_table)];
+    /*@ assert \valid_read(ttf_dir + (0 .. numTables -1)); */
+    /*@
+      @ loop assigns i, max_offset;
+      @ loop variant numTables - i;
+      @*/
+    for(i=0; i<numTables; i++)
     {
-      /* | 0x3: align the end of the table */
-      const uint64_t new_offset=(be32(ttf_dir->offset) + be32(ttf_dir->length))|0x3;
+      /*@ assert 0 <= i < numTables; */
+      /*@ assert \valid_read(&ttf_dir[i]); */
+      /* Do not align the end of the table with "|0x3;"*/
+      const uint64_t new_offset=((uint64_t)be32(ttf_dir[i].offset) + be32(ttf_dir[i].length));
       if(max_offset < new_offset)
 	max_offset=new_offset;
     }
@@ -113,3 +148,4 @@ static void register_header_check_ttf(file_stat_t *file_stat)
   static const unsigned char header_ttf[5]= 	{0x00 , 0x01, 0x00, 0x00, 0x00};
   register_header_check(0, header_ttf, sizeof(header_ttf), &header_check_ttf, file_stat);
 }
+#endif
