@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_v2i)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,24 +31,32 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_v2i(file_stat_t *file_stat);
 
-const file_hint_t file_hint_v2i= {
-  .extension="v2i",
-  .description="v2i backup",
-  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .recover=1,
-  .enable_by_default=1,
-  .register_header_check=&register_header_check_v2i
+const file_hint_t file_hint_v2i = {
+  .extension = "v2i",
+  .description = "v2i backup",
+  .max_filesize = PHOTOREC_MAX_FILE_SIZE,
+  .recover = 1,
+  .enable_by_default = 1,
+  .register_header_check = &register_header_check_v2i
 };
 
+/*@
+  @ requires buffer_size >= 0x18 ;
+  @ requires separation: \separated(&file_hint_v2i, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_v2i(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
-  const unsigned char v2i_sign[4]={0x13, 0x04, 0x08, 0x00};
-  if(memcmp(&buffer[0x14], v2i_sign, sizeof(v2i_sign))==0)
+  const unsigned char v2i_sign[4] = { 0x13, 0x04, 0x08, 0x00 };
+  if(memcmp(&buffer[0x14], v2i_sign, sizeof(v2i_sign)) == 0)
   {
     reset_file_recovery(file_recovery_new);
-    file_recovery_new->extension=file_hint_v2i.extension;
+    file_recovery_new->extension = file_hint_v2i.extension;
     return 1;
   }
   return 0;
@@ -57,3 +66,4 @@ static void register_header_check_v2i(file_stat_t *file_stat)
 {
   register_header_check(0, "$CAN", 4, &header_check_v2i, file_stat);
 }
+#endif
