@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_wallet)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,28 +31,36 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_wallet(file_stat_t *file_stat);
 
-const file_hint_t file_hint_wallet= {
-  .extension="wallet",
-  .description="Armory bitcoin wallet",
-  .max_filesize=10*1024*1024,
-  .recover=1,
-  .enable_by_default=1,
-  .register_header_check=&register_header_check_wallet
+const file_hint_t file_hint_wallet = {
+  .extension = "wallet",
+  .description = "Armory bitcoin wallet",
+  .max_filesize = 10 * 1024 * 1024,
+  .recover = 1,
+  .enable_by_default = 1,
+  .register_header_check = &register_header_check_wallet
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_wallet, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_wallet(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
-  file_recovery_new->extension=file_hint_wallet.extension;
+  file_recovery_new->extension = file_hint_wallet.extension;
   return 1;
 }
 
 static void register_header_check_wallet(file_stat_t *file_stat)
 {
-  static const unsigned char wallet_header[8]=  {
-    0xba, 'W' , 'A' , 'L' , 'L' , 'E' , 'T' , 0x00
+  static const unsigned char wallet_header[8] = {
+    0xba, 'W', 'A', 'L', 'L', 'E', 'T', 0x00
   };
   register_header_check(0, wallet_header, sizeof(wallet_header), &header_check_wallet, file_stat);
 }
+#endif
