@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_wnk)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,26 +31,34 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_wnk(file_stat_t *file_stat);
 
-const file_hint_t file_hint_wnk= {
-  .extension="wnk",
-  .description="Wink",
-  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .recover=1,
-  .enable_by_default=1,
-  .register_header_check=&register_header_check_wnk
+const file_hint_t file_hint_wnk = {
+  .extension = "wnk",
+  .description = "Wink",
+  .max_filesize = PHOTOREC_MAX_FILE_SIZE,
+  .recover = 1,
+  .enable_by_default = 1,
+  .register_header_check = &register_header_check_wnk
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_wnk, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_wnk(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
-  file_recovery_new->extension=file_hint_wnk.extension;
+  file_recovery_new->extension = file_hint_wnk.extension;
   return 1;
 }
 
 static void register_header_check_wnk(file_stat_t *file_stat)
 {
-  static const unsigned char wnk_header[6]= {'R','P','K','W','I',0x02};
-  register_header_check(0, wnk_header,sizeof(wnk_header), &header_check_wnk, file_stat);
+  static const unsigned char wnk_header[6] = { 'R', 'P', 'K', 'W', 'I', 0x02 };
+  register_header_check(0, wnk_header, sizeof(wnk_header), &header_check_wnk, file_stat);
 }
+#endif
