@@ -20,6 +20,7 @@
 
  */
 
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_wpb)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -30,28 +31,36 @@
 #include "types.h"
 #include "filegen.h"
 
+/*@ requires valid_register_header_check(file_stat); */
 static void register_header_check_wpb(file_stat_t *file_stat);
 
-const file_hint_t file_hint_wpb= {
-  .extension="wpb",
-  .description="OpenCanvas",
-  .max_filesize=PHOTOREC_MAX_FILE_SIZE,
-  .recover=1,
-  .enable_by_default=1,
-  .register_header_check=&register_header_check_wpb
+const file_hint_t file_hint_wpb = {
+  .extension = "wpb",
+  .description = "OpenCanvas",
+  .max_filesize = PHOTOREC_MAX_FILE_SIZE,
+  .recover = 1,
+  .enable_by_default = 1,
+  .register_header_check = &register_header_check_wpb
 };
 
+/*@
+  @ requires separation: \separated(&file_hint_wpb, buffer+(..), file_recovery, file_recovery_new);
+  @ requires valid_header_check_param(buffer, buffer_size, safe_header_only, file_recovery, file_recovery_new);
+  @ ensures  valid_header_check_result(\result, file_recovery_new);
+  @ assigns  *file_recovery_new;
+  @*/
 static int header_check_wpb(const unsigned char *buffer, const unsigned int buffer_size, const unsigned int safe_header_only, const file_recovery_t *file_recovery, file_recovery_t *file_recovery_new)
 {
   reset_file_recovery(file_recovery_new);
-  file_recovery_new->extension=file_hint_wpb.extension;
+  file_recovery_new->extension = file_hint_wpb.extension;
   return 1;
 }
 
 static void register_header_check_wpb(file_stat_t *file_stat)
 {
-  static const unsigned char wpb_header[8]=  {
-    'W' , 'P' , 'B' , '2' , 0x00, 0x00, 0x00, 0x00
+  static const unsigned char wpb_header[8] = {
+    'W', 'P', 'B', '2', 0x00, 0x00, 0x00, 0x00
   };
   register_header_check(0, wpb_header, sizeof(wpb_header), &header_check_wpb, file_stat);
 }
+#endif
