@@ -19,12 +19,21 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _GEOMETRYN_H
+#define _GEOMETRYN_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifdef HAVE_NCURSES
+/*@
+  @ requires \valid(disk);
+  @ requires valid_disk(disk);
+  @*/
 int change_geometry_ncurses(disk_t *disk);
+#endif
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
