@@ -19,12 +19,20 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _HIDDEN_H
+#define _HIDDEN_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires \valid_read(disk);
+  @ requires valid_disk(disk);
+  @ assigns  \nothing;
+  @*/
 int is_hpa_or_dco(const disk_t *disk);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
