@@ -22,8 +22,12 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
-#ifdef HAVE_NCURSES
 
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_NCURSES
+#endif
+
+#ifdef HAVE_NCURSES
 #include <stdio.h>
 #include "types.h"
 #include "common.h"
