@@ -19,12 +19,21 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _HIDDENN_H
+#define _HIDDENN_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+#ifdef HAVE_NCURSES
+/*@
+  @ requires \valid(disk);
+  @ requires valid_disk(disk);
+  @*/
 int interface_check_hidden_ncurses(disk_t *disk, const int hpa_dco);
+#endif
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
