@@ -19,13 +19,28 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _INTRFACE_H
+#define _INTRFACE_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-list_part_t *ask_structure(disk_t *disk_car,list_part_t *list_part, const int verbose, char **current_cmd);
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_list_part(list_part);
+  @ requires valid_disk(disk_car);
+  @ requires \valid(current_cmd);
+  @ requires \separated(disk_car, list_part, current_cmd);
+  @*/
+list_part_t *ask_structure(disk_t *disk_car, list_part_t *list_part, const int verbose, char **current_cmd);
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @*/
 void interface_list(disk_t *disk_car, const int verbose, const int saveheader, const int backup);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
