@@ -37,27 +37,6 @@
 #include "log.h"
 #include "guid_cpy.h"
 
-static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb, const partition_t *partition, const int dump_ind);
-static void set_JFS_info(const struct jfs_superblock *sb, partition_t *partition);
-
-int check_JFS(disk_t *disk_car, partition_t *partition)
-{
-  unsigned char *buffer=(unsigned char*)MALLOC(JFS_SUPERBLOCK_SIZE);
-  if(disk_car->pread(disk_car, buffer, JFS_SUPERBLOCK_SIZE, partition->part_offset + 64 * 512) != JFS_SUPERBLOCK_SIZE)
-  {
-    free(buffer);
-    return 1;
-  }
-  if(test_JFS(disk_car, (struct jfs_superblock*)buffer, partition,0)!=0)
-  {
-    free(buffer);
-    return 1;
-  }
-  set_JFS_info((struct jfs_superblock*)buffer, partition);
-  free(buffer);
-  return 0;
-}
-
 static void set_JFS_info(const struct jfs_superblock *sb, partition_t *partition)
 {
   partition->upart_type=UP_JFS;
@@ -71,37 +50,7 @@ static void set_JFS_info(const struct jfs_superblock *sb, partition_t *partition
   }
 }
 
-/*
-Primary superblock is at 0x8000
-*/
-int recover_JFS(disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind)
-{
-  if(test_JFS(disk_car, sb, partition, dump_ind)!=0)
-    return 1;
-  set_JFS_info(sb, partition);
-  partition->part_type_i386=P_LINUX;
-  partition->part_type_sun=PSUN_LINUX;
-  partition->part_type_mac=PMAC_LINUX;
-  partition->part_type_gpt=GPT_ENT_TYPE_LINUX_DATA;
-  partition->part_size=(uint64_t)le32(sb->s_pbsize) * le64(sb->s_size) +
-    le32(sb->s_bsize) * (le24(sb->s_fsckpxd.len)+le24(sb->s_logpxd.len));
-  partition->sborg_offset=64*512;
-  partition->sb_size=JFS_SUPERBLOCK_SIZE;
-  partition->sb_offset=0;
-  guid_cpy(&partition->part_uuid, (const efi_guid_t *)&sb->s_uuid);
-  if(verbose>0)
-  {
-    log_info("\n");
-    log_info("recover_JFS: s_blocksize=%u\n",partition->blocksize);
-    log_info("recover_JFS: s_size %lu\n",(long unsigned int)le64(sb->s_size));
-    log_info("recover_JFS: s_fsckpxd.len:%d\n", (int)le24(sb->s_fsckpxd.len));
-    log_info("recover_JFS: s_logpxd.len:%d\n", (int)le24(sb->s_logpxd.len));
-    log_info("recover_JFS: part_size %lu\n",(long unsigned)(partition->part_size/disk_car->sector_size));
-  }
-  return 0;
-}
-
-static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb, const partition_t *partition, const int dump_ind)
+static int test_JFS(const disk_t *disk_car, const struct jfs_superblock *sb, const partition_t *partition, const int dump_ind)
 {
   if(memcmp(sb->s_magic,"JFS1",4)!=0)
     return 1;
@@ -124,3 +73,51 @@ static int test_JFS(disk_t *disk_car, const struct jfs_superblock *sb, const par
     */
   return 0;
 }
+
+int check_JFS(disk_t *disk_car, partition_t *partition)
+{
+  unsigned char *buffer=(unsigned char*)MALLOC(JFS_SUPERBLOCK_SIZE);
+  if(disk_car->pread(disk_car, buffer, JFS_SUPERBLOCK_SIZE, partition->part_offset + 64 * 512) != JFS_SUPERBLOCK_SIZE)
+  {
+    free(buffer);
+    return 1;
+  }
+  if(test_JFS(disk_car, (struct jfs_superblock*)buffer, partition,0)!=0)
+  {
+    free(buffer);
+    return 1;
+  }
+  set_JFS_info((struct jfs_superblock*)buffer, partition);
+  free(buffer);
+  return 0;
+}
+
+/*
+Primary superblock is at 0x8000
+*/
+int recover_JFS(const disk_t *disk_car, const struct jfs_superblock *sb,partition_t *partition,const int verbose, const int dump_ind)
+{
+  if(test_JFS(disk_car, sb, partition, dump_ind)!=0)
+    return 1;
+  set_JFS_info(sb, partition);
+  partition->part_type_i386=P_LINUX;
+  partition->part_type_sun=PSUN_LINUX;
+  partition->part_type_mac=PMAC_LINUX;
+  partition->part_type_gpt=GPT_ENT_TYPE_LINUX_DATA;
+  partition->part_size=(uint64_t)le32(sb->s_pbsize) * le64(sb->s_size) +
+    (uint64_t)le32(sb->s_bsize) * (le24(sb->s_fsckpxd.len)+le24(sb->s_logpxd.len));
+  partition->sborg_offset=64*512;
+  partition->sb_size=JFS_SUPERBLOCK_SIZE;
+  partition->sb_offset=0;
+  guid_cpy(&partition->part_uuid, (const efi_guid_t *)&sb->s_uuid);
+  if(verbose>0)
+  {
+    log_info("\n");
+    log_info("recover_JFS: s_blocksize=%u\n",partition->blocksize);
+    log_info("recover_JFS: s_size %lu\n",(long unsigned int)le64(sb->s_size));
+    log_info("recover_JFS: s_fsckpxd.len:%d\n", (int)le24(sb->s_fsckpxd.len));
+    log_info("recover_JFS: s_logpxd.len:%d\n", (int)le24(sb->s_logpxd.len));
+    log_info("recover_JFS: part_size %lu\n",(long unsigned)(partition->part_size/disk_car->sector_size));
+  }
+  return 0;
+}
