@@ -47,20 +47,30 @@
 #include "common.h"
 #include "log.h"
 
+#ifdef DISABLED_FOR_FRAMAC
+#undef HAVE_DUP2
+#endif
+
 static FILE *log_handle=NULL;
 static int f_status=0;
-static int log_handler(const char *_format, va_list ap) __attribute__((format(printf, 1, 0)));
+
 /* static unsigned int log_levels=LOG_LEVEL_DEBUG|LOG_LEVEL_TRACE|LOG_LEVEL_QUIET|LOG_LEVEL_INFO|LOG_LEVEL_VERBOSE|LOG_LEVEL_PROGRESS|LOG_LEVEL_WARNING|LOG_LEVEL_ERROR|LOG_LEVEL_PERROR|LOG_LEVEL_CRITICAL; */
 static unsigned int log_levels=LOG_LEVEL_TRACE|LOG_LEVEL_QUIET|LOG_LEVEL_INFO|LOG_LEVEL_VERBOSE|LOG_LEVEL_PROGRESS|LOG_LEVEL_WARNING|LOG_LEVEL_ERROR|LOG_LEVEL_PERROR|LOG_LEVEL_CRITICAL;
 
-int log_set_levels(const unsigned int levels)
+/*@ assigns log_levels; */
+unsigned int log_set_levels(const unsigned int levels)
 {
-  const int old_levels=log_levels;
+  const unsigned int old_levels=log_levels;
   log_levels=levels;
   return old_levels;
 }
 
-FILE *log_open(const char*default_filename, const int mode, int *errsv)
+/*@
+  @ requires separation: \separated(default_filename, errsv, log_handle, &errno);
+  @ assigns log_handle;
+  @ assigns \result,errno,*errsv;
+  @*/
+int log_open(const char*default_filename, const int mode, int *errsv)
 {
   log_handle=fopen(default_filename,(mode==TD_LOG_CREATE?"w":"a"));
   *errsv=errno;
@@ -76,16 +86,27 @@ FILE *log_open(const char*default_filename, const int mode, int *errsv)
     }
   }
 #endif
-  return log_handle;
+  if(log_handle==NULL)
+    return 0;
+#if defined(HAVE_DUP2)
+  dup2(fileno(log_handle),2);
+#endif
+  return 1;
 }
 
+/*@
+  @ requires separation: \separated(default_filename, errsv, log_handle, &errno);
+  @ assigns log_handle;
+  @ assigns \result,errno,*errsv,__fc_heap_status;
+  @*/
 #if defined(__CYGWIN__) || defined(__MINGW32__)
-FILE *log_open_default(const char*default_filename, const int mode, int *errsv)
+int log_open_default(const char*default_filename, const int mode, int *errsv)
 {
   char*filename;
   char *path;
+  int result;
   if(log_handle != NULL)
-    return log_handle;
+    return 1;
   path = getenv("USERPROFILE");
   if (path == NULL)
     path = getenv("HOMEPATH");
@@ -102,34 +123,50 @@ FILE *log_open_default(const char*default_filename, const int mode, int *errsv)
 #endif
   strcat(filename, "/");
   strcat(filename, default_filename);
-  log_open(filename, mode, errsv);
+  result=log_open(filename, mode, errsv);
   free(filename);
-  return log_handle;
+  return result;
 }
 #else
-FILE *log_open_default(const char*default_filename, const int mode, int *errsv)
+int log_open_default(const char*default_filename, const int mode, int *errsv)
 {
   char*filename;
   char *path;
-  FILE *handle;
+  int result;
   path = getenv("HOME");
   if(path == NULL)
     return log_open(default_filename, mode, errsv);
+  /*@ assert strlen(path)+strlen(default_filename)+2 < UINT_MAX; */
   filename=(char*)MALLOC(strlen(path)+strlen(default_filename)+2);
+#if defined(__FRAMAC__)
+  result=0;
+#else
   strcpy(filename, path);
   strcat(filename, "/");
   strcat(filename, default_filename);
-  handle=log_open(filename, mode, errsv);
+  result=log_open(filename, mode, errsv);
+#endif
   free(filename);
-  return handle;
+  return result;
 }
 #endif
 
+/*@
+  @ requires log_handle==\null || \valid(log_handle);
+  @*/
 int log_flush(void)
 {
   return fflush(log_handle);
 }
 
+/*@
+  @ requires \valid(log_handle);
+  @ requires valid_read_string(_format);
+  @ assigns f_status, *log_handle;
+  @*/
+// assigns *log_handle \from _format[..], ap;
+static int log_handler(const char *_format, va_list ap) __attribute__((format(printf, 1, 0)));
+
 static int log_handler(const char *_format, va_list ap)
 {
   int res;
@@ -149,6 +186,11 @@ static int log_handler(const char *_format, va_list ap)
   return res;
 }
 
+/*@
+  @ requires log_handle == \null || \valid(log_handle);
+  @ assigns \result,errno,log_handle;
+  @ assigns f_status;
+  @*/
 int log_close(void)
 {
   if(log_handle!=NULL)
@@ -160,7 +202,12 @@ int log_close(void)
   return f_status;
 }
 
-int log_redirect(unsigned int level, const char *format, ...)
+/*@
+  @ requires log_handle == \null || \valid(log_handle);
+  @ assigns *log_handle;
+  @ assigns f_status;
+  @*/
+int log_redirect(const unsigned int level, const char *format, ...)
 {
   if((log_levels & level)==0)
     return 0;
@@ -176,22 +223,36 @@ int log_redirect(unsigned int level, const char *format, ...)
   }
 }
 
-void dump_log(const void *nom_dump,unsigned int lng)
+void dump_log(const void *nom_dump, const unsigned int lng)
 {
-  unsigned int i,j;
-  unsigned int nbr_line;
-  unsigned char car;
-  nbr_line=(lng+0x10-1)/0x10;
+#ifndef DISABLED_FOR_FRAMAC
+  const char *ptr=(const char*)nom_dump;
+  const unsigned int nbr_line=(lng+0x10-1)/0x10;
+  unsigned int i;
+  /*@ assert \valid_read(ptr + (0 .. lng-1)); */
   /* write dump to log file*/
-  for (i=0; (i<nbr_line); i++)
+  /*@
+    @ loop invariant 0 <= i <= nbr_line;
+    @ loop assigns *log_handle, f_status, i;
+    @ loop variant nbr_line - i;
+    @*/
+  for (i=0; i<nbr_line; i++)
   {
+    unsigned int j;
     log_info("%04X ",i*0x10);
+    /*@
+      @ loop invariant 0 <= j <= 0x10;
+      @ loop assigns *log_handle, f_status, j;
+      @ loop variant 0x10 - j;
+      @*/
     for(j=0; j< 0x10;j++)
     {
-      if(i*0x10+j<lng)
+      const unsigned int o=i*0x10+j;
+      if(o<lng)
       {
-        car=*((const unsigned char *)nom_dump+i*0x10+j);
-        log_info("%02x", car);
+	/*@ assert o<lng; */
+        /*@ assert \valid_read(ptr + (0 .. lng-1)); */
+        log_info("%02x", ptr[o]);
       }
       else
         log_info("  ");
@@ -199,12 +260,18 @@ void dump_log(const void *nom_dump,unsigned int lng)
         log_info(" ");
     }
     log_info("  ");
+    /*@
+      @ loop invariant 0 <= j <= 0x10;
+      @ loop assigns *log_handle, f_status, j;
+      @ loop variant 0x10 - j;
+      @*/
     for(j=0; j< 0x10;j++)
     {
-      if(i*0x10+j<lng)
+      const unsigned int o=i*0x10+j;
+      if(o<lng)
       {
-        car=*((const unsigned char*)nom_dump+i*0x10+j);
-        if ((car<32)||(car >= 127))
+        const char car=ptr[o];
+        if (car<32 || car >= 127)
           log_info(".");
         else
           log_info("%c",  car);
@@ -214,23 +281,36 @@ void dump_log(const void *nom_dump,unsigned int lng)
     }
     log_info("\n");
   }
+#endif
 }
 
 void dump2_log(const void *dump_1, const void *dump_2, const unsigned int lng)
 {
+#ifndef DISABLED_FOR_FRAMAC
+  const char *ptr1=(const char*)dump_1;
+  const char *ptr2=(const char*)dump_2;
+  const unsigned int nbr_line=(lng+0x08-1)/0x08;
   unsigned int i,j;
-  unsigned int nbr_line;
-  nbr_line=(lng+0x08-1)/0x08;
   /* write dump to log file*/
-  for (i=0; (i<nbr_line); i++)
+  /*@
+    @ loop invariant 0 <= i <= nbr_line;
+    @ loop assigns *log_handle, f_status, i, j;
+    @ loop variant nbr_line - i;
+    @*/
+  for (i=0; i<nbr_line; i++)
   {
     log_info("%04X ",i*0x08);
+    /*@
+      @ loop invariant 0 <= j <= 8;
+      @ loop assigns *log_handle, f_status, j;
+      @ loop variant 0x8 - j;
+      @*/
     for(j=0; j<0x08;j++)
     {
-      if(i*0x08+j<lng)
+      const unsigned int o=i*0x08+j;
+      if(o<lng)
       {
-        unsigned char car=*((const unsigned char *)dump_1+i*0x08+j);
-        log_info("%02x", car);
+        log_info("%02x", ptr1[o]);
       }
       else
         log_info("  ");
@@ -238,12 +318,18 @@ void dump2_log(const void *dump_1, const void *dump_2, const unsigned int lng)
         log_info(" ");
     }
     log_info("  ");
+    /*@
+      @ loop invariant 0 <= j <= 8;
+      @ loop assigns *log_handle, f_status, j;
+      @ loop variant 0x8 - j;
+      @*/
     for(j=0; j<0x08;j++)
     {
-      if(i*0x08+j<lng)
+      const unsigned int o=i*0x08+j;
+      if(o<lng)
       {
-        unsigned char car=*((const unsigned char*)dump_1+i*0x08+j);
-        if ((car<32)||(car >= 127))
+        const char car=ptr1[o];
+        if (car<32 || car >= 127)
           log_info(".");
         else
           log_info("%c",  car);
@@ -252,12 +338,17 @@ void dump2_log(const void *dump_1, const void *dump_2, const unsigned int lng)
         log_info(" ");
     }
     log_info("  ");
+    /*@
+      @ loop invariant 0 <= j <= 8;
+      @ loop assigns *log_handle, f_status, j;
+      @ loop variant 0x8 - j;
+      @*/
     for(j=0; j<0x08;j++)
     {
-      if(i*0x08+j<lng)
+      const unsigned int o=i*0x08+j;
+      if(o<lng)
       {
-        unsigned char car=*((const unsigned char *)dump_2+i*0x08+j);
-        log_info("%02x", car);
+        log_info("%02x", ptr2[o]);
       }
       else
         log_info("  ");
@@ -265,12 +356,18 @@ void dump2_log(const void *dump_1, const void *dump_2, const unsigned int lng)
         log_info(" ");
     }
     log_info("  ");
+    /*@
+      @ loop invariant 0 <= j <= 8;
+      @ loop assigns *log_handle, f_status, j;
+      @ loop variant 0x8 - j;
+      @*/
     for(j=0; j<0x08;j++)
     {
-      if(i*0x08+j<lng)
+      const unsigned int o=i*0x08+j;
+      if(o<lng)
       {
-        unsigned char car=*((const unsigned char*)dump_2+i*0x08+j);
-        if ((car<32)||(car >= 127))
+        const char car=ptr2[o];
+        if (car<32 || car >= 127)
           log_info(".");
         else
           log_info("%c",  car);
@@ -280,4 +377,5 @@ void dump2_log(const void *dump_1, const void *dump_2, const unsigned int lng)
     }
     log_info("\n");
   }
+#endif
 }
