@@ -19,11 +19,15 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _NODISK_H
+#define _NODISK_H
 #ifdef __cplusplus
 extern "C" {
 #endif
+
 int intrf_no_disk_ncurses(const char *prog_name);
+
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
-
+#endif
