@@ -80,20 +80,28 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
   mft_pos=partition->part_offset+(uint64_t)(le16(ntfs_header->reserved)+le64(ntfs_header->mft_lcn)*ntfs_header->sectors_per_cluster)*ntfs_sector_size(ntfs_header);
   mftmirr_pos=partition->part_offset+(uint64_t)(le16(ntfs_header->reserved)+le64(ntfs_header->mftmirr_lcn)*ntfs_header->sectors_per_cluster)*ntfs_sector_size(ntfs_header);
   if(ntfs_header->clusters_per_mft_record>0)
-    mft_record_size=ntfs_header->sectors_per_cluster*ntfs_header->clusters_per_mft_record;
+    mft_record_size=ntfs_header->clusters_per_mft_record * ntfs_header->sectors_per_cluster * ntfs_sector_size(ntfs_header);
   else
     mft_record_size=1<<(-ntfs_header->clusters_per_mft_record);
 
-  cluster_size=ntfs_header->sectors_per_cluster;
+  if(mft_record_size < 42)
+  {
+    display_message("Invalid NTFS MFT record size.\n");
+    log_error("Invalid NTFS MFT record size.\n");
+    free(ntfs_header);
+    return -1;
+  }
 
-  mftmirr_size_bytes = (cluster_size <= 4 * mft_record_size ? 4 * mft_record_size : cluster_size) * ntfs_sector_size(ntfs_header);
+  cluster_size=ntfs_header->sectors_per_cluster * ntfs_sector_size(ntfs_header);
+
+  mftmirr_size_bytes = td_max(cluster_size , 4 * mft_record_size);
 #ifdef DEBUG_REPAIR_MFT
-  log_debug("mft_pos %lu\n",(unsigned long)(mft_pos/disk_car->sector_size));
-  log_debug("mftmirr_pos %lu\n",(unsigned long)(mftmirr_pos/disk_car->sector_size));
-  log_debug("cluster_size %u\n", cluster_size);
-  log_debug("mft_record_size    %u\n", mft_record_size);
-  log_debug("ntfs_sector_size   %u\n", ntfs_sector_size(ntfs_header));
-  log_debug("mftmirr_size_bytes %u\n", mftmirr_size_bytes);
+  log_info("mft_pos          %lu\n",(unsigned long)(mft_pos/disk_car->sector_size));
+  log_info("mftmirr_pos      %lu\n",(unsigned long)(mftmirr_pos/disk_car->sector_size));
+  log_info("cluster_size     %5u bytes\n", cluster_size);
+  log_info("mft_record_size  %5u bytes\n", mft_record_size);
+  log_info("ntfs_sector_size %5u bytes\n", ntfs_sector_size(ntfs_header));
+  log_info("mftmirr_size     %5u bytes\n", mftmirr_size_bytes);
 #endif
   if(mftmirr_size_bytes==0)
   {
@@ -124,9 +132,9 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
   }
   if(memcmp(buffer_mft, buffer_mftmirr, mftmirr_size_bytes)==0)
   {
-    log_info("MFT and MFT mirror matches perfectly.\n");
+    log_info("MFT and MFT mirror match perfectly.\n");
     if(*current_cmd==NULL)
-      display_message("MFT and MFT mirror matches perfectly.\n");
+      display_message("MFT and MFT mirror match perfectly.\n");
     free(buffer_mftmirr);
     free(buffer_mft);
     free(ntfs_header);
@@ -141,12 +149,13 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     free(ntfs_header);
     return -1;
   }
-/*
-  log_debug("MFT\n");
+#ifdef DEBUG_REPAIR_MFT
+  log_info("MFT\n");
   dump_log(buffer_mft, mftmirr_size_bytes);
-  log_debug("MFT mirror\n");
+  log_info("MFT mirror\n");
   dump_log(buffer_mftmirr, mftmirr_size_bytes);
-  */
+  log_flush();
+#endif
   /*
   The idea is to use the internal IO redirector built-in TestDisk
   to redirect read access to the MFT to the MFT backup instead (or
@@ -158,7 +167,7 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     dir_data_t dir_data;
     /* Use MFT */
     io_redir_add_redir(disk_car, mftmirr_pos, mftmirr_size_bytes, 0, buffer_mft);
-    res1=dir_partition_ntfs_init(disk_car,partition,&dir_data,verbose);
+    res1=dir_partition_ntfs_init(disk_car, partition, &dir_data, verbose, 0);
     if(res1==DIR_PART_ENOSYS)
     {
 	display_message("Can't determine which MFT is correct, ntfslib is missing.\n");
@@ -171,10 +180,8 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     }
     if(res1==DIR_PART_OK)
     {
-      file_info_t dir_list = {
-	.list = TD_LIST_HEAD_INIT(dir_list.list),
-	.name = NULL
-      };
+      file_info_t dir_list;
+      TD_INIT_LIST_HEAD(&dir_list.list);
       dir_data.get_dir(disk_car,partition,&dir_data,dir_data.current_inode, &dir_list);
       if(!td_list_empty(&dir_list.list))
       {
@@ -188,13 +195,11 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     io_redir_del_redir(disk_car,mftmirr_pos);
     /* Use MFT mirror */
     io_redir_add_redir(disk_car, mft_pos, mftmirr_size_bytes, 0, buffer_mftmirr);
-    res2=dir_partition_ntfs_init(disk_car,partition,&dir_data,verbose);
+    res2=dir_partition_ntfs_init(disk_car, partition, &dir_data, verbose, 0);
     if(res2==DIR_PART_OK)
     {
-      file_info_t dir_list = {
-	.list = TD_LIST_HEAD_INIT(dir_list.list),
-	.name = NULL
-      };
+      file_info_t dir_list;
+      TD_INIT_LIST_HEAD(&dir_list.list);
       dir_data.get_dir(disk_car,partition,&dir_data,dir_data.current_inode, &dir_list);
       if(!td_list_empty(&dir_list.list))
       {
@@ -221,7 +226,7 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
     {
       /* Use MFT mirror */
 #ifdef HAVE_NCURSES
-      if(ask_confirmation("Fix MFT using its mirror ? (Y/N) - DANGEROUS NON REVERSIBLE OPERATION\nUse it ONLY IF Windows failed to access this filesystem.")!=0)
+      if(ask_confirmation("Fix MFT using its mirror ? (Y/N) - DANGEROUS NON REVERSIBLE OPERATION\nUse it ONLY IF TestDisk and Windows failed to access this filesystem.")!=0)
 	use_MFT=2;
       else
 #endif
@@ -261,7 +266,7 @@ int repair_MFT(disk_t *disk_car, partition_t *partition, const int verbose, cons
 	aff_part(stdscr,AFF_PART_ORDER|AFF_PART_STATUS,disk_car,partition);
 	wmove(stdscr,8,0);
 	if(res1>0 && res2>0)
-	  wprintw(stdscr, "Both MFT seems ok but they don't match.\n");
+	  wprintw(stdscr, "Both MFT seem ok but they don't match.\n");
 	else
 	  wprintw(stdscr, "MFT and MFT mirror are bad.\n");
 	command=wmenuSelect_ext(stdscr, 23, INTER_MFT_Y, INTER_MFT_X, menuMFT, 10, "MBQ",
