@@ -19,12 +19,24 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _NTFSP_H
+#define _NTFSP_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-unsigned int ntfs_remove_used_space(disk_t *disk_car,const partition_t *partition, alloc_data_t *list_search_space);
+#if defined(HAVE_LIBNTFS) || defined(HAVE_LIBNTFS3G)
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires valid_list_search_space(list_search_space);
+  @ requires \separated(disk_car, partition, list_search_space);
+  @ ensures  valid_list_search_space(list_search_space);
+  @*/
+unsigned int ntfs_remove_used_space(disk_t *disk_car, const partition_t *partition, alloc_data_t *list_search_space);
+#endif
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
