@@ -24,6 +24,10 @@
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
+
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_NCURSES
+#endif
  
 #ifdef HAVE_NCURSES
 #include <stdio.h>
@@ -38,8 +42,7 @@
 #include <uuid.h>
 #elif defined(HAVE_UUID_UUID_H)
 #include <uuid/uuid.h>
-#endif
-#if defined(HAVE_SYS_UUID_H)
+#elif defined(HAVE_SYS_UUID_H)
 #include <sys/uuid.h>
 #endif
 #include "common.h"
