@@ -20,7 +20,7 @@
 
  */
 
-
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_HUMAX)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -43,14 +43,63 @@
 #include "log.h"
 #include "parthumax.h"
 
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @*/
+// ensures  valid_list_part(\result);
 static list_part_t *read_part_humax(disk_t *disk_car, const int verbose, const int saveheader);
+
+/*@
+  @ requires \valid_read(disk_car);
+  @ requires valid_disk(disk_car);
+  @ requires \valid(list_part);
+  @ requires separation: \separated(disk_car, list_part);
+  @*/
 static int write_part_humax(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose);
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires list_part == \null || \valid(list_part);
+  @ requires separation: \separated(disk_car, list_part);
+  @*/
 static list_part_t *init_part_order_humax(const disk_t *disk_car, list_part_t *list_part);
+
+/*@
+  @ requires \valid_read(disk_car);
+  @ requires \valid(partition);
+  @ requires separation: \separated(disk_car, partition);
+  @ assigns partition->status;
+  @*/
 static void set_next_status_humax(const disk_t *disk_car, partition_t *partition);
-static int test_structure_humax(list_part_t *list_part);
+
+/*@
+  @ requires list_part == \null || \valid_read(list_part);
+  @*/
+static int test_structure_humax(const list_part_t *list_part);
+
+/*@
+  @ requires \valid(partition);
+  @ assigns \nothing;
+  @*/
 static int is_part_known_humax(const partition_t *partition);
+
+/*@
+  @ requires \valid_read(disk_car);
+  @ requires list_part == \null || \valid(list_part);
+  @*/
 static void init_structure_humax(const disk_t *disk_car,list_part_t *list_part, const int verbose);
+
+/*@
+  @ requires \valid_read(partition);
+  @ assigns \nothing;
+  @*/
 static const char *get_partition_typename_humax(const partition_t *partition);
+
+/*@
+  @ requires \valid_read(partition);
+  @ assigns \nothing;
+  @*/
 static unsigned int get_part_type_humax(const partition_t *partition);
 
 #if 0
@@ -113,6 +162,7 @@ static list_part_t *read_part_humax(disk_t *disk_car, const int verbose, const i
   list_part_t *new_list_part=NULL;
   uint32_t *p32;
   unsigned char *buffer;
+  /*@ assert valid_list_part(new_list_part); */
   if(disk_car->sector_size < DEFAULT_SECTOR_SIZE)
     return NULL;
   buffer=(unsigned char *)MALLOC(disk_car->sector_size);
@@ -134,6 +184,9 @@ static list_part_t *read_part_humax(disk_t *disk_car, const int verbose, const i
     free(buffer);
     return NULL;
   }
+  /*@
+    @ loop invariant valid_list_part(new_list_part);
+    @*/
   for(i=0;i<4;i++)
   {
      if (humaxlabel->partitions[i].num_sectors > 0)
@@ -183,7 +236,7 @@ static list_part_t *init_part_order_humax(const disk_t *disk_car, list_part_t *l
   return list_part;
 }
 
-list_part_t *add_partition_humax_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
+list_part_t *add_partition_humax_cli(const disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   CHS_t start,end;
   partition_t *new_partition=partition_new(&arch_humax);
@@ -194,9 +247,14 @@ list_part_t *add_partition_humax_cli(disk_t *disk_car,list_part_t *list_part, ch
   end.cylinder=disk_car->geom.cylinders-1;
   end.head=disk_car->geom.heads_per_cylinder-1;
   end.sector=disk_car->geom.sectors_per_head;
+  /*@
+    @ loop invariant valid_list_part(list_part);
+    @ loop invariant valid_read_string(*current_cmd);
+    @ */
   while(1)
   {
     skip_comma_in_command(current_cmd);
+    /*@ assert valid_read_string(*current_cmd); */
     if(check_command(current_cmd,"c,",2)==0)
     {
       start.cylinder=ask_number_cli(current_cmd, start.cylinder,0,disk_car->geom.cylinders-1,"Enter the starting cylinder ");
@@ -214,19 +272,25 @@ list_part_t *add_partition_humax_cli(disk_t *disk_car,list_part_t *list_part, ch
     {
       int insert_error=0;
       list_part_t *new_list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
+      /*@ assert valid_list_part(new_list_part); */
       if(insert_error>0)
       {
 	free(new_partition);
+	/*@ assert valid_list_part(new_list_part); */
 	return new_list_part;
       }
       new_partition->status=STATUS_PRIM;
       if(test_structure_humax(list_part)!=0)
 	new_partition->status=STATUS_DELETED;
+      /*@ assert valid_read_string(*current_cmd); */
+      /*@ assert valid_list_part(new_list_part); */
       return new_list_part;
     }
     else
     {
       free(new_partition);
+      /*@ assert valid_read_string(*current_cmd); */
+      /*@ assert valid_list_part(list_part); */
       return list_part;
     }
   }
@@ -240,25 +304,17 @@ static void set_next_status_humax(const disk_t *disk_car, partition_t *partition
     partition->status=STATUS_DELETED;
 }
 
-static int test_structure_humax(list_part_t *list_part)
+static int test_structure_humax(const list_part_t *list_part)
 { /* Return 1 if bad*/
   list_part_t *new_list_part=NULL;
   int res;
   unsigned int nbr_prim=0;
-  list_part_t *element;
+  const list_part_t *element;
+  /*@ loop assigns element, nbr_prim; */
   for(element=list_part;element!=NULL;element=element->next)
   {
-    switch(element->part->status)
-    {
-      case STATUS_PRIM:
+    if(element->part->status == STATUS_PRIM)
 	nbr_prim++;
-	break;
-      case STATUS_DELETED:
-	break;
-      default:
-	log_critical("test_structure_humax: severe error\n");
-	break;
-    }
   }
   if(nbr_prim>4)
     return 1;
@@ -294,7 +350,7 @@ static void init_structure_humax(const disk_t *disk_car,list_part_t *list_part,
   }
   for(element=new_list_part;element!=NULL;element=element->next)
     element->part->status=STATUS_PRIM;
-  if(disk_car->arch->test_structure(new_list_part))
+  if(test_structure_humax(new_list_part))
   {
     for(element=new_list_part;element!=NULL;element=element->next)
       element->part->status=STATUS_DELETED;
@@ -306,3 +362,4 @@ static const char *get_partition_typename_humax(const partition_t *partition)
 {
   return "Partition";
 }
+#endif
