@@ -19,12 +19,26 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _PARTHUMAX_H
+#define _PARTHUMAX_H
 #ifdef __cplusplus
 extern "C" {
 #endif
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_HUMAX)
+/*@
+  @ requires valid_disk(disk_car);
+  @ requires \valid_read(disk_car);
+  @ requires valid_list_part(list_part);
+  @ requires \valid(current_cmd);
+  @ requires separation: \separated(disk_car, list_part, current_cmd, *current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @*/
+// ensures  valid_list_part(\result);
+// ensures  valid_read_string(*current_cmd);
+list_part_t *add_partition_humax_cli(const disk_t *disk_car,list_part_t *list_part, char **current_cmd);
 
-list_part_t *add_partition_humax_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd);
-
+#endif
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
