@@ -25,6 +25,10 @@
 #include <config.h>
 #endif
 
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_NCURSES
+#endif
+
 #ifdef HAVE_NCURSES
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
