@@ -20,7 +20,7 @@
 
  */
 
-
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_SUN)
 #ifdef HAVE_CONFIG_H
 #include <config.h>
 #endif
@@ -39,7 +39,9 @@
 #include "fnctdsk.h"
 #include "lang.h"
 #include "intrf.h"
+#ifndef DISABLED_FOR_FRAMAC
 #include "analyse.h"
+#endif
 #include "chgtype.h"
 #include "sun.h"
 #include "swap.h"
@@ -51,17 +53,80 @@
 #include "partsun.h"
 
 static int check_part_sun(disk_t *disk_car, const int verbose,partition_t *partition,const int saveheader);
+/*@
+  @ requires \valid_read(buffer + (0 .. 0x200-1));
+  @ requires \valid(geometry);
+  @ requires \separated(buffer + (0 .. 0x200-1), geometry);
+  @ ensures geometry->cylinders == 0;
+  @ assigns geometry->sectors_per_head, geometry->heads_per_cylinder, geometry->cylinders;
+  @*/
 static int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry);
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires valid_disk(disk_car);
+  @*/
+// ensures  valid_list_part(\result);
 static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int saveheader);
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires list_part == \null || \valid(list_part);
+  @ requires separation: \separated(disk_car, list_part);
+  @*/
 static int write_part_sun(disk_t *disk_car, const list_part_t *list_part, const int ro , const int verbose);
+
+/*@
+  @ requires \valid(disk_car);
+  @ requires list_part == \null || \valid(list_part);
+  @*/
 static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *list_part);
+
+/*@
+  @ requires \valid_read(disk_car);
+  @ requires \valid(partition);
+  @ assigns partition->status;
+  @*/
 static void set_next_status_sun(const disk_t *disk_car, partition_t *partition);
-static int test_structure_sun(list_part_t *list_part);
+
+/*@
+  @ requires list_part == \null || \valid_read(list_part);
+  @*/
+static int test_structure_sun(const list_part_t *list_part);
+
+/*@
+  @ requires \valid(partition);
+  @ assigns partition->part_type_sun;
+  @*/
 static int set_part_type_sun(partition_t *partition, unsigned int part_type_sun);
+
+/*@
+  @ requires \valid(partition);
+  @ assigns \nothing;
+  @*/
 static int is_part_known_sun(const partition_t *partition);
+
+/*@
+  @ requires \valid_read(disk_car);
+  @ requires list_part == \null || \valid(list_part);
+  @*/
 static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, const int verbose);
+
+/*@
+  @ requires \valid_read(partition);
+  @ assigns \nothing;
+  @*/
 static const char *get_partition_typename_sun(const partition_t *partition);
+
+/*@
+  @ assigns \nothing;
+  @*/
 static const char *get_partition_typename_sun_aux(const unsigned int part_type_sun);
+
+/*@
+  @ requires \valid_read(partition);
+  @ assigns \nothing;
+  @*/
 static unsigned int get_part_type_sun(const partition_t *partition);
 
 static const struct systypes sun_sys_types[] = {
@@ -113,18 +178,22 @@ static unsigned int get_part_type_sun(const partition_t *partition)
 static int get_geometry_from_sunmbr(const unsigned char *buffer, const int verbose, CHSgeometry_t *geometry)
 {
   const sun_disklabel *sunlabel=(const sun_disklabel*)buffer;
+#ifndef DISABLED_FOR_FRAMAC
   if(verbose>1)
   {
     log_trace("get_geometry_from_sunmbr\n");
   }
+#endif
   geometry->cylinders=0;
   geometry->heads_per_cylinder=be16(sunlabel->ntrks);
   geometry->sectors_per_head=be16(sunlabel->nsect);
+#ifndef DISABLED_FOR_FRAMAC
   if(geometry->sectors_per_head>0)
   {
     log_info("Geometry from SUN MBR: head=%u sector=%u\n",
 	geometry->heads_per_cylinder, geometry->sectors_per_head);
   }
+#endif
   return 0;
 }
 
@@ -134,6 +203,7 @@ static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int
   sun_disklabel *sunlabel;
   list_part_t *new_list_part=NULL;
   unsigned char *buffer;
+  /*@ assert valid_list_part(new_list_part); */
   if(disk_car->sector_size < DEFAULT_SECTOR_SIZE)
     return NULL;
   buffer=(unsigned char *)MALLOC(disk_car->sector_size);
@@ -151,6 +221,9 @@ static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int
     free(buffer);
     return NULL;
   }
+  /*@
+    @ loop invariant valid_list_part(new_list_part);
+    @*/
   for(i=0;i<8;i++)
   {
      if (sunlabel->partitions[i].num_sectors > 0
@@ -164,7 +237,7 @@ static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int
        new_partition->part_offset=be32(sunlabel->partitions[i].start_cylinder)*be16(sunlabel->ntrks)*be16(sunlabel->nsect)*disk_car->sector_size;
        new_partition->part_size=(uint64_t)be32(sunlabel->partitions[i].num_sectors)*disk_car->sector_size;
        new_partition->status=STATUS_PRIM;
-       disk_car->arch->check_part(disk_car,verbose,new_partition,saveheader);
+       check_part_sun(disk_car,verbose,new_partition,saveheader);
        aff_part_buffer(AFF_PART_ORDER|AFF_PART_STATUS,disk_car,new_partition);
        new_list_part=insert_new_partition(new_list_part, new_partition, 0, &insert_error);
        if(insert_error>0)
@@ -172,6 +245,7 @@ static list_part_t *read_part_sun(disk_t *disk_car, const int verbose, const int
      }
   }
   free(buffer);
+  /*@ assert valid_list_part(new_list_part); */
   return new_list_part;
 }
 
@@ -216,7 +290,7 @@ static list_part_t *init_part_order_sun(const disk_t *disk_car, list_part_t *lis
   return list_part;
 }
 
-list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd)
+list_part_t *add_partition_sun_cli(const disk_t *disk_car,list_part_t *list_part, char **current_cmd)
 {
   CHS_t start,end;
   partition_t *new_partition;
@@ -228,6 +302,10 @@ list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_part, char
   end.cylinder=disk_car->geom.cylinders-1;
   end.head=disk_car->geom.heads_per_cylinder-1;
   end.sector=disk_car->geom.sectors_per_head;
+  /*@
+    @ loop invariant valid_list_part(list_part);
+    @ loop invariant valid_read_string(*current_cmd);
+    @ */
   while(1)
   {
     skip_comma_in_command(current_cmd);
@@ -248,19 +326,23 @@ list_part_t *add_partition_sun_cli(disk_t *disk_car,list_part_t *list_part, char
     {
       int insert_error=0;
       list_part_t *new_list_part=insert_new_partition(list_part, new_partition, 0, &insert_error);
+      /*@ assert valid_list_part(new_list_part); */
       if(insert_error>0)
       {
 	free(new_partition);
+	/*@ assert valid_list_part(new_list_part); */
 	return new_list_part;
       }
       new_partition->status=STATUS_PRIM;
       if(test_structure_sun(list_part)!=0)
 	new_partition->status=STATUS_DELETED;
+      /*@ assert valid_list_part(new_list_part); */
       return new_list_part;
     }
     else
     {
       free(new_partition);
+      /*@ assert valid_list_part(list_part); */
       return list_part;
     }
   }
@@ -274,7 +356,7 @@ static void set_next_status_sun(const disk_t *disk_car, partition_t *partition)
     partition->status=STATUS_DELETED;
 }
 
-static int test_structure_sun(list_part_t *list_part)
+static int test_structure_sun(const list_part_t *list_part)
 { /* Return 1 if bad*/
   int res;
   list_part_t *new_list_part=gen_sorted_partition_list(list_part);
@@ -324,7 +406,7 @@ static void init_structure_sun(const disk_t *disk_car,list_part_t *list_part, co
   }
   for(element=new_list_part;element!=NULL;element=element->next)
     element->part->status=STATUS_PRIM;
-  if(disk_car->arch->test_structure(new_list_part))
+  if(test_structure_sun(new_list_part))
   {
     for(element=new_list_part;element!=NULL;element=element->next)
       element->part->status=STATUS_DELETED;
@@ -384,6 +466,7 @@ static int check_part_sun(disk_t *disk_car,const int verbose,partition_t *partit
 static const char *get_partition_typename_sun_aux(const unsigned int part_type_sun)
 {
   int i;
+  /*@ loop assigns i; */
   for (i=0; sun_sys_types[i].name!=NULL; i++)
     if (sun_sys_types[i].part_type == part_type_sun)
       return sun_sys_types[i].name;
@@ -394,3 +477,4 @@ static const char *get_partition_typename_sun(const partition_t *partition)
 {
   return get_partition_typename_sun_aux(partition->part_type_sun);
 }
+#endif
