@@ -19,12 +19,20 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _PARTSUNN_H
+#define _PARTSUNN_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
-list_part_t *add_partition_sun_ncurses(disk_t *disk_car,list_part_t *list_part, char **current_cmd);
+/*@
+  @ requires valid_disk(disk);
+  @ requires valid_list_part(list_part);
+  @ ensures  valid_list_part(list_part);
+  @*/
+list_part_t *add_partition_sun_ncurses(disk_t *disk, list_part_t *list_part);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
