@@ -20,9 +20,12 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _PARTXBOX_H
+#define _PARTXBOX_H
 #ifdef __cplusplus
 extern "C" {
 #endif
+#if !defined(SINGLE_PARTITION_TYPE) || defined(SINGLE_PARTITION_XBOX)
 
 struct xbox_partition
 {
@@ -31,8 +34,20 @@ struct xbox_partition
 	uint32_t bootnbr;
 } __attribute__ ((gcc_struct, __packed__));
 
-list_part_t *add_partition_xbox_cli(disk_t *disk_car,list_part_t *list_part, char **current_cmd);
+/*@
+  @ requires valid_disk(disk_car);
+  @ requires \valid_read(disk_car);
+  @ requires valid_list_part(list_part);
+  @ requires \valid(current_cmd);
+  @ requires separation: \separated(disk_car, list_part, current_cmd, *current_cmd);
+  @ requires valid_read_string(*current_cmd);
+  @*/
+// ensures  valid_list_part(\result);
+// ensures  valid_read_string(*current_cmd);
+list_part_t *add_partition_xbox_cli(const disk_t *disk_car,list_part_t *list_part, char **current_cmd);
 
+#endif
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
