@@ -19,13 +19,22 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
-
+#ifndef _PDISKSEL_H
+#define _PDISKSEL_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires valid_read_string(cmd_device);
+  @ requires valid_list_disk(list_disk);
+  @ requires \valid(list_search_space);
+  @ requires \separated(cmd_device, list_disk, list_search_space);
+  @ ensures  \result==\null || valid_disk(\result);
+  @*/
 disk_t *photorec_disk_selection_cli(const char *cmd_device, const list_disk_t *list_disk, alloc_data_t *list_search_space);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
