@@ -19,12 +19,27 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _PDISKSELN_H
+#define _PDISKSELN_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires \valid(params);
+  @ requires valid_ph_param(params);
+  @ requires \valid(options);
+  @ requires valid_list_disk(list_disk);
+  @ requires params->cmd_device==\null || valid_read_string(params->cmd_device);
+  @ requires params->cmd_run==\null || valid_read_string(params->cmd_run);
+  @ requires params->disk==\null;
+  @ requires \separated(params, options, list_disk);
+  @ ensures  params->cmd_run==\null || valid_read_string(params->cmd_run);
+  @*/
+// ensures  params->disk==\null || valid_disk(params->disk);
 int do_curses_photorec(struct ph_param *params, struct ph_options *options, const list_disk_t *list_disk);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
