@@ -54,10 +54,19 @@
 #include "file_found.h"
 
 #define READ_SIZE 1024*512
-extern const file_hint_t file_hint_tar;
-extern file_check_list_t file_check_list;
 
-static inline void file_recovery_cpy(file_recovery_t *dst, file_recovery_t *src)
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_tar)
+extern const file_hint_t file_hint_tar;
+#endif
+extern file_check_list_t file_check_list;
+extern int need_to_stop;
+
+/*@
+  @ requires \valid(dst);
+  @ requires \valid_read(src);
+  @ requires \separated(src, dst);
+  @*/
+static inline void file_recovery_cpy(file_recovery_t *dst, const file_recovery_t *src)
 {
   memcpy(dst, src, sizeof(*dst));
   dst->location.list.prev=&dst->location.list;
@@ -75,9 +84,10 @@ pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_optio
   unsigned int buffer_size;
   const unsigned int blocksize=params->blocksize;
   const unsigned int read_size=(blocksize>65536?blocksize:65536);
+  /*@ assert read_size >= 65536; */
   alloc_data_t *current_search_space;
   file_recovery_t file_recovery;
-
+#ifndef DISABLED_FOR_FRAMAC
   params->file_nbr=0;
   reset_file_recovery(&file_recovery);
   file_recovery.blocksize=blocksize;
@@ -100,11 +110,14 @@ pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_optio
     {
       file_recovery_t file_recovery_new;
       file_recovery_new.blocksize=blocksize;
+      file_recovery_new.location.start=offset;
+#if !defined(SINGLE_FORMAT) || defined(SINGLE_FORMAT_tar)
       if(file_recovery.file_stat!=NULL && file_recovery.file_stat->file_hint==&file_hint_tar &&
-          header_check_tar(buffer-0x200,0x200,0,&file_recovery,&file_recovery_new))
+          is_valid_tar_header((const struct tar_posix_header *)(buffer-0x200)))
       { /* Currently saving a tar, do not check the data for know header */
       }
       else
+#endif
       {
 	const struct td_list_head *tmpl;
         file_recovery_new.file_stat=NULL;
@@ -115,6 +128,7 @@ pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_optio
 	  td_list_for_each(tmp, &pos->file_checks[buffer[pos->offset]].list)
 	  {
 	    const file_check_t *file_check=td_list_entry_const(tmp, const file_check_t, list);
+	    /*@ assert valid_file_check_node(file_check); */
 	    if((file_check->length==0 || memcmp(buffer + file_check->offset, file_check->value, file_check->length)==0) &&
 		file_check->header_check(buffer, read_size, 1, &file_recovery, &file_recovery_new)!=0)
 	    {
@@ -202,8 +216,14 @@ pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_optio
 	}
       }
 #endif
+      if(need_to_stop!=0)
+      {
+	log_info("PhotoRec has been stopped\n");
+	current_search_space=list_search_space;
+      }
     }
   } /* end while(current_search_space!=list_search_space) */
   free(buffer_start);
+#endif
   return PSTATUS_OK;
 }
