@@ -19,11 +19,21 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _PHBS_H
+#define _PHBS_H
 #ifdef __cplusplus
 extern "C" {
 #endif
+
+/*@
+  @ requires \valid(params);
+  @ requires \valid_read(options);
+  @ requires valid_list_search_space(list_search_space);
+  @ requires \separated(params, options, list_search_space);
+  @*/
 pstatus_t photorec_find_blocksize(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space);
+
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
-
+#endif
