@@ -24,6 +24,14 @@
 #include <config.h>
 #endif
 
+#if defined(DISABLED_FOR_FRAMAC)
+#undef HAVE_FTRUNCATE
+#undef HAVE_LIBEXT2FS
+#undef HAVE_LIBNTFS
+#undef HAVE_LIBNTFS3G
+#undef ENABLE_DFXML
+#endif
+
 #include <stdio.h>
 #ifdef HAVE_STDLIB_H
 #include <stdlib.h>
@@ -41,6 +49,10 @@
 #ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#if defined(__FRAMAC__)
+#include "__fc_builtin.h"
+#endif
+#include <limits.h>
 #include "types.h"
 #include "common.h"
 #include "fnctdsk.h"
@@ -58,9 +70,22 @@
 /* #define DEBUG_FILE_FINISH */
 /* #define DEBUG_UPDATE_SEARCH_SPACE */
 /* #define DEBUG_FREE */
+uint64_t gpfh_nbr=0;
 
 static void update_search_space_aux(alloc_data_t *list_search_space, uint64_t start, uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset);
+
+/*@
+  @ requires valid_file_recovery(file_recovery);
+  @ requires valid_list_search_space(list_search_space);
+  @ requires \separated(file_recovery, list_search_space);
+  @*/
 static void file_block_truncate_zero(const file_recovery_t *file_recovery, alloc_data_t *list_search_space);
+
+/*@
+  @ requires valid_file_recovery(file_recovery);
+  @ requires valid_list_search_space(list_search_space);
+  @ requires \separated(file_recovery, list_search_space);
+  @*/
 static int file_block_truncate(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, const unsigned int blocksize);
 
 void file_block_log(const file_recovery_t *file_recovery, const unsigned int sector_size)
@@ -68,6 +93,7 @@ void file_block_log(const file_recovery_t *file_recovery, const unsigned int sec
   struct td_list_head *tmp;
   if(file_recovery->filename[0]=='\0')
     return;
+#ifndef DISABLED_FOR_FRAMAC
   log_info("%s\t",file_recovery->filename);
   td_list_for_each(tmp, &file_recovery->location.list)
   {
@@ -78,6 +104,7 @@ void file_block_log(const file_recovery_t *file_recovery, const unsigned int sec
       log_info(" (%lu-%lu)", (unsigned long)(element->start/sector_size), (unsigned long)(element->end/sector_size));
   }
   log_info("\n");
+#endif
 }
 
 void del_search_space(alloc_data_t *list_search_space, const uint64_t start, const uint64_t end)
@@ -85,6 +112,12 @@ void del_search_space(alloc_data_t *list_search_space, const uint64_t start, con
   update_search_space_aux(list_search_space, start, end, NULL, NULL);
 }
 
+/*@
+  @ requires \valid(list_search_space);
+  @ requires new_current_search_space == \null || (\valid(new_current_search_space) && \valid(*new_current_search_space));
+  @ requires offset == \null || \valid(offset);
+  @ decreases end-start;
+  @*/
 static void update_search_space_aux(alloc_data_t *list_search_space, const uint64_t start, const uint64_t end, alloc_data_t **new_current_search_space, uint64_t *offset)
 {
   struct td_list_head *search_walker = NULL;
@@ -96,10 +129,16 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
 #endif
   if(start > end)
     return ;
+  /*@
+    @ loop invariant \valid(list_search_space);
+    @ loop invariant new_current_search_space == \null || (\valid(new_current_search_space) && \valid(*new_current_search_space));
+    @ loop invariant offset == \null || \valid(offset);
+    @*/
   td_list_for_each_prev(search_walker, &list_search_space->list)
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(current_search_space); */
 #ifdef DEBUG_UPDATE_SEARCH_SPACE
     log_trace("update_search_space_aux offset=%llu remove [%llu-%llu] in [%llu-%llu]\n",
 	(long long unsigned)(offset==NULL?0:((*offset)/512)),
@@ -116,6 +155,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
         if(offset!=NULL && new_current_search_space!=NULL &&
             current_search_space->start<=*offset && *offset<=end)
         {
+	  /*@ assert \valid(new_current_search_space); */
           *new_current_search_space=current_search_space;
           *offset=end+1;
         }
@@ -128,6 +168,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
           current_search_space->start<=*offset && *offset<=current_search_space->end)
       {
         *new_current_search_space=td_list_next_entry(current_search_space, list);
+	/*@ assert \valid(*new_current_search_space); */
         *offset=(*new_current_search_space)->start;
       }
       td_list_del(search_walker);
@@ -147,6 +188,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
             start<=*offset && *offset<=current_search_space->end)
         {
 	  *new_current_search_space=td_list_next_entry(current_search_space, list);
+	  /*@ assert \valid(*new_current_search_space); */
           *offset=(*new_current_search_space)->start;
         }
         current_search_space->end=start-1;
@@ -157,6 +199,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
           current_search_space->start<=*offset && *offset<=current_search_space->end)
       {
 	*new_current_search_space=td_list_next_entry(current_search_space, list);
+	/*@ assert \valid(*new_current_search_space); */
         *offset=(*new_current_search_space)->start;
       }
       td_list_del(search_walker);
@@ -182,6 +225,7 @@ static void update_search_space_aux(alloc_data_t *list_search_space, const uint6
     {
       alloc_data_t *new_free_space;
       new_free_space=(alloc_data_t*)MALLOC(sizeof(*new_free_space));
+      /*@ assert \valid(new_free_space); */
       new_free_space->start=start;
       new_free_space->end=current_search_space->end;
       new_free_space->file_stat=NULL;
@@ -203,6 +247,7 @@ void init_search_space(alloc_data_t *list_search_space, const disk_t *disk_car,
 {
   alloc_data_t *new_sp;
   new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+  /*@ assert \valid(new_sp); */
   new_sp->start=partition->part_offset;
   new_sp->end=partition->part_offset+partition->part_size-1;
   if(new_sp->end > disk_car->disk_size-1)
@@ -220,12 +265,19 @@ void free_list_search_space(alloc_data_t *list_search_space)
 {
   struct td_list_head *search_walker = NULL;
   struct td_list_head *search_walker_next = NULL;
+  /*@
+    @ loop invariant \valid(search_walker);
+    @ loop invariant \valid(search_walker_next);
+    @ loop invariant valid_list_search_space(list_search_space);
+    @*/
   td_list_for_each_safe(search_walker,search_walker_next,&list_search_space->list)
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(current_search_space); */
     td_list_del(search_walker);
     free(current_search_space);
+    /*@ assert \valid(search_walker); */
   }
 }
 
@@ -237,74 +289,97 @@ void free_list_search_space(alloc_data_t *list_search_space)
 
 unsigned int photorec_mkdir(const char *recup_dir, const unsigned int initial_dir_num)
 {
-  char working_recup_dir[2048];
-  int dir_ok=0;
-  int dir_num=initial_dir_num;
+  unsigned int dir_num=initial_dir_num;
 #ifdef DJGPP
   int i=0;
+  /*@ assert valid_read_string(recup_dir); */
 #endif
-  do
+  /*@
+    @ loop invariant \separated(recup_dir, &errno);
+    @ loop assigns errno, dir_num;
+    @*/
+  while(1)
   {
-    snprintf(working_recup_dir,sizeof(working_recup_dir)-1,"%s.%d",recup_dir,dir_num);
+    char working_recup_dir[2048];
+    snprintf(working_recup_dir, sizeof(working_recup_dir)-1, "%s.%u", recup_dir, dir_num);
+    working_recup_dir[sizeof(working_recup_dir)-1]='\0';
+    /*@ assert valid_read_string(&working_recup_dir[0]); */
 #ifdef HAVE_MKDIR
 #ifdef __MINGW32__
-    if(mkdir(working_recup_dir)!=0 && errno==EEXIST)
+    if(mkdir(working_recup_dir)==0 || errno!=EEXIST)
+      return dir_num;
 #else
-      if(mkdir(working_recup_dir, 0775)!=0 && errno==EEXIST)
+    if(mkdir(working_recup_dir, 0775)==0 || errno!=EEXIST)
+      return dir_num;
 #endif
 #else
 #warning "You need a mkdir function!"
 #endif
-      {
-	dir_num++;
-      }
-      else
-      {
-	dir_ok=1;
-      }
+    if(dir_num == UINT_MAX)
+      dir_num=0;
+    else
+      dir_num++;
 #ifdef DJGPP
   /* Avoid endless loop in Dos version of Photorec after 999 directories if working with short name */
     i++;
-    if(dir_ok==0 && i==1000)
+    if(i==1000)
     {
       dir_num=initial_dir_num;
-      dir_ok=1;
+      return dir_num;
     }
 #endif
-  } while(dir_ok==0);
-  return dir_num;
+  }
 }
 
-int get_prev_file_header(alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
+/*@
+  @ requires \separated(list_search_space, current_search_space, offset, &gpfh_nbr);
+  @ assigns  gpfh_nbr, *current_search_space, *offset;
+  @ */
+int get_prev_file_header(const alloc_data_t *list_search_space, alloc_data_t **current_search_space, uint64_t *offset)
 {
   int nbr;
+  /*@ assert \valid(current_search_space); */
   alloc_data_t *file_space=*current_search_space;
   uint64_t size=0;
+  gpfh_nbr++;
   /* Search backward the first fragment of a file not successfully recovered
-   * Limit the search to 10 fragments or 1GB */
+   * Limit the search to 3 fragments or 200MB */
+  /*@
+    @ loop invariant \valid(current_search_space);
+    @ loop invariant \valid(offset);
+    @ loop invariant \valid_read(file_space);
+    @ loop assigns nbr, file_space, *current_search_space, *offset;
+    @ loop variant 3 - nbr;
+    @*/
   for(nbr=0; nbr<3 && size < (uint64_t)200*1024*1024; nbr++)
   {
     file_space=td_list_prev_entry(file_space, list);
+    /*@ assert \valid(file_space); */
     if(file_space==list_search_space)
       return -1;
+    /*@ assert file_space->end > file_space->start; */
     size+=file_space->end - file_space->start + 1;
     if(file_space->file_stat!=NULL)
     {
       *current_search_space=file_space;
       *offset=file_space->start;
+      /*@ assert *current_search_space!=list_search_space && *offset == (*current_search_space)->start; */
       return 0;
     }
   }
   return -1;
 }
 
-void forget(alloc_data_t *list_search_space, alloc_data_t *current_search_space)
+void forget(const alloc_data_t *list_search_space, alloc_data_t *current_search_space)
 {
   struct td_list_head *search_walker = NULL;
   struct td_list_head *prev= NULL;
   int nbr=0;
   if(current_search_space==list_search_space)
     return ;
+  /*@
+    @ loop invariant \valid(search_walker);
+    @*/
   for(search_walker=&current_search_space->list;
       search_walker!=&list_search_space->list;
       search_walker=prev)
@@ -314,6 +389,7 @@ void forget(alloc_data_t *list_search_space, alloc_data_t *current_search_space)
     {
       alloc_data_t *tmp;
       tmp=td_list_entry(search_walker, alloc_data_t, list);
+      /*@ assert \valid(tmp); */
       td_list_del(&tmp->list);
       free(tmp);
     }
@@ -324,6 +400,7 @@ void forget(alloc_data_t *list_search_space, alloc_data_t *current_search_space)
 
 unsigned int remove_used_space(disk_t *disk_car, const partition_t *partition, alloc_data_t *list_search_space)
 {
+#ifndef DISABLED_FOR_FRAMAC
   if( partition->upart_type==UP_FAT12 ||
       partition->upart_type==UP_FAT16 || 
       partition->upart_type==UP_FAT32)
@@ -334,43 +411,58 @@ unsigned int remove_used_space(disk_t *disk_car, const partition_t *partition, a
   else if(partition->upart_type==UP_NTFS)
     return ntfs_remove_used_space(disk_car, partition, list_search_space);
 #endif
-#ifdef HAVE_LIBEXT2FS
+#if defined(HAVE_LIBEXT2FS)
   else if(partition->upart_type==UP_EXT2 || partition->upart_type==UP_EXT3 || partition->upart_type==UP_EXT4)
     return ext2_remove_used_space(disk_car, partition, list_search_space);
+#endif
 #endif
   return 0;
 }
 
 void update_stats(file_stat_t *file_stats, alloc_data_t *list_search_space)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *search_walker = NULL;
   int i;
   /* Reset */
+  /*@
+    @ loop assigns i, file_stats[i].not_recovered;
+    @*/
   for(i=0;file_stats[i].file_hint!=NULL;i++)
+  {
+    /*@ assert \valid(file_stats[i]); */
     file_stats[i].not_recovered=0;
+  }
   /* Update */
   td_list_for_each(search_walker, &list_search_space->list)
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(current_search_space); */
     if(current_search_space->file_stat!=NULL)
     {
       current_search_space->file_stat->not_recovered++;
     }
   }
+#endif
 }
 
 void write_stats_log(const file_stat_t *file_stats)
 {
+#ifndef DISABLED_FOR_FRAMAC
   unsigned int file_nbr=0;
   unsigned int i;
   unsigned int nbr;
   file_stat_t *new_file_stats;
+  /*@
+    @ loop assigns i;
+    @*/
   for(i=0;file_stats[i].file_hint!=NULL;i++);
   if(i==0)
     return ;
   nbr=i;
   new_file_stats=(file_stat_t*)MALLOC(nbr*sizeof(file_stat_t));
+  /*@ assert \valid(new_file_stats); */
   memcpy(new_file_stats, file_stats, nbr*sizeof(file_stat_t));
   qsort(new_file_stats, nbr, sizeof(file_stat_t), sorfile_stat_ts);
   for(i=0;i<nbr;i++)
@@ -393,12 +485,15 @@ void write_stats_log(const file_stat_t *file_stats)
   {
     log_info("Total: %u file found\n\n",file_nbr);
   }
+#endif
 }
 
 int sorfile_stat_ts(const void *p1, const void *p2)
 {
   const file_stat_t *f1=(const file_stat_t *)p1;
   const file_stat_t *f2=(const file_stat_t *)p2;
+  /*@ assert \valid_read(f1); */
+  /*@ assert \valid_read(f2); */
   /* bigest to lowest */
   if(f1->recovered < f2->recovered)
     return 1;
@@ -417,21 +512,39 @@ partition_t *new_whole_disk(const disk_t *disk_car)
   return fake_partition;
 }
 
-unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int default_blocksize, uint64_t *offset)
+unsigned int find_blocksize(const alloc_data_t *list_search_space, const unsigned int default_blocksize, uint64_t *offset)
 {
+  /* NTFS cluster size can be 2 MB since Windows 10 Creators edition */
+  // FIXME unsigned int blocksize=2*1024*1024;
   unsigned int blocksize=128*512;
-  struct td_list_head *search_walker = NULL;
   int run_again;
   *offset=0;
   if(td_list_empty(&list_search_space->list))
     return default_blocksize;
-  *offset=(td_list_first_entry(&list_search_space->list, alloc_data_t, list))->start % blocksize;
+  {
+    const alloc_data_t *tmp=td_list_first_entry(&list_search_space->list, alloc_data_t, list);
+    /*@ assert \valid_read(tmp); */
+    *offset=tmp->start % blocksize;
+  }
+  /*@
+    @ loop invariant blocksize > 0;
+    @ loop invariant \valid(offset);
+    @ loop invariant valid_list_search_space(list_search_space);
+    @*/
   do
   {
+    const struct td_list_head *search_walker = NULL;
     run_again=0;
+    /*@
+      @ loop invariant blocksize > 0;
+      @ loop invariant \valid(offset);
+      @ loop invariant valid_list_search_space(list_search_space);
+      @ loop invariant \valid(search_walker);
+      @*/
     td_list_for_each(search_walker, &list_search_space->list)
     {
       const alloc_data_t *tmp=td_list_entry_const(search_walker, const alloc_data_t, list);
+      /*@ assert \valid_read(tmp); */
       if(tmp->file_stat!=NULL)
       {
 	if(tmp->start%blocksize!=*offset && blocksize>default_blocksize)
@@ -448,6 +561,7 @@ unsigned int find_blocksize(alloc_data_t *list_search_space, const unsigned int
 
 void update_blocksize(const unsigned int blocksize, alloc_data_t *list_search_space, const uint64_t offset)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *search_walker;
   struct td_list_head *search_walker_prev = NULL;
   log_info("blocksize=%u, offset=%u\n", blocksize, (unsigned int)(offset%blocksize));
@@ -456,16 +570,20 @@ void update_blocksize(const unsigned int blocksize, alloc_data_t *list_search_sp
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(current_search_space); */
     current_search_space->end=(current_search_space->end+1-offset%blocksize+blocksize-1)/blocksize*blocksize+offset%blocksize-1;
   }
   /* Align start of each range */
   td_list_for_each_prev_safe(search_walker,search_walker_prev,&list_search_space->list)
   {
     alloc_data_t *current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid_read(current_search_space); */
+    /*@ assert current_search_space->start >= offset; */
     const uint64_t aligned_start=(current_search_space->start-offset%blocksize+blocksize-1)/blocksize*blocksize+offset%blocksize;
     if(current_search_space->start!=aligned_start)
     {
       alloc_data_t *prev_search_space=td_list_entry(search_walker_prev, alloc_data_t, list);
+      /*@ assert \valid_read(prev_search_space); */
       if(prev_search_space->end + 1 == current_search_space->start)
       {
 	/* merge with previous block */
@@ -490,6 +608,7 @@ void update_blocksize(const unsigned int blocksize, alloc_data_t *list_search_sp
   td_list_for_each_prev_safe(search_walker, search_walker_prev, &list_search_space->list)
   {
     alloc_data_t *current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid_read(current_search_space); */
     current_search_space->end=(current_search_space->end+1-offset%blocksize)/blocksize*blocksize+offset%blocksize-1;
     if(current_search_space->start>=current_search_space->end)
     {
@@ -498,35 +617,52 @@ void update_blocksize(const unsigned int blocksize, alloc_data_t *list_search_sp
       free(current_search_space);
     }
   }
+#endif
 }
 
 uint64_t free_list_allocation_end=0;
 
-void file_block_free(alloc_list_t *list_allocation)
+/*@
+  @ requires \valid(list_allocation);
+  @*/
+static void file_block_free(alloc_list_t *list_allocation)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *tmp = NULL;
   struct td_list_head *tmp_next = NULL;
   td_list_for_each_safe(tmp,tmp_next,&list_allocation->list)
   {
     alloc_list_t *allocated_space;
     allocated_space=td_list_entry(tmp, alloc_list_t, list);
+    /*@ assert \valid(allocated_space); */
     header_ignored_cond_reset(allocated_space->start, allocated_space->end);
     free_list_allocation_end=allocated_space->end;
     td_list_del(tmp);
     free(allocated_space);
   }
+#endif
 }
-/* file_finish_aux()
-    @param file_recovery - handle!=NULL
-    @param struct ph_param *params
-*/
 
+/*@
+  @ requires \valid(file_recovery);
+  @ requires \valid(params);
+  @ requires valid_ph_param(params);
+  @ requires \valid(file_recovery->handle);
+  @ requires valid_file_recovery(file_recovery);
+  @ requires \separated(file_recovery, params, file_recovery->handle);
+  @ decreases 0;
+  @*/
 static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *params, const int paranoid)
 {
+#ifndef DISABLED_FOR_FRAMAC
+  /*@ assert valid_file_recovery(file_recovery); */
+  /*@ assert file_recovery->file_check == \null || \valid_function(file_recovery->file_check); */
   if(params->status!=STATUS_EXT2_ON_SAVE_EVERYTHING &&
       params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING &&
       file_recovery->file_stat!=NULL && file_recovery->file_check!=NULL && paranoid>0)
     { /* Check if recovered file is valid */
+      /*@ assert file_recovery->file_check != \null; */
+      /*@ assert \valid_function(file_recovery->file_check); */
       file_recovery->file_check(file_recovery);
     }
   /* FIXME: need to adapt read_size to volume size to avoid this */
@@ -537,10 +673,12 @@ static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *par
   if(file_recovery->file_stat!=NULL && file_recovery->file_size> 0 &&
       file_recovery->file_size < file_recovery->min_filesize)
   { 
+#ifndef DISABLED_FOR_FRAMAC
     log_info("%s File too small ( %llu < %llu), reject it\n",
 	file_recovery->filename,
 	(long long unsigned) file_recovery->file_size,
 	(long long unsigned) file_recovery->min_filesize);
+#endif
     file_recovery->file_size=0;
   }
   if(file_recovery->file_size==0)
@@ -550,10 +688,11 @@ static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *par
     fclose(file_recovery->handle);
     file_recovery->handle=NULL;
     /* File is zero-length; erase it */
+    /*@ assert valid_read_string((const char *)file_recovery->filename); */
     unlink(file_recovery->filename);
     return;
   }
-#ifdef HAVE_FTRUNCATE
+#if defined(HAVE_FTRUNCATE)
   fflush(file_recovery->handle);
   if(ftruncate(fileno(file_recovery->handle), file_recovery->file_size)<0)
   {
@@ -564,8 +703,13 @@ static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *par
   file_recovery->handle=NULL;
   if(file_recovery->time!=0 && file_recovery->time!=(time_t)-1)
     set_date(file_recovery->filename, file_recovery->time, file_recovery->time);
+  /*@ assert valid_file_recovery(file_recovery); */
   if(file_recovery->file_rename!=NULL)
+  {
+    /*@ assert file_recovery->file_rename != \null; */
+    /*@ assert \valid_function(file_recovery->file_rename); */
     file_recovery->file_rename(file_recovery);
+  }
   if((++params->file_nbr)%MAX_FILES_PER_DIR==0)
   {
     params->dir_num=photorec_mkdir(params->recup_dir, params->dir_num+1);
@@ -574,6 +718,7 @@ static void file_finish_aux(file_recovery_t *file_recovery, struct ph_param *par
       params->status!=STATUS_EXT2_OFF_SAVE_EVERYTHING &&
       file_recovery->file_stat!=NULL)
     file_recovery->file_stat->recovered++;
+#endif
 }
 
 /** file_finish_bf()
@@ -625,6 +770,7 @@ void file_recovery_aborted(file_recovery_t *file_recovery, struct ph_param *para
   {
     fclose(file_recovery->handle);
     file_recovery->handle=NULL;
+    /*@ assert valid_file_recovery(file_recovery); */
     /* File is zero-length; erase it */
     unlink(file_recovery->filename);
   }
@@ -657,6 +803,7 @@ pfstatus_t file_finish2(file_recovery_t *file_recovery, struct ph_param *params,
 
 void info_list_search_space(const alloc_data_t *list_search_space, const alloc_data_t *current_search_space, const unsigned int sector_size, const int keep_corrupted_file, const int verbose)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *search_walker = NULL;
   unsigned long int nbr_headers=0;
   uint64_t sectors_with_unknown_data=0;
@@ -664,6 +811,7 @@ void info_list_search_space(const alloc_data_t *list_search_space, const alloc_d
   {
     alloc_data_t *tmp;
     tmp=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(tmp); */
     if(tmp->file_stat!=NULL)
     {
       nbr_headers++;
@@ -685,6 +833,7 @@ void info_list_search_space(const alloc_data_t *list_search_space, const alloc_d
   log_info("%llu sectors contain unknown data, %lu invalid files found %s.\n",
       (long long unsigned)sectors_with_unknown_data, (long unsigned)nbr_headers,
       (keep_corrupted_file>0?"but saved":"and rejected"));
+#endif
 }
 
 void free_search_space(alloc_data_t *list_search_space)
@@ -695,6 +844,7 @@ void free_search_space(alloc_data_t *list_search_space)
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(current_search_space); */
     td_list_del(search_walker);
     free(current_search_space);
   }
@@ -720,13 +870,25 @@ void set_filename(file_recovery_t *file_recovery, struct ph_param *params)
   }
 }
 
+/*@
+  @ requires \valid(new_current_search_space);
+  @ requires \valid(list_search_space);
+  @ requires \separated(new_current_search_space, list_search_space);
+  @ assigns  *new_current_search_space;
+  @*/
 static void set_search_start_aux(alloc_data_t **new_current_search_space, alloc_data_t *list_search_space, const uint64_t offset)
 {
   struct td_list_head *search_walker = NULL;
+  /*@
+    @ loop invariant \valid(list_search_space);
+    @ loop invariant \valid(search_walker);
+    @ loop assigns search_walker, *new_current_search_space;
+    @*/
   td_list_for_each(search_walker, &list_search_space->list)
   {
     alloc_data_t *current_search_space;
     current_search_space=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(current_search_space); */
     if(current_search_space->start<=offset && offset<= current_search_space->end)
     {
       *new_current_search_space=current_search_space;
@@ -740,8 +902,9 @@ static void set_search_start_aux(alloc_data_t **new_current_search_space, alloc_
 
 uint64_t set_search_start(struct ph_param *params, alloc_data_t **new_current_search_space, alloc_data_t *list_search_space)
 {
+  /*@ assert \valid(new_current_search_space); */
   uint64_t offset=(*new_current_search_space)->start;
-  if(params->offset!=-1)
+  if(params->offset!=PH_INVALID_OFFSET)
   {
     offset=params->offset;
     set_search_start_aux(new_current_search_space, list_search_space, offset);
@@ -750,6 +913,12 @@ uint64_t set_search_start(struct ph_param *params, alloc_data_t **new_current_se
   {
     offset=0;
     skip_comma_in_command(&params->cmd_run);
+    /*@
+      @ loop invariant valid_disk(params->disk);
+      @ loop invariant \valid(params);
+      @ loop invariant valid_read_string(params->cmd_run);
+      @ loop assigns offset, params->cmd_run;
+      @*/
     while(*params->cmd_run >= '0' && *params->cmd_run <= '9')
     {
       offset=offset * 10 + (*params->cmd_run - '0');
@@ -761,16 +930,43 @@ uint64_t set_search_start(struct ph_param *params, alloc_data_t **new_current_se
   return offset;
 }
 
-void params_reset(struct ph_param *params, const struct ph_options *options)
+/*@
+  @ requires \valid(params);
+  @ requires valid_ph_param(params);
+  @ requires params->disk->sector_size > 0;
+  @ requires valid_read_string(params->recup_dir);
+  @ assigns params->file_nbr;
+  @ assigns params->status;
+  @ assigns params->real_start_time;
+  @ assigns params->dir_num;
+  @ assigns params->offset;
+  @ assigns params->blocksize;
+  @ ensures  valid_ph_param(params);
+  @ ensures  params->file_nbr == 0;
+  @ ensures  params->status == STATUS_FIND_OFFSET;
+  @ ensures  params->dir_num == 1;
+  @ ensures  params->offset == PH_INVALID_OFFSET;
+  @ ensures  params->blocksize > 0;
+  @ ensures  valid_read_string(params->recup_dir);
+  @*/
+static void params_reset_aux(struct ph_param *params)
 {
   params->file_nbr=0;
   params->status=STATUS_FIND_OFFSET;
   params->real_start_time=time(NULL);
   params->dir_num=1;
-  params->file_stats=init_file_stats(options->list_file_format);
-  params->offset=-1;
+  params->offset=PH_INVALID_OFFSET;
   if(params->blocksize==0)
     params->blocksize=params->disk->sector_size;
+  /*@ assert params->blocksize > 0; */
+}
+
+void params_reset(struct ph_param *params, const struct ph_options *options)
+{
+  /*@ assert valid_ph_param(params); */
+  params->file_stats=init_file_stats(options->list_file_format);
+  /*@ assert valid_ph_param(params); */
+  params_reset_aux(params);
 }
 
 const char *status_to_name(const photorec_status_t status)
@@ -801,7 +997,7 @@ const char *status_to_name(const photorec_status_t status)
 
 void status_inc(struct ph_param *params, const struct ph_options *options)
 {
-  params->offset=-1;
+  params->offset=PH_INVALID_OFFSET;
   switch(params->status)
   {
     case STATUS_UNFORMAT:
@@ -851,18 +1047,27 @@ list_part_t *init_list_part(disk_t *disk, const struct ph_options *options)
   list_part_t *list_part;
   partition_t *partition_wd;
   list_part=disk->arch->read_part(disk, (options!=NULL?options->verbose:0), 0);
+  /*@ assert valid_list_part(list_part); */
   partition_wd=new_whole_disk(disk);
   list_part=insert_new_partition(list_part, partition_wd, 0, &insert_error);
   if(insert_error>0)
   {
     free(partition_wd);
   }
+  /*@ assert valid_list_part(list_part); */
   return list_part;
 }
 
 /* file_block_remove_from_sp: remove block from list_search_space, update offset and new_current_search_space in consequence */
+/*@
+  @ requires \valid(tmp);
+  @ requires \valid(new_current_search_space);
+  @ requires \valid(offset);
+  @ requires \separated(tmp, new_current_search_space, offset);
+  @*/
 static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize)
 {
+#ifndef DISABLED_FOR_FRAMAC
   if(tmp->start == *offset)
   {
     tmp->start+=blocksize;
@@ -871,6 +1076,7 @@ static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t
     if(tmp->start <= tmp->end)
       return ;
     *new_current_search_space=td_list_next_entry(tmp, list);
+    /*@ assert \valid(*new_current_search_space); */
     *offset=(*new_current_search_space)->start;
     td_list_del(&tmp->list);
     free(tmp);
@@ -886,6 +1092,7 @@ static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t
   {
     alloc_data_t *new_sp;
     new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+    /*@ assert \valid(new_sp); */
     new_sp->start=*offset + blocksize;
     new_sp->end=tmp->end;
     new_sp->file_stat=NULL;
@@ -897,10 +1104,18 @@ static inline void file_block_remove_from_sp_aux(alloc_data_t *tmp, alloc_data_t
     *new_current_search_space=new_sp;
     *offset += blocksize;
   }
+#endif
 }
 
+/*@
+  @ requires \valid(list_search_space);
+  @ requires \valid(new_current_search_space);
+  @ requires \valid(offset);
+  @ requires \separated(list_search_space, new_current_search_space, offset);
+  @*/
 static inline void file_block_remove_from_sp(alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *search_walker = &(*new_current_search_space)->list;
   if(search_walker!=NULL)
   {
@@ -923,10 +1138,19 @@ static inline void file_block_remove_from_sp(alloc_data_t *list_search_space, al
     }
   }
   log_critical("file_block_remove_from_sp(list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize) failed\n");
+  log_flush();
+  exit(1);
+#endif
 }
 
+/*@
+  @ requires \valid(list);
+  @ requires \valid(list->list.prev);
+  @ requires offset + blocksize < 0xffffffffffffffff;
+  @*/
 static inline void file_block_add_to_file(alloc_list_t *list, const uint64_t offset, const uint64_t blocksize, const unsigned int data)
 {
+#ifndef DISABLED_FOR_FRAMAC
   if(!td_list_empty(&list->list))
   {
     alloc_list_t *prev=td_list_last_entry(&list->list, alloc_list_t, list);
@@ -938,11 +1162,13 @@ static inline void file_block_add_to_file(alloc_list_t *list, const uint64_t off
   }
   {
     alloc_list_t *new_list=(alloc_list_t *)MALLOC(sizeof(*new_list));
+    /*@ assert \valid(new_list); */
     new_list->start=offset;
     new_list->end=offset+blocksize-1;
     new_list->data=data;
     td_list_add_tail(&new_list->list, &list->list);
   }
+#endif
 }
 
 void file_block_append(file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset, const unsigned int blocksize, const unsigned int data)
@@ -951,15 +1177,20 @@ void file_block_append(file_recovery_t *file_recovery, alloc_data_t *list_search
   file_block_remove_from_sp(list_search_space, new_current_search_space, offset, blocksize);
 }
 
+/*@
+  @ requires \valid(list_search_space);
+  @*/
 static void file_block_truncate_aux(const uint64_t start, const uint64_t end, alloc_data_t *list_search_space)
 {
   struct td_list_head *search_walker = NULL;
   if(start >= end)
     return ;
+#ifndef DISABLED_FOR_FRAMAC
   td_list_for_each(search_walker, &list_search_space->list)
   {
     alloc_data_t *tmp;
     tmp=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(tmp); */
     if(tmp->start == end + 1 && tmp->file_stat==NULL)
     {
       tmp->start=start;
@@ -974,6 +1205,7 @@ static void file_block_truncate_aux(const uint64_t start, const uint64_t end, al
     {
       alloc_data_t *new_sp;
       new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+      /*@ assert \valid(new_sp); */
       new_sp->start=start;
       new_sp->end=end;
       new_sp->file_stat=NULL;
@@ -987,6 +1219,7 @@ static void file_block_truncate_aux(const uint64_t start, const uint64_t end, al
   {
     alloc_data_t *new_sp;
     new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+    /*@ assert \valid(new_sp); */
     new_sp->start=start;
     new_sp->end=end;
     new_sp->file_stat=NULL;
@@ -995,17 +1228,25 @@ static void file_block_truncate_aux(const uint64_t start, const uint64_t end, al
     new_sp->list.next=&new_sp->list;
     td_list_add_tail(&new_sp->list, &list_search_space->list);
   }
+#endif
 }
 
+/*@
+  @ requires \valid(list_search_space);
+  @ requires \valid(file_stat);
+  @ requires \separated(list_search_space, file_stat);
+  @*/
 static void file_block_truncate_zero_aux(const uint64_t start, const uint64_t end, alloc_data_t *list_search_space, file_stat_t *file_stat)
 {
   struct td_list_head *search_walker = NULL;
   if(start >= end)
     return ;
+#ifndef DISABLED_FOR_FRAMAC
   td_list_for_each(search_walker, &list_search_space->list)
   {
     alloc_data_t *tmp;
     tmp=td_list_entry(search_walker, alloc_data_t, list);
+    /*@ assert \valid(tmp); */
     if(tmp->start == end + 1 && tmp->file_stat==NULL)
     {
       tmp->start=start;
@@ -1016,6 +1257,7 @@ static void file_block_truncate_zero_aux(const uint64_t start, const uint64_t en
     {
       alloc_data_t *new_sp;
       new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+      /*@ assert \valid(new_sp); */
       new_sp->start=start;
       new_sp->end=end;
       new_sp->file_stat=file_stat;
@@ -1029,6 +1271,7 @@ static void file_block_truncate_zero_aux(const uint64_t start, const uint64_t en
   {
     alloc_data_t *new_sp;
     new_sp=(alloc_data_t*)MALLOC(sizeof(*new_sp));
+    /*@ assert \valid(new_sp); */
     new_sp->start=start;
     new_sp->end=end;
     new_sp->file_stat=file_stat;
@@ -1037,10 +1280,12 @@ static void file_block_truncate_zero_aux(const uint64_t start, const uint64_t en
     new_sp->list.next=&new_sp->list;
     td_list_add_tail(&new_sp->list, &list_search_space->list);
   }
+#endif
 }
 
 static void file_block_truncate_zero(const file_recovery_t *file_recovery, alloc_data_t *list_search_space)
 {
+#ifndef DISABLED_FOR_FRAMAC
   struct td_list_head *tmp;
   struct td_list_head *next;
   int first=1;
@@ -1057,6 +1302,7 @@ static void file_block_truncate_zero(const file_recovery_t *file_recovery, alloc
     td_list_del(tmp);
     free(element);
   }
+#endif
 }
 
 static int file_block_truncate(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, const unsigned int blocksize)
@@ -1065,6 +1311,7 @@ static int file_block_truncate(const file_recovery_t *file_recovery, alloc_data_
   struct td_list_head *next;
   uint64_t size=0;
   int file_truncated=0;
+#ifndef DISABLED_FOR_FRAMAC
   td_list_for_each_safe(tmp, next, &file_recovery->location.list)
   {
     alloc_list_t *element=td_list_entry(tmp, alloc_list_t, list);
@@ -1088,20 +1335,38 @@ static int file_block_truncate(const file_recovery_t *file_recovery, alloc_data_
 	size+=(element->end-element->start+1);
     }
   }
+#endif
   return file_truncated;
 }
 
+/*@
+  @ requires valid_file_recovery(file_recovery);
+  @ terminates \true;
+  @ assigns  \nothing;
+  @*/
 static uint64_t file_offset_end(const file_recovery_t *file_recovery)
 {
+  /*@ assert valid_file_recovery(file_recovery); */
   const struct td_list_head *tmp=file_recovery->location.list.prev;
   const alloc_list_t *element=td_list_entry_const(tmp, const alloc_list_t, list);
+  /*@ assert \valid_read(element); */
   return element->end;
 }
 
+/*@
+  @ requires \valid_read(file_recovery);
+  @ requires \valid(list_search_space);
+  @ requires \valid(new_current_search_space);
+  @ requires \valid(offset);
+  @ requires \separated(file_recovery, list_search_space, new_current_search_space, offset);
+  @ assigns  *new_current_search_space, *offset;
+  @*/
 static void file_block_move(const file_recovery_t *file_recovery, alloc_data_t *list_search_space, alloc_data_t **new_current_search_space, uint64_t *offset)
 {
+#ifndef DISABLED_FOR_FRAMAC
   const uint64_t end=file_offset_end(file_recovery);
   struct td_list_head *tmp;
+  /*@ loop assigns tmp; */
   td_list_for_each(tmp, &list_search_space->list)
   {
     alloc_data_t *element=td_list_entry(tmp, alloc_data_t, list);
@@ -1113,10 +1378,12 @@ static void file_block_move(const file_recovery_t *file_recovery, alloc_data_t *
     }
   }
   *new_current_search_space=list_search_space;
+#endif
 }
 
 void file_block_truncate_and_move(file_recovery_t *file_recovery, alloc_data_t *list_search_space, const unsigned int blocksize,  alloc_data_t **new_current_search_space, uint64_t *offset, unsigned char *buffer)
 {
+#ifndef DISABLED_FOR_FRAMAC
   file_block_truncate(file_recovery, list_search_space, blocksize);
   file_block_move(file_recovery, list_search_space, new_current_search_space, offset);
   if(file_recovery->offset_ok > file_recovery->file_size)
@@ -1136,6 +1403,8 @@ void file_block_truncate_and_move(file_recovery_t *file_recovery, alloc_data_t *
       if(fread(block_buffer, blocksize, 1, file_recovery->handle) != 1)
 	return ;
       file_recovery->data_check(buffer, 2*blocksize, file_recovery);
+      if(file_recovery->data_check==NULL)
+	return ;
       memcpy(buffer, block_buffer, blocksize);
     }
   }
@@ -1144,4 +1413,5 @@ void file_block_truncate_and_move(file_recovery_t *file_recovery, alloc_data_t *
     if(my_fseek(file_recovery->handle, file_recovery->file_size, SEEK_SET) < 0)
       return ;
   }
+#endif
 }
