@@ -19,12 +19,24 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _PSEARCHN_H
+#define _PSEARCHN_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires \valid(params);
+  @ requires valid_ph_param(params);
+  @ requires \valid_read(options);
+  @ requires \valid(list_search_space);
+  @ requires \separated(params, options, list_search_space);
+  @ decreases 0;
+  @ ensures  valid_ph_param(params);
+  @*/
 pstatus_t photorec_aux(struct ph_param *params, const struct ph_options *options, alloc_data_t *list_search_space);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
