@@ -19,6 +19,8 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _TDELETE_H
+#define _TDELETE_H
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -28,3 +30,4 @@ int write_clean_table(disk_t *disk_car);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
