@@ -19,6 +19,8 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _TDISKOP_H
+#define _TDISKOP_H
 #ifdef __cplusplus
 extern "C" {
 #endif
@@ -28,3 +30,4 @@ int menu_disk(disk_t *disk_car, const int verbose,int dump_ind, const int savehe
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
