@@ -19,14 +19,21 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _TLOAD_H
+#define _TLOAD_H
 #ifdef HAVE_NCURSES
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires valid_disk(disk_car);
+  @ requires valid_list_part(list_part);
+  @*/
 list_part_t *interface_load(disk_t *disk_car,list_part_t *list_part, const int verbose);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
 #endif
+#endif
