@@ -19,12 +19,19 @@
     Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.
 
  */
+#ifndef _TPARTWR_H
+#define _TPARTWR_H
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/*@
+  @ requires valid_disk(disk_car);
+  @ requires valid_list_part(list_part);
+  @*/
 int interface_write(disk_t *disk_car,list_part_t *list_part,const int can_search_deeper, const int can_ask_minmax_ext, int *no_confirm, char **current_cmd, unsigned int *menu);
 
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
+#endif
