@@ -41,8 +41,11 @@ struct vmfs_lvm
   uint64_t blocks;
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ decreases 0;
+  @*/
 int check_VMFS(disk_t *disk,partition_t *partition);
-int recover_VMFS(disk_t *disk, const struct vmfs_volume *sb, partition_t *partition, const int verbose, const int dump_ind);
+int recover_VMFS(const disk_t *disk, const struct vmfs_volume *sb, partition_t *partition, const int verbose, const int dump_ind);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
