@@ -35,11 +35,16 @@ struct wbfs_head
   uint8_t  hd_sec_sz_s;       	// sector size in this partition
   uint8_t  wbfs_sec_sz_s;     	// size of a wbfs sec
   uint8_t  padding3[2];
+#if !defined(__FRAMAC__)
   uint8_t  disc_table[0];    	// size depends on hd sector size
+#endif
 } __attribute__ ((gcc_struct, __packed__));
 
+/*@
+  @ decreases 0;
+  @*/
 int check_WBFS(disk_t *disk,partition_t *partition);
-int recover_WBFS(disk_t *disk, const struct wbfs_head *sb, partition_t *partition, const int verbose, const int dump_ind);
+int recover_WBFS(const disk_t *disk, const struct wbfs_head *sb, partition_t *partition, const int verbose, const int dump_ind);
 #ifdef __cplusplus
 } /* closing brace for extern "C" */
 #endif
