@@ -36,8 +36,27 @@
 #include "log.h"
 #include "guid_cpy.h"
 
-static int test_ZFS(disk_t *disk, const struct vdev_boot_header *sb, const partition_t *partition, const int dump_ind);
-static void set_ZFS_info(const struct vdev_boot_header *sb, partition_t *partition);
+static void set_ZFS_info(const struct vdev_boot_header *sb, partition_t *partition)
+{
+  partition->upart_type=UP_ZFS;
+  sprintf(partition->info,"ZFS %lu (Data size unknown)", (long unsigned)le64(sb->vb_version));
+}
+
+static int test_ZFS(const disk_t *disk, const struct vdev_boot_header *sb, const partition_t *partition, const int dump_ind)
+{
+  if(le64(sb->vb_magic)!=VDEV_BOOT_MAGIC)
+    return 1;
+  if(dump_ind!=0)
+  {
+    if(partition!=NULL && disk!=NULL)
+      log_info("\nZFS magic value at %u/%u/%u\n",
+          offset2cylinder(disk,partition->part_offset),
+          offset2head(disk,partition->part_offset),
+          offset2sector(disk,partition->part_offset));
+    dump_log(sb,DEFAULT_SECTOR_SIZE);
+  }
+  return 0;
+}
 
 int check_ZFS(disk_t *disk,partition_t *partition)
 {
@@ -57,13 +76,7 @@ int check_ZFS(disk_t *disk,partition_t *partition)
   return 0;
 }
 
-static void set_ZFS_info(const struct vdev_boot_header *sb, partition_t *partition)
-{
-  partition->upart_type=UP_ZFS;
-  sprintf(partition->info,"ZFS %lu (Data size unknown)", (long unsigned)le64(sb->vb_version));
-}
-
-int recover_ZFS(disk_t *disk, const struct vdev_boot_header *sb,partition_t *partition,const int verbose, const int dump_ind)
+int recover_ZFS(const disk_t *disk, const struct vdev_boot_header *sb,partition_t *partition,const int verbose, const int dump_ind)
 {
   if(test_ZFS(disk, sb, partition, dump_ind)!=0)
     return 1;
@@ -84,19 +97,3 @@ int recover_ZFS(disk_t *disk, const struct vdev_boot_header *sb,partition_t *par
   }
   return 0;
 }
-
-static int test_ZFS(disk_t *disk, const struct vdev_boot_header *sb, const partition_t *partition, const int dump_ind)
-{
-  if(le64(sb->vb_magic)!=VDEV_BOOT_MAGIC)
-    return 1;
-  if(dump_ind!=0)
-  {
-    if(partition!=NULL && disk!=NULL)
-      log_info("\nZFS magic value at %u/%u/%u\n",
-          offset2cylinder(disk,partition->part_offset),
-          offset2head(disk,partition->part_offset),
-          offset2sector(disk,partition->part_offset));
-    dump_log(sb,DEFAULT_SECTOR_SIZE);
-  }
-  return 0;
-}
